#include <typeinfo>
#include <Looper.h>
#include <stdlib.h>
#include <stdio.h>
#include <Window.h>
#include <Alert.h>
#include "BeComponents.h"
#include "BePropEditors.h"

// BArchivable ----------------------------------------------------------------------

TBCL_BArchivable::TBCL_BArchivable(instantiation_info_t *info)
	:	TBerryComponentLibrary(info)
{
}

TBCL_BArchivable::~TBCL_BArchivable()
{
}

BArchivable *TBCL_BArchivable::Instantiate()
{
	return new BArchivable();
}

const char *TBCL_BArchivable::SupportedClass() const
{
	return typeid(BArchivable).name();
}

void TBCL_BArchivable::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	TBerryComponentLibrary::PropertyList(propertyRegistrar);
}

status_t TBCL_BArchivable::SetValue(BArchivable *object, const char *propName, const char *value)
{
	return B_ERROR;
}

status_t TBCL_BArchivable::GetValue(BArchivable *object, const char *propName, BString *value)
{
	return B_ERROR;
}

// BHandler ----------------------------------------------------------------------

TBCL_BHandler::TBCL_BHandler(instantiation_info_t *info)
	:	TBCL_BArchivable(info)
{
}

TBCL_BHandler::~TBCL_BHandler()
{
}

BArchivable *TBCL_BHandler::Instantiate()
{
	return new BHandler();
}

const char *TBCL_BHandler::SupportedClass() const
{
	return typeid(BHandler).name();
}

void TBCL_BHandler::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	propertyRegistrar->AddProperty(this, "Name", B_STRING_TYPE);
	TBCL_BArchivable::PropertyList(propertyRegistrar);
}

status_t TBCL_BHandler::SetValue(BArchivable *object, const char *propName, const char *value)
{
	_CAST(BHandler, object, handler);
	status_t status = B_OK;
	
	if (strcmp(propName, "Name")==0) {
		status = fSetName(handler, value);
	} else {
		status = TBCL_BArchivable::SetValue(object, propName, value);
	}
	
	return status;
}

status_t TBCL_BHandler::GetValue(BArchivable *object, const char *propName, BString *value)
{
	_CAST(BHandler, object, handler);
	
	if (strcmp(propName, "Name")==0) {
		value->SetTo(handler->Name());
	} else {
		TBCL_BArchivable::GetValue(object, propName, value);
	}
	return B_OK;
}

status_t TBCL_BHandler::fSetName(BHandler *object, const char *name)
{
	if (strlen(name) == 0) return B_ERROR;
	BWindow *window = (BWindow *)object->Looper();
	if (window) {
		if (window->FindView(name)) {
			BString errMsg;
			errMsg << "WARNING!\nThe name \"" << name << "\" has already been used.\n";
			if ((new BAlert("", errMsg.String(), "OK", "Cancel"))->Go() == 1) return B_OK;
		}
	}
	object->SetName(name);
	return B_OK;
}

const char *TBCL_BHandler::fGetName(BHandler *object)
{
	return object->Name();
}

// BView -----------------------------------------------------------------------------


TBCL_BView::TBCL_BView(instantiation_info_t *info)
	:	TBCL_BHandler(info)
{
	SetControlStyle(CS_ACCEPTS_CONTROLS | ControlStyle());
}

TBCL_BView::~TBCL_BView()
{
}

BArchivable *TBCL_BView::Instantiate()
{
	return new BView(BRect(0,0,0,0), "", B_FOLLOW_NONE, B_WILL_DRAW);
}

const char *TBCL_BView::SupportedClass() const
{
	return typeid(BView).name();
}

void TBCL_BView::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	propertyRegistrar->AddProperty(this, "Left", B_FLOAT_TYPE);
	propertyRegistrar->AddProperty(this, "Top", B_FLOAT_TYPE);
	propertyRegistrar->AddProperty(this, "Width", B_FLOAT_TYPE);
	propertyRegistrar->AddProperty(this, "Height", B_FLOAT_TYPE);
	propertyRegistrar->AddProperty(this, "Font", B_ANY_TYPE, PROPEDITOR(TFontEditor));
//	propertyRegistrar->AddProperty(this, "ResizingMode", B_INT32_TYPE);
	propertyRegistrar->AddProperty(this, "FollowH", B_INT32_TYPE, NULL, &H_RESIZING_VALUES);
	propertyRegistrar->AddProperty(this, "FollowV", B_INT32_TYPE, NULL, &V_RESIZING_VALUES);
	propertyRegistrar->AddProperty(this, "Flags", B_INT32_TYPE, NULL, &BVIEW_FLAG_VALUES);
	propertyRegistrar->AddProperty(this, "ViewColor", B_INT32_TYPE, PROPEDITOR(TColorEditor));
	propertyRegistrar->AddProperty(this, "HighColor", B_INT32_TYPE, PROPEDITOR(TColorEditor));
	propertyRegistrar->AddProperty(this, "LowColor", B_INT32_TYPE, PROPEDITOR(TColorEditor));
	TBCL_BHandler::PropertyList(propertyRegistrar);
}

status_t TBCL_BView::SetValue(BArchivable *object, const char *propName, const char *value)
{
	_CAST(BView, object, view);
	status_t status = B_OK;
	
	if (strcmp(propName, "Left")==0) {
		view->MoveBy(atof(value) - view->Frame().left, 0);
	} else if (strcmp(propName, "Top")==0) {
		view->MoveBy(0, atof(value) - view->Frame().top);
	} else if (strcmp(propName, "Width")==0) {
		view->ResizeBy(atof(value) - view->Frame().Width(), 0);
	} else if (strcmp(propName, "Height")==0) {
		view->ResizeBy(0, atof(value) - view->Frame().Height());
	} else if (strcmp(propName, "Font")==0) {
		status = fSetFont(view, value);
	} else if (strcmp(propName, "ResizingMode")==0) {
		status = fSetResizingMode(view, value);
	} else if (strcmp(propName, "FollowH")==0) {
		status = fSetFollowH(view, value);
	} else if (strcmp(propName, "FollowV")==0) {
		status = fSetFollowV(view, value);
	} else if (strcmp(propName, "Flags")==0) {
		status = fSetFlags(view, value);
	} else if (strcmp(propName, "ViewColor")==0) {
		rgb_color rgb;
		status = get_rgb_color(value, &rgb);
		if (status == B_OK) view->SetViewColor(rgb);
	} else if (strcmp(propName, "HighColor")==0) {
		rgb_color rgb;
		status = get_rgb_color(value, &rgb);
		if (status == B_OK) view->SetHighColor(rgb);
	} else if (strcmp(propName, "LowColor")==0) {
		rgb_color rgb;
		status = get_rgb_color(value, &rgb);
		if (status == B_OK) view->SetLowColor(rgb);
	} else {
		status = TBCL_BHandler::SetValue(object, propName, value);
	}
	
	return status;
}

status_t TBCL_BView::GetValue(BArchivable *object, const char *propName, BString *value)
{
	_CAST(BView, object, view);
	
	if (strcmp(propName, "Left")==0) {
		get_double_value(view->Frame().left, value);
	} else if (strcmp(propName, "Top")==0) {
		get_double_value(view->Frame().top, value);
	} else if (strcmp(propName, "Width")==0) {
		get_double_value(view->Frame().Width(), value);
	} else if (strcmp(propName, "Height")==0) {
		get_double_value(view->Frame().Height(), value);
	} else if (strcmp(propName, "Font")==0) {
		BFont font;
		view->GetFont(&font);
		fGetFont(&font, value);
	} else if (strcmp(propName, "ResizingMode")==0) {
		fGetResizingMode(view->ResizingMode(), value);
	} else if (strcmp(propName, "FollowH")==0) {
		fGetFollowH(view->ResizingMode(), value);
	} else if (strcmp(propName, "FollowV")==0) {
		fGetFollowV(view->ResizingMode(), value);
	} else if (strcmp(propName, "Flags")==0) {
		fGetFlags(view->Flags(), value);
	} else if (strcmp(propName, "ViewColor")==0) {
		get_rgb_color_value(view->ViewColor(), value);
	} else if (strcmp(propName, "HighColor")==0) {
		get_rgb_color_value(view->HighColor(), value);
	} else if (strcmp(propName, "LowColor")==0) {
		get_rgb_color_value(view->LowColor(), value);
	} else {
		TBCL_BHandler::GetValue(object, propName, value);
	}
	
	return B_OK;
}

status_t TBCL_BView::GetDefaultSize(float *width, float *height)
{
	*width = 100;
	*height = 50;
	return B_OK;
}

void TBCL_BView::fGetResizingMode(uint32 resizingMode, BString *value)
{
	BString comma("");
	if (resizingMode==0) {
		value->SetTo("B_FOLLOW_NONE");
		return;
	}
	if ((resizingMode & B_FOLLOW_ALL_SIDES) == B_FOLLOW_ALL_SIDES) {
		value->SetTo("B_FOLLOW_ALL_SIDES");
	} else {
		if ((resizingMode & B_FOLLOW_H_CENTER) == B_FOLLOW_H_CENTER) {
			value->SetTo("B_FOLLOW_H_CENTER");
		} else if ((resizingMode & B_FOLLOW_LEFT_RIGHT) == B_FOLLOW_LEFT_RIGHT) {
			value->SetTo("B_FOLLOW_LEFT_RIGHT");
		} else if ((resizingMode & B_FOLLOW_LEFT) == B_FOLLOW_LEFT) {
			value->SetTo("B_FOLLOW_LEFT");
		} else if ((resizingMode & B_FOLLOW_RIGHT) == B_FOLLOW_RIGHT) {
			value->SetTo("B_FOLLOW_RIGHT");
		}
		if (value->Length()>0) comma = " | ";
		if ((resizingMode & B_FOLLOW_V_CENTER) == B_FOLLOW_V_CENTER) {
			*value << comma << "B_FOLLOW_V_CENTER";
		} else if ((resizingMode & B_FOLLOW_TOP_BOTTOM) == B_FOLLOW_TOP_BOTTOM) {
			*value << comma << "B_FOLLOW_TOP_BOTTOM";
		} else if ((resizingMode & B_FOLLOW_TOP) == B_FOLLOW_TOP) {
			*value << comma << "B_FOLLOW_TOP";
		} else if ((resizingMode & B_FOLLOW_BOTTOM) == B_FOLLOW_BOTTOM) {
			*value << comma << "B_FOLLOW_BOTTOM";
		}
	}
}

status_t TBCL_BView::fSetResizingMode(BView *view, const char *value)
{
	uint32 mode = 0;
	TStrTok strTok(value, "|", 0);
	for(int i=0; i < strTok.CountTokens(); i++) {
		TStringEx str(strTok.Token(i));
		str.Trim();
		if (str.Length()<=0) continue;
		if (str.Compare("B_FOLLOW_LEFT")==0) {
			mode |= B_FOLLOW_LEFT;
		} else if (str.Compare("B_FOLLOW_RIGHT")==0) {
			mode |= B_FOLLOW_RIGHT;
		} else if (str.Compare("B_FOLLOW_LEFT_RIGHT")==0) {
			mode |= B_FOLLOW_LEFT_RIGHT;
		} else if (str.Compare("B_FOLLOW_H_CENTER")==0) {
			mode |= B_FOLLOW_H_CENTER;
		} else if (str.Compare("B_FOLLOW_TOP")==0) {
			mode |= B_FOLLOW_TOP;
		} else if (str.Compare("B_FOLLOW_BOTTOM")==0) {
			mode |= B_FOLLOW_BOTTOM;
		} else if (str.Compare("B_FOLLOW_TOP_BOTTOM")==0) {
			mode |= B_FOLLOW_TOP_BOTTOM;
		} else if (str.Compare("B_FOLLOW_V_CENTER")==0) {
			mode |= B_FOLLOW_V_CENTER;
		} else if (str.Compare("B_FOLLOW_ALL_SIDES")==0) {
			mode |= B_FOLLOW_ALL_SIDES;
		} else if (str.Compare("B_FOLLOW_NONE")==0) {
			mode |= B_FOLLOW_NONE;
		} else {
			return B_ERROR;
		}
	}
	view->SetResizingMode(mode);
	return B_OK;
}

void TBCL_BView::fGetFollowH(uint32 resizingMode, BString *value)
{
	if ((resizingMode & B_FOLLOW_ALL_SIDES) == B_FOLLOW_ALL_SIDES) {
		value->SetTo("Left&Right");
	} else {
		if ((resizingMode & B_FOLLOW_H_CENTER) == B_FOLLOW_H_CENTER) {
			value->SetTo("HCenter");
		} else if ((resizingMode & B_FOLLOW_LEFT_RIGHT) == B_FOLLOW_LEFT_RIGHT) {
			value->SetTo("Left&Right");
		} else if ((resizingMode & B_FOLLOW_LEFT) == B_FOLLOW_LEFT) {
			value->SetTo("Left");
		} else if ((resizingMode & B_FOLLOW_RIGHT) == B_FOLLOW_RIGHT) {
			value->SetTo("Right");
		} else value->SetTo("None");
	}
}

status_t TBCL_BView::fSetFollowH(BView *view, const char *value)
{
	uint32 mode = view->ResizingMode();
	mode &= ~B_FOLLOW_LEFT_RIGHT;
	mode &= ~B_FOLLOW_H_CENTER;
	TStringEx str(value);
	str.Trim();
	if (str.ICompare("None") == 0) {
		
	} else if (str.ICompare("Left") == 0) {
		mode |= B_FOLLOW_LEFT;
	} else if (str.ICompare("Right") == 0) {
		mode |= B_FOLLOW_RIGHT;
	} else if (str.ICompare("Left&Right") == 0) {
		mode |= B_FOLLOW_LEFT_RIGHT;
	} else if (str.ICompare("HCenter") == 0) {
		mode |= B_FOLLOW_H_CENTER;
	} else {
		return B_ERROR;
	}
	view->SetResizingMode(mode);
	return B_OK;
}

void TBCL_BView::fGetFollowV(uint32 resizingMode, BString *value)
{
	if ((resizingMode & B_FOLLOW_ALL_SIDES) == B_FOLLOW_ALL_SIDES) {
		value->SetTo("Top&Bottom");
	} else {
		if ((resizingMode & B_FOLLOW_V_CENTER) == B_FOLLOW_V_CENTER) {
			value->SetTo("VCenter");
		} else if ((resizingMode & B_FOLLOW_TOP_BOTTOM) == B_FOLLOW_TOP_BOTTOM) {
			value->SetTo("Top&Bottom");
		} else if ((resizingMode & B_FOLLOW_TOP) == B_FOLLOW_TOP) {
			value->SetTo("Top");
		} else if ((resizingMode & B_FOLLOW_BOTTOM) == B_FOLLOW_BOTTOM) {
			value->SetTo("Bottom");
		} else value->SetTo("None");
	}
}

status_t TBCL_BView::fSetFollowV(BView *view, const char *value)
{
	uint32 mode = view->ResizingMode();
	mode &= ~B_FOLLOW_TOP_BOTTOM;
	mode &= ~B_FOLLOW_V_CENTER;
	TStringEx str(value);
	str.Trim();
	if (str.ICompare("None") == 0) {
		
	} else if (str.ICompare("Top") == 0) {
		mode |= B_FOLLOW_TOP;
	} else if (str.ICompare("Bottom") == 0) {
		mode |= B_FOLLOW_BOTTOM;
	} else if (str.ICompare("Top&Bottom") == 0) {
		mode |= B_FOLLOW_TOP_BOTTOM;
	} else if (str.ICompare("VCenter") == 0) {
		mode |= B_FOLLOW_V_CENTER;
	} else {
		return B_ERROR;
	}
	view->SetResizingMode(mode);
	return B_OK;
}

void TBCL_BView::fGetFlags(uint32 flags, BString *value)
{
	if (flags==0) {
		value->SetTo("0");
		return;
	}
	if (flags & B_WILL_DRAW)             {*value << (value->Length() > 0 ? " | " : "") << "B_WILL_DRAW";}
	if (flags & B_FULL_UPDATE_ON_RESIZE) {*value << (value->Length() > 0 ? " | " : "") << "B_FULL_UPDATE_ON_RESIZE";}
	if (flags & B_FRAME_EVENTS)          {*value << (value->Length() > 0 ? " | " : "") << "B_FRAME_EVENTS";}
	if (flags & B_PULSE_NEEDED)          {*value << (value->Length() > 0 ? " | " : "") << "B_PULSE_NEEDED";}
	if (flags & B_NAVIGABLE)             {*value << (value->Length() > 0 ? " | " : "") << "B_NAVIGABLE";}
	if (flags & B_NAVIGABLE_JUMP)        {*value << (value->Length() > 0 ? " | " : "") << "B_NAVIGABLE_JUMP";}
	if (flags & B_SUBPIXEL_PRECISE)      {*value << (value->Length() > 0 ? " | " : "") << "B_SUBPIXEL_PRECISE";}
}

status_t TBCL_BView::fSetFlags(BView *view, const char *value)
{
	uint32 flags = 0;
	TStrTok strTok(value, "|", 0);
	for(int i=0; i < strTok.CountTokens(); i++) {
		TStringEx str(strTok.Token(i));
		str.Trim();
		if (str.Length()<=0) continue;
		if (str.Compare("B_WILL_DRAW")==0) {
			flags |= B_WILL_DRAW;
		} else if (str.Compare("B_FULL_UPDATE_ON_RESIZE")==0) {
			flags |= B_FULL_UPDATE_ON_RESIZE;
		} else if (str.Compare("B_FRAME_EVENTS")==0) {
			flags |= B_FRAME_EVENTS;
		} else if (str.Compare("B_PULSE_NEEDED")==0) {
			flags |= B_PULSE_NEEDED;
		} else if (str.Compare("B_NAVIGABLE")==0) {
			flags |= B_NAVIGABLE;
		} else if (str.Compare("B_NAVIGABLE_JUMP")==0) {
			flags |= B_NAVIGABLE_JUMP;
		} else if (str.Compare("B_SUBPIXEL_PRECISE")==0) {
			flags |= B_SUBPIXEL_PRECISE;
		} else {
			return B_ERROR;
		}
	}
	view->SetFlags(flags);
	return B_OK;
}

void TBCL_BView::fGetFont(BFont *font, BString *value)
{
	font_family family;
	font_style style;
	font->GetFamilyAndStyle(&family, &style);
	*value << family << "," << style << "," << font->Size();
}

status_t TBCL_BView::fSetFont(BView *view, const char *value)
{
	TStrTok strTok(value, ",", 0);
	if (strTok.CountTokens() == 3) {
		TStringEx family(strTok.Token(0));
		TStringEx style(strTok.Token(1));
		TStringEx size(strTok.Token(2));
		family.Trim();
		style.Trim();
		size.Trim();
		BFont font;
		font.SetFamilyAndStyle(family.String(), style.String());
		font.SetSize(atof(size.String()));
		view->SetFont(&font, B_FONT_FAMILY_AND_STYLE | B_FONT_SIZE);
		BControl *control = dynamic_cast<BControl *>(view);
		if (control != NULL) {
			BString label(control->Label());
			control->SetLabel("");
			control->SetLabel(label.String());
		}
		return B_OK;
	} else {
		return B_ERROR;
	}
}

// BControl --------------------------------------------------------------------------

TBCL_BControl::TBCL_BControl(instantiation_info_t *info)
	:	TBCL_BView(info)
{
	SetControlStyle(!CS_ACCEPTS_CONTROLS & ControlStyle());
}

TBCL_BControl::~TBCL_BControl()
{
}

BArchivable *TBCL_BControl::Instantiate()
{
	return new BControl(BRect(0,0,0,0), "", "", NULL, B_FOLLOW_NONE, B_WILL_DRAW);
}

void TBCL_BControl::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	propertyRegistrar->AddProperty(this, "Label", B_STRING_TYPE);
	propertyRegistrar->AddProperty(this, "Value", B_INT32_TYPE);
	propertyRegistrar->AddProperty(this, "Enabled", B_BOOL_TYPE, NULL, &BOOLEAN_VALUES);
	TBCL_BView::PropertyList(propertyRegistrar);
}

const char *TBCL_BControl::SupportedClass() const
{
	return typeid(BControl).name();
}

status_t TBCL_BControl::SetValue(BArchivable *object, const char *propName, const char *value)
{
	_CAST(BControl, object, control);
	status_t status = B_OK;
	
	if (strcmp(propName, "Enabled")==0) {
		bool enabled;
		status = get_bool(value, &enabled);
		if (status == B_OK) control->SetEnabled(enabled);
	} else if (strcmp(propName, "Label")==0) {
		control->SetLabel(value);
	} else if (strcmp(propName, "Value")==0) {
		control->SetValue(atoi(value));
	} else {
		status = TBCL_BView::SetValue(object, propName, value);
	}
	
	return status;
}

status_t TBCL_BControl::GetValue(BArchivable *object, const char *propName, BString *value)
{
	_CAST(BControl, object, control);
	
	if (strcmp(propName, "Enabled")==0) {
		value->SetTo(control->IsEnabled() ? "True" : "False");
	} else if (strcmp(propName, "Label")==0) {
		value->SetTo(control->Label());
	} else if (strcmp(propName, "Value")==0) {
		get_int_value(control->Value(), value);
	} else {
		TBCL_BView::GetValue(object, propName, value);
	}
	return B_OK;
}

status_t TBCL_BControl::GetDefaultSize(float *width, float *height)
{
	*width = 100;
	*height = 50;
	return B_OK;
}


// BButton -----------------------------------------------------------------------------

TBCL_BButton::TBCL_BButton(instantiation_info_t *info)
	:	TBCL_BControl(info)
{
}

TBCL_BButton::~TBCL_BButton()
{
}

BArchivable *TBCL_BButton::Instantiate()
{
	return new BButton(BRect(0,0,0,0), "", "", NULL);
}

const char *TBCL_BButton::SupportedClass() const
{
	return typeid(BButton).name();
}

void TBCL_BButton::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	propertyRegistrar->AddProperty(this, "Default", B_BOOL_TYPE, NULL, &BOOLEAN_VALUES);
	TBCL_BControl::PropertyList(propertyRegistrar);
}

status_t TBCL_BButton::SetValue(BArchivable *object, const char *propName, const char *value)
{
	_CAST(BButton, object, button);
	status_t status = B_ERROR;
	
	if (strcmp(propName, "Default")==0) {
		bool _default;
		status = get_bool(value, &_default);
		if (status == B_OK) button->MakeDefault(_default);
	} else {
		status = TBCL_BControl::SetValue(object, propName, value);
	}
	
	return status;
}

status_t TBCL_BButton::GetValue(BArchivable *object, const char *propName, BString *value)
{
	_CAST(BButton, object, button);
	
	if (strcmp(propName, "Default")==0) {
		value->SetTo(button->IsDefault() ? "True" : "False");
	} else {
		TBCL_BControl::GetValue(object, propName, value);
	}
	return B_OK;
}

status_t TBCL_BButton::GetDefaultSize(float *width, float *height)
{
	*width = 80;
	*height = 30;
	return B_OK;
}


// BMenu ---------------------------------------------------------------------------------

TBCL_BMenu::TBCL_BMenu(instantiation_info_t *info)
	:	TBCL_BView(info)
{
	SetControlStyle(!CS_ACCEPTS_CONTROLS & ControlStyle());
}

TBCL_BMenu::~TBCL_BMenu()
{
}

BArchivable *TBCL_BMenu::Instantiate()
{
	return new BMenu("New BMenu");
}

const char *TBCL_BMenu::SupportedClass() const
{
	return typeid(BMenu).name();
}

void TBCL_BMenu::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	propertyRegistrar->AddProperty(this, "Enabled", B_BOOL_TYPE, NULL, &BOOLEAN_VALUES);
	propertyRegistrar->AddProperty(this, "RadioMode", B_BOOL_TYPE, NULL, &BOOLEAN_VALUES);
	propertyRegistrar->AddProperty(this, "Trigger", B_BOOL_TYPE, NULL, &BOOLEAN_VALUES);
	propertyRegistrar->AddProperty(this, "MaxContentWidth", B_FLOAT_TYPE);
	propertyRegistrar->AddProperty(this, "Items", B_RAW_TYPE, PROPEDITOR(TMenuEditor));
	propertyRegistrar->AddProperty(this, "Label", B_MESSAGE_TYPE);
	TBCL_BView::PropertyList(propertyRegistrar);
}

status_t TBCL_BMenu::SetValue(BArchivable *object, const char *propName, const char *value)
{
	_CAST(BMenu, object, menu);
	status_t status = B_OK;
	
	if (strcmp(propName, "Enabled")==0) {
		bool enabled;
		status = get_bool(value, &enabled);
		if (status == B_OK) menu->SetEnabled(enabled);
	} else if (strcmp(propName, "RadioMode")==0) {
		bool radioMode;
		status = get_bool(value, &radioMode);
		if (status == B_OK) menu->SetRadioMode(radioMode);
	} else if (strcmp(propName, "Trigger")==0) {
		bool trigger;
		status = get_bool(value, &trigger);
		if (status == B_OK) menu->SetTriggersEnabled(trigger);
	} else if (strcmp(propName, "MaxContentWidth")==0) {
		menu->SetMaxContentWidth(atof(value));
	} else if (strcmp(propName, "Label")==0) {
		BMenuItem *superItem = menu->Superitem();
		if (superItem!=NULL) {
			superItem->SetLabel(value);
		}
		status = B_OK;
	} else {
		status = TBCL_BView::SetValue(object, propName, value);
	}
	
	return status;
}

status_t TBCL_BMenu::GetValue(BArchivable *object, const char *propName, BString *value)
{
	_CAST(BMenu, object, menu);
	
	if (strcmp(propName, "Enabled")==0) {
		value->SetTo(menu->IsEnabled() ? "True" : "False");
	} else if (strcmp(propName, "RadioMode")==0) {
		value->SetTo(menu->IsRadioMode() ? "True" : "False");
	} else if (strcmp(propName, "Trigger")==0) {
		value->SetTo(menu->AreTriggersEnabled() ? "True" : "False");
	} else if (strcmp(propName, "MaxContentWidth")==0) {
		get_double_value(menu->MaxContentWidth(), value);
	} else if (strcmp(propName, "Items")==0) {
		value->SetTo("(Menu)");
	} else if (strcmp(propName, "Label")==0) {
		BMenuItem *superItem = menu->Superitem();
		if (superItem!=NULL) {
			value->SetTo(superItem->Label());
		} else {
			value->SetTo("(nil)");
		}
	} else {
		TBCL_BView::GetValue(object, propName, value);
	}
	
	return B_OK;
}

status_t TBCL_BMenu::GetDefaultSize(float *width, float *height)
{
	*width = 100;
	*height = 100;
	return B_OK;
}


// BMenuItem -----------------------------------------------------------------------------

TBCL_BMenuItem::TBCL_BMenuItem(instantiation_info_t *info)
	:	TBCL_BArchivable(info)
{
}

TBCL_BMenuItem::~TBCL_BMenuItem()
{
}

BArchivable *TBCL_BMenuItem::Instantiate()
{
	return new BMenuItem("New BMenuItem", NULL);
}

const char *TBCL_BMenuItem::SupportedClass() const
{
	return typeid(BMenuItem).name();
}

void TBCL_BMenuItem::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	propertyRegistrar->AddProperty(this, "Enabled", B_BOOL_TYPE, NULL, &BOOLEAN_VALUES);
	propertyRegistrar->AddProperty(this, "Label", B_STRING_TYPE);
	TBCL_BArchivable::PropertyList(propertyRegistrar);
}

status_t TBCL_BMenuItem::SetValue(BArchivable *object, const char *propName, const char *value)
{
	_CAST(BMenuItem, object, menuItem);
	status_t status = B_ERROR;
	
	if (strcmp(propName, "Enabled")==0) {
		bool enabled;
		status = get_bool(value, &enabled);
		if (status == B_OK) menuItem->SetEnabled(enabled);
	} else if (strcmp(propName, "Label")==0) {
		menuItem->SetLabel(value);
		status = B_OK;
	} else {
		status = TBCL_BArchivable::SetValue(object, propName, value);
	}
	
	return status;
}

status_t TBCL_BMenuItem::GetValue(BArchivable *object, const char *propName, BString *value)
{
	_CAST(BMenuItem, object, menuItem);
	
	if (strcmp(propName, "Enabled")==0) {
		value->SetTo(menuItem->IsEnabled() ? "True" : "False");
	} else if (strcmp(propName, "Label")==0) {
		value->SetTo(menuItem->Label());
	} else {
		TBCL_BArchivable::GetValue(object, propName, value);
	}
	return B_OK;
}


// BSeparatorItem -----------------------------------------------------------------------------

TBCL_BSeparatorItem::TBCL_BSeparatorItem(instantiation_info_t *info)
	:	TBCL_BMenuItem(info)
{
}

TBCL_BSeparatorItem::~TBCL_BSeparatorItem()
{
}

BArchivable *TBCL_BSeparatorItem::Instantiate()
{
	return new BSeparatorItem();
}

const char *TBCL_BSeparatorItem::SupportedClass() const
{
	return typeid(BSeparatorItem).name();
}

void TBCL_BSeparatorItem::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	TBCL_BMenuItem::PropertyList(propertyRegistrar);
}

status_t TBCL_BSeparatorItem::SetValue(BArchivable *object, const char *propName, const char *value)
{
	if (strcmp(propName, "Label")==0) return B_ERROR;
	status_t status = TBCL_BMenuItem::SetValue(object, propName, value);
	return status;
}

status_t TBCL_BSeparatorItem::GetValue(BArchivable *object, const char *propName, BString *value)
{
	TBCL_BMenuItem::GetValue(object, propName, value);
	return B_OK;
}


// BMenuBar -------------------------------------------------------------------------------

TBCL_BMenuBar::TBCL_BMenuBar(instantiation_info_t *info)
	:	TBCL_BMenu(info)
{
}

TBCL_BMenuBar::~TBCL_BMenuBar()
{
}

BArchivable *TBCL_BMenuBar::Instantiate()
{
	BMenuBar *menuBar = new BMenuBar(BRect(0,0,0,0), "");
	BMenu *menu = new BMenu("File");
	menu->AddItem(new BMenuItem("Open", NULL));
	menu->AddItem(new BMenuItem("Save", NULL));
	menu->AddItem(new BSeparatorItem());
	menu->AddItem(new BMenuItem("Quit", NULL));
	menuBar->AddItem(menu);
	menu = new BMenu("Edit");
	menu->AddItem(new BMenuItem("Copy", NULL));
	menu->AddItem(new BMenuItem("Paste", NULL));
	menu->AddItem(new BSeparatorItem());
	menu->AddItem(new BMenuItem("Select All", NULL));
	menuBar->AddItem(menu);
	return menuBar;
}

const char *TBCL_BMenuBar::SupportedClass() const
{
	return typeid(BMenuBar).name();
}

void TBCL_BMenuBar::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	propertyRegistrar->AddProperty(this, "Border", B_INT32_TYPE, NULL, &BORDER_VALUES);
	TBCL_BMenu::PropertyList(propertyRegistrar);
}

status_t TBCL_BMenuBar::SetValue(BArchivable *object, const char *propName, const char *value)
{
	_CAST(BMenuBar, object, menuBar);
	status_t status = B_ERROR;
	
	if (strcmp(propName, "Border")==0) {
		status = B_OK;
		TStringEx str(value);
		str.Trim();
		if (str.Compare("B_BORDER_FRAME")==0) { menuBar->SetBorder(B_BORDER_FRAME); } else
		if (str.Compare("B_BORDER_CONTENTS")==0) { menuBar->SetBorder(B_BORDER_CONTENTS); } else
		if (str.Compare("B_BORDER_EACH_ITEM")==0) { menuBar->SetBorder(B_BORDER_EACH_ITEM); } else
#if B_BEOS_VERSION >= 0x0600
		if (str.Compare("B_BORDER_OFF")==0) { menuBar->SetBorder(B_BORDER_OFF); } else
#endif
		status = B_ERROR;
	} else {
		status = TBCL_BMenu::SetValue(object, propName, value);
	}
	
	return status;
}

status_t TBCL_BMenuBar::GetValue(BArchivable *object, const char *propName, BString *value)
{
	_CAST(BMenuBar, object, menuBar);
	
	if (strcmp(propName, "Border")==0) {
		switch(menuBar->Border()) {
			case B_BORDER_FRAME: value->SetTo("B_BORDER_FRAME"); break;
			case B_BORDER_CONTENTS: value->SetTo("B_BORDER_CONTENTS"); break;
			case B_BORDER_EACH_ITEM: value->SetTo("B_BORDER_EACH_ITEM"); break;
#if B_BEOS_VERSION >= 0x0510
			case B_BORDER_OFF: value->SetTo("B_BORDER_OFF"); break;
#endif
		}
	} else {
		TBCL_BMenu::GetValue(object, propName, value);
	}
	return B_OK;
}

// BBox -------------------------------------------------------------------------------

TBCL_BBox::TBCL_BBox(instantiation_info_t *info)
	:	TBCL_BView(info)
{
	SetControlStyle(CS_ACCEPTS_CONTROLS | ControlStyle());
}

TBCL_BBox::~TBCL_BBox()
{
}

BArchivable *TBCL_BBox::Instantiate()
{
	BBox *box = new BBox(BRect(0,0,0,0), "");
	box->SetLabel("New BBox");
	return box;
}

const char *TBCL_BBox::SupportedClass() const
{
	return typeid(BBox).name();
}

void TBCL_BBox::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	propertyRegistrar->AddProperty(this, "Label", B_STRING_TYPE);
	propertyRegistrar->AddProperty(this, "Style", B_INT32_TYPE, NULL, &BORDER_STYLE_VALUES);
	TBCL_BView::PropertyList(propertyRegistrar);
}

status_t TBCL_BBox::SetValue(BArchivable *object, const char *propName, const char *value)
{
	_CAST(BBox, object, box);
	status_t status = B_ERROR;
	
	if (strcmp(propName, "Label")==0) {
		box->SetLabel(value);
		status = B_OK;
	} else if (strcmp(propName, "Style")==0) {
		status = B_OK;
		TStringEx str(value);
		str.Trim();
		if (str.Compare("B_PLAIN_BORDER")==0) { box->SetBorder(B_PLAIN_BORDER); } else
		if (str.Compare("B_FANCY_BORDER")==0) { box->SetBorder(B_FANCY_BORDER); } else
		if (str.Compare("B_NO_BORDER")==0) { box->SetBorder(B_NO_BORDER); } else
		status = B_ERROR;
	} else {
		status = TBCL_BView::SetValue(object, propName, value);
	}
	
	return status;
}

status_t TBCL_BBox::GetValue(BArchivable *object, const char *propName, BString *value)
{
	_CAST(BBox, object, box);
	
	if (strcmp(propName, "Label")==0) {
		value->SetTo(box->Label());
	} else if (strcmp(propName, "Style")==0) {
		switch(box->Border()) {
			case B_PLAIN_BORDER: value->SetTo("B_PLAIN_BORDER"); break;
			case B_FANCY_BORDER: value->SetTo("B_FANCY_BORDER"); break;
			case B_NO_BORDER: value->SetTo("B_NO_BORDER"); break;
		}
	} else {
		TBCL_BView::GetValue(object, propName, value);
	}
	return B_OK;
}

status_t TBCL_BBox::GetDefaultSize(float *width, float *height)
{
	*width = 100;
	*height = 100;
	return B_OK;
}


// BPictureButton -------------------------------------------------------------------------------

TBCL_BPictureButton::TBCL_BPictureButton(instantiation_info_t *info)
	:	TBCL_BControl(info)
{
}

TBCL_BPictureButton::~TBCL_BPictureButton()
{
}

BArchivable *TBCL_BPictureButton::Instantiate()
{
	return new BPictureButton(BRect(0,0,0,0), "", new BPicture, new BPicture, NULL);
}

const char *TBCL_BPictureButton::SupportedClass() const
{
	return typeid(BPictureButton).name();
}

void TBCL_BPictureButton::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	propertyRegistrar->AddProperty(this, "BeHave", B_INT32_TYPE, NULL, &BEHAVE_VALUES);
	TBCL_BControl::PropertyList(propertyRegistrar);
}

status_t TBCL_BPictureButton::SetValue(BArchivable *object, const char *propName, const char *value)
{
	_CAST(BPictureButton, object, pictureButton);
	status_t status = B_ERROR;
	
	if (strcmp(propName, "BeHave")==0) {
		status = B_OK;
		TStringEx str(value);
		str.Trim();
		if (str.Compare("B_ONE_STATE_BUTTON")==0) { pictureButton->SetBehavior(B_ONE_STATE_BUTTON); } else
		if (str.Compare("B_TWO_STATE_BUTTON")==0) { pictureButton->SetBehavior(B_TWO_STATE_BUTTON); } else
			status = B_ERROR;
	} else {
		status = TBCL_BControl::SetValue(object, propName, value);
	}
	
	return status;
}

status_t TBCL_BPictureButton::GetValue(BArchivable *object, const char *propName, BString *value)
{
	_CAST(BPictureButton, object, pictureButton);
	
	if (strcmp(propName, "BeHave")==0) {
		switch(pictureButton->Behavior()) {
			case B_ONE_STATE_BUTTON: value->SetTo("B_ONE_STATE_BUTTON"); break;
			case B_TWO_STATE_BUTTON: value->SetTo("B_TWO_STATE_BUTTON"); break;
		}
	} else {
		TBCL_BControl::GetValue(object, propName, value);
	}
	
	return B_OK;
}


// BCheckBox -------------------------------------------------------------------------------

TBCL_BCheckBox::TBCL_BCheckBox(instantiation_info_t *info)
	:	TBCL_BControl(info)
{
}

TBCL_BCheckBox::~TBCL_BCheckBox()
{
}

BArchivable *TBCL_BCheckBox::Instantiate()
{
	return new BCheckBox(BRect(0,0,0,0), "", "", NULL);
}

const char *TBCL_BCheckBox::SupportedClass() const
{
	return typeid(BCheckBox).name();
}

void TBCL_BCheckBox::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	TBCL_BControl::PropertyList(propertyRegistrar);
}

status_t TBCL_BCheckBox::SetValue(BArchivable *object, const char *propName, const char *value)
{
	return TBCL_BControl::SetValue(object, propName, value);
}

status_t TBCL_BCheckBox::GetValue(BArchivable *object, const char *propName, BString *value)
{
	TBCL_BControl::GetValue(object, propName, value);
	return B_OK;
}


status_t TBCL_BCheckBox::GetDefaultSize(float *width, float *height)
{
	*width = 100;
	*height = 20;
	return B_OK;
}


// BColorControl -------------------------------------------------------------------------------

TBCL_BColorControl::TBCL_BColorControl(instantiation_info_t *info)
	:	TBCL_BControl(info)
{
}

TBCL_BColorControl::~TBCL_BColorControl()
{
}

BArchivable *TBCL_BColorControl::Instantiate()
{
	BColorControl *colorControl = new BColorControl(BPoint(0,0), B_CELLS_32x8, 0, "");
	return colorControl;
}

const char *TBCL_BColorControl::SupportedClass() const
{
	return typeid(BColorControl).name();
}

void TBCL_BColorControl::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	propertyRegistrar->AddProperty(this, "Color", B_INT32_TYPE, PROPEDITOR(TColorEditor));
	TBCL_BControl::PropertyList(propertyRegistrar);
}

status_t TBCL_BColorControl::SetValue(BArchivable *object, const char *propName, const char *value)
{
	_CAST(BColorControl, object, colorControl);
	status_t status = B_ERROR;
	
	if (strcmp(propName, "Color")==0) {
		rgb_color rgb;
		status = get_rgb_color(value, &rgb);
		if (status == B_OK) colorControl->SetValue(rgb);
	} else {
		status = TBCL_BControl::SetValue(object, propName, value);
	}
	
	return status;
}

status_t TBCL_BColorControl::GetValue(BArchivable *object, const char *propName, BString *value)
{
	_CAST(BColorControl, object, colorControl);
	
	if (strcmp(propName, "Color")==0) {
		get_rgb_color_value(colorControl->ValueAsColor(), value);
	} else {
		TBCL_BControl::GetValue(object, propName, value);
	}
	return B_OK;
}

status_t TBCL_BColorControl::GetDefaultSize(float *width, float *height)
{
	return B_ERROR;
}


// BTextControl -------------------------------------------------------------------------------

TBCL_BTextControl::TBCL_BTextControl(instantiation_info_t *info)
	:	TBCL_BControl(info)
{
}

TBCL_BTextControl::~TBCL_BTextControl()
{
}

BArchivable *TBCL_BTextControl::Instantiate()
{
	BTextControl *textControl = new BTextControl(BRect(0,0,200,0), "", "", "", NULL);
	textControl->SetDivider(90);
	return textControl;
}

const char *TBCL_BTextControl::SupportedClass() const
{
	return typeid(BTextControl).name();
}

void TBCL_BTextControl::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	propertyRegistrar->AddProperty(this, "Divide", B_FLOAT_TYPE);
	propertyRegistrar->AddProperty(this, "LabelAlignment", B_INT32_TYPE, NULL, &ALIGN_VALUES);
	propertyRegistrar->AddProperty(this, "TextAlignment", B_INT32_TYPE, NULL, &ALIGN_VALUES);
	TBCL_BControl::PropertyList(propertyRegistrar);
}

status_t TBCL_BTextControl::SetValue(BArchivable *object, const char *propName, const char *value)
{
	_CAST(BTextControl, object, textControl);
	status_t status = B_ERROR;
	
	if (strcmp(propName, "Divide")==0) {
		textControl->SetDivider(atof(value));
		status = B_OK;
	} else if (strcmp(propName, "LabelAlignment")==0) {
		alignment forLabel, forText;
		textControl->GetAlignment(&forLabel, &forText);
		status = fGetAlignment(value, &forLabel);
		if (status == B_OK) textControl->SetAlignment(forLabel, forText);
	} else if (strcmp(propName, "TextAlignment")==0) {
		alignment forLabel, forText;
		textControl->GetAlignment(&forLabel, &forText);
		status = fGetAlignment(value, &forText);
		if (status == B_OK) textControl->SetAlignment(forLabel, forText);
	} else {
		status = TBCL_BControl::SetValue(object, propName, value);
	}
	
	return status;
}

status_t TBCL_BTextControl::GetValue(BArchivable *object, const char *propName, BString *value)
{
	_CAST(BTextControl, object, textControl);
	
	if (strcmp(propName, "Divide")==0) {
		get_double_value(textControl->Divider(), value);
	} else if (strcmp(propName, "LabelAlignment")==0) {
		alignment forLabel, forText;
		textControl->GetAlignment(&forLabel, &forText);
		switch(forLabel){
			case B_ALIGN_LEFT: value->SetTo("B_ALIGN_LEFT"); break;
			case B_ALIGN_RIGHT: value->SetTo("B_ALIGN_RIGHT"); break;
			case B_ALIGN_CENTER: value->SetTo("B_ALIGN_CENTER"); break;
		}
	} else if (strcmp(propName, "TextAlignment")==0) {
		alignment forLabel, forText;
		textControl->GetAlignment(&forLabel, &forText);
		switch(forText){
			case B_ALIGN_LEFT: value->SetTo("B_ALIGN_LEFT"); break;
			case B_ALIGN_RIGHT: value->SetTo("B_ALIGN_RIGHT"); break;
			case B_ALIGN_CENTER: value->SetTo("B_ALIGN_CENTER"); break;
		}
	} else {
		TBCL_BControl::GetValue(object, propName, value);
	}
	return B_OK;
}

status_t TBCL_BTextControl::GetDefaultSize(float *width, float *height)
{
	return B_ERROR;
}

status_t TBCL_BTextControl::fGetAlignment(const char *value, alignment *al)
{
	TStringEx v(value);
	v.Trim();
	if (v.Compare("B_ALIGN_LEFT")==0) { *al = B_ALIGN_LEFT; } else
	if (v.Compare("B_ALIGN_RIGHT")==0) { *al = B_ALIGN_RIGHT; } else
	if (v.Compare("B_ALIGN_CENTER")==0) { *al = B_ALIGN_CENTER; } else return B_ERROR;
	return B_OK;
}


// BRadioButton -------------------------------------------------------------------------------

TBCL_BRadioButton::TBCL_BRadioButton(instantiation_info_t *info)
	:	TBCL_BControl(info)
{
}

TBCL_BRadioButton::~TBCL_BRadioButton()
{
}

BArchivable *TBCL_BRadioButton::Instantiate()
{
	return new BRadioButton(BRect(0,0,110,0), "", "", NULL);
}

const char *TBCL_BRadioButton::SupportedClass() const
{
	return typeid(BRadioButton).name();
}

void TBCL_BRadioButton::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	TBCL_BControl::PropertyList(propertyRegistrar);
}

status_t TBCL_BRadioButton::SetValue(BArchivable *object, const char *propName, const char *value)
{
	return TBCL_BControl::SetValue(object, propName, value);
}

status_t TBCL_BRadioButton::GetValue(BArchivable *object, const char *propName, BString *value)
{
	TBCL_BControl::GetValue(object, propName, value);
	return B_OK;
}

status_t TBCL_BRadioButton::GetDefaultSize(float *width, float *height)
{
	return B_ERROR;
}


// BSlider -------------------------------------------------------------------------------

TBCL_BSlider::TBCL_BSlider(instantiation_info_t *info)
	:	TBCL_BControl(info)
{
}

TBCL_BSlider::~TBCL_BSlider()
{
}

BArchivable *TBCL_BSlider::Instantiate()
{
	BSlider *slider = new BSlider(BRect(0,0,0,0), "", "", NULL, 0, 100);
	slider->SetLimitLabels("0", "100");
	return slider;
}

const char *TBCL_BSlider::SupportedClass() const
{
	return typeid(BSlider).name();
}

void TBCL_BSlider::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	propertyRegistrar->AddProperty(this, "Style", B_INT32_TYPE, NULL, &STYLE_VALUES);
	propertyRegistrar->AddProperty(this, "Orientation", B_INT32_TYPE, NULL, &ORIENTATION_VALUES);
	propertyRegistrar->AddProperty(this, "HashMarks", B_INT16_TYPE, NULL, &HASH_MARKS_VALUES);
	propertyRegistrar->AddProperty(this, "HashMarkCount", B_INT32_TYPE);
	propertyRegistrar->AddProperty(this, "KeyIncrementValue", B_INT32_TYPE);
	propertyRegistrar->AddProperty(this, "Max", B_INT32_TYPE);
	propertyRegistrar->AddProperty(this, "Min", B_INT32_TYPE);
	propertyRegistrar->AddProperty(this, "MaxLimitLabel", B_STRING_TYPE);
	propertyRegistrar->AddProperty(this, "MinLimitLabel", B_STRING_TYPE);
	propertyRegistrar->AddProperty(this, "BarColor", B_INT32_TYPE, PROPEDITOR(TColorEditor));
	propertyRegistrar->AddProperty(this, "UseFillColor", B_BOOL_TYPE, NULL, &BOOLEAN_VALUES);
	propertyRegistrar->AddProperty(this, "FillColor", B_INT32_TYPE, PROPEDITOR(TColorEditor));
	propertyRegistrar->AddProperty(this, "SnoozeAmount", B_INT32_TYPE);
	TBCL_BControl::PropertyList(propertyRegistrar);
}

status_t TBCL_BSlider::SetValue(BArchivable *object, const char *propName, const char *value)
{
	_CAST(BSlider, object, slider);
	status_t status = B_ERROR;
	
	if (strcmp(propName, "Style")==0) {
		status = B_OK;
		TStringEx str(value);
		str.Trim();
		if (str.Compare("B_BLOCK_THUMB")==0) { slider->SetStyle(B_BLOCK_THUMB); } else
		if (str.Compare("B_TRIANGLE_THUMB")==0) { slider->SetStyle(B_TRIANGLE_THUMB); } else
			status = B_ERROR;
	} else if (strcmp(propName, "Orientation")==0) {
		status = B_OK;
		TStringEx str(value);
		str.Trim();
		if (str.Compare("B_HORIZONTAL")==0) { slider->SetOrientation(B_HORIZONTAL); } else
		if (str.Compare("B_VERTICAL")==0) { slider->SetOrientation(B_VERTICAL); } else
			status = B_ERROR;
	} else if (strcmp(propName, "HashMarks")==0) {
		status = B_OK;
		TStringEx str(value);
		str.Trim();
		if (str.Compare("B_HASH_MARKS_NONE")==0) { slider->SetHashMarks(B_HASH_MARKS_NONE); } else
		if (str.Compare("B_HASH_MARKS_TOP")==0) { slider->SetHashMarks(B_HASH_MARKS_TOP); } else
		if (str.Compare("B_HASH_MARKS_BOTTOM")==0) { slider->SetHashMarks(B_HASH_MARKS_BOTTOM); } else
		if (str.Compare("B_HASH_MARKS_BOTH")==0) { slider->SetHashMarks(B_HASH_MARKS_BOTH); } else
		if (str.Compare("B_HASH_MARKS_LEFT")==0) { slider->SetHashMarks(B_HASH_MARKS_LEFT); } else
		if (str.Compare("B_HASH_MARKS_RIGHT")==0) { slider->SetHashMarks(B_HASH_MARKS_RIGHT); } else
			status = B_ERROR;
	} else if (strcmp(propName, "HashMarkCount")==0) {
		slider->SetHashMarkCount(atoi(value));
		status = B_OK;
	} else if (strcmp(propName, "KeyIncrementValue")==0) {
		slider->SetKeyIncrementValue(atoi(value));
		status = B_OK;
	} else if (strcmp(propName, "Max")==0) {
//		slider->fMaxValue = atoi(value);
		status = B_OK;
	} else if (strcmp(propName, "Min")==0) {
//		slider->fMinValue = atoi(value);
		status = B_OK;
	} else if (strcmp(propName, "MaxLimitLabel")==0) {
		BString minLabel(slider->MinLimitLabel());
		slider->SetLimitLabels(minLabel.String(), value);
		status = B_OK;
	} else if (strcmp(propName, "MinLimitLabel")==0) {
		BString maxLabel(slider->MaxLimitLabel());
		slider->SetLimitLabels(value, maxLabel.String());
		status = B_OK;
	} else if (strcmp(propName, "UseFillColor")==0) {
		status = fSetUseFillColor(slider, value);
	} else if (strcmp(propName, "FillColor")==0) {
		rgb_color fillColor;
		bool use = slider->FillColor(&fillColor);
		status = get_rgb_color(value, &fillColor);
		if (status == B_OK) slider->UseFillColor(use, &fillColor);
	} else if (strcmp(propName, "BarColor")==0) {
		rgb_color rgb;
		status = get_rgb_color(value, &rgb);
		if (status == B_OK) slider->SetBarColor(rgb);
	} else if (strcmp(propName, "SnoozeAmount")==0) {
		slider->SetSnoozeAmount(atoi(value));
		status = B_OK;
	} else {
		status = TBCL_BControl::SetValue(object, propName, value);
	}
	
	return status;
}

status_t TBCL_BSlider::GetValue(BArchivable *object, const char *propName, BString *value)
{
	_CAST(BSlider, object, slider);
	
	if (strcmp(propName, "Style")==0) {
		switch(slider->Style()) {
			case B_BLOCK_THUMB: value->SetTo("B_BLOCK_THUMB"); break;
			case B_TRIANGLE_THUMB: value->SetTo("B_TRIANGLE_THUMB"); break;
		}
	} else if (strcmp(propName, "Orientation")==0) {
		switch(slider->Orientation()) {
			case B_HORIZONTAL: value->SetTo("B_HORIZONTAL"); break;
			case B_VERTICAL: value->SetTo("B_VERTICAL"); break;
		}
	} else if (strcmp(propName, "HashMarks")==0) {
		switch(slider->Orientation()) {
			case B_HORIZONTAL:
				switch(slider->HashMarks()) {
					case B_HASH_MARKS_NONE: value->SetTo("B_HASH_MARKS_NONE"); break;
					case B_HASH_MARKS_TOP: value->SetTo("B_HASH_MARKS_TOP"); break;
					case B_HASH_MARKS_BOTTOM: value->SetTo("B_HASH_MARKS_BOTTOM"); break;
					case B_HASH_MARKS_BOTH: value->SetTo("B_HASH_MARKS_BOTH"); break;
				}
				break;
			case B_VERTICAL:
				switch(slider->HashMarks()) {
					case B_HASH_MARKS_NONE: value->SetTo("B_HASH_MARKS_NONE"); break;
					case B_HASH_MARKS_LEFT: value->SetTo("B_HASH_MARKS_LEFT"); break;
					case B_HASH_MARKS_RIGHT: value->SetTo("B_HASH_MARKS_RIGHT"); break;
					case B_HASH_MARKS_BOTH: value->SetTo("B_HASH_MARKS_BOTH"); break;
				}
			break;
		}
	} else if (strcmp(propName, "HashMarkCount")==0) {
		get_int_value(slider->HashMarkCount(), value);
	} else if (strcmp(propName, "KeyIncrementValue")==0) {
		get_int_value(slider->KeyIncrementValue(), value);
	} else if (strcmp(propName, "Max")==0) {
		int32 min, max;
		slider->GetLimits(&min, &max);
		get_int_value(max, value);
	} else if (strcmp(propName, "Min")==0) {
		int32 min, max;
		slider->GetLimits(&min, &max);
		get_int_value(min, value);
	} else if (strcmp(propName, "MaxLimitLabel")==0) {
		value->SetTo(slider->MaxLimitLabel());
	} else if (strcmp(propName, "MinLimitLabel")==0) {
		value->SetTo(slider->MinLimitLabel());
	} else if (strcmp(propName, "UseFillColor")==0) {
		rgb_color fillColor;
		value->SetTo(slider->FillColor(&fillColor) ? "True" : "False");
	} else if (strcmp(propName, "FillColor")==0) {
		rgb_color fillColor;
		slider->FillColor(&fillColor);
		get_rgb_color_value(fillColor, value);
	} else if (strcmp(propName, "BarColor")==0) {
		get_rgb_color_value(slider->BarColor(), value);
	} else if (strcmp(propName, "SnoozeAmount")==0) {
		get_int_value(slider->SnoozeAmount(), value);
	} else {
		TBCL_BControl::GetValue(object, propName, value);
	}
	
	return B_OK;
}

status_t TBCL_BSlider::fSetUseFillColor(BSlider *slider, const char *value)
{
	BString v(value);
	rgb_color barColor;
	slider->FillColor(&barColor);
	if (v.ICompare("TRUE")==0) { slider->UseFillColor(true, &barColor); } else
	if (v.ICompare("FALSE")==0) { slider->UseFillColor(false, &barColor); } else
		return B_ERROR;
	return B_OK;
}


// BListView -------------------------------------------------------------------------------

TBCL_BListView::TBCL_BListView(instantiation_info_t *info)
	:	TBCL_BView(info)
{
	SetControlStyle(!CS_ACCEPTS_CONTROLS & ControlStyle());
}

TBCL_BListView::~TBCL_BListView()
{
}

BArchivable *TBCL_BListView::Instantiate()
{
	return new BListView(BRect(0,0,0,0), "", B_SINGLE_SELECTION_LIST);
}

const char *TBCL_BListView::SupportedClass() const
{
	return typeid(BListView).name();
}

void TBCL_BListView::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	propertyRegistrar->AddProperty(this, "ListType", B_INT32_TYPE, NULL, &LIST_TYPE_VALUES);
	TBCL_BView::PropertyList(propertyRegistrar);
}

status_t TBCL_BListView::SetValue(BArchivable *object, const char *propName, const char *value)
{
	_CAST(BListView, object, listView);
	status_t status = B_ERROR;
	
	if (strcmp(propName, "ListType")==0) {
		status = B_OK;
		TStringEx str(value);
		str.Trim();
		if (str.Compare("B_SINGLE_SELECTION_LIST")==0) { listView->SetListType(B_SINGLE_SELECTION_LIST); } else
		if (str.Compare("B_MULTIPLE_SELECTION_LIST")==0) { listView->SetListType(B_MULTIPLE_SELECTION_LIST); } else
			status = B_ERROR;
	} else {
		status = TBCL_BView::SetValue(object, propName, value);
	}
	
	return status;
}

status_t TBCL_BListView::GetValue(BArchivable *object, const char *propName, BString *value)
{
	_CAST(BListView, object, listView);
	
	if (strcmp(propName, "ListType")==0) {
		switch(listView->ListType()) {
			case B_SINGLE_SELECTION_LIST: value->SetTo("B_SINGLE_SELECTION_LIST"); break;
			case B_MULTIPLE_SELECTION_LIST: value->SetTo("B_MULTIPLE_SELECTION_LIST"); break;
		}
	} else {
		TBCL_BView::GetValue(object, propName, value);
	}
	
	return B_OK;
}


// BOutlineListView -------------------------------------------------------------------------------

TBCL_BOutlineListView::TBCL_BOutlineListView(instantiation_info_t *info)
	:	TBCL_BListView(info)
{
}

TBCL_BOutlineListView::~TBCL_BOutlineListView()
{
}

BArchivable *TBCL_BOutlineListView::Instantiate()
{
	return new BOutlineListView(BRect(0,0,0,0), "", B_SINGLE_SELECTION_LIST);
}

const char *TBCL_BOutlineListView::SupportedClass() const
{
	return typeid(BOutlineListView).name();
}

void TBCL_BOutlineListView::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	TBCL_BListView::PropertyList(propertyRegistrar);
}

status_t TBCL_BOutlineListView::SetValue(BArchivable *object, const char *propName, const char *value)
{
	return TBCL_BListView::SetValue(object, propName, value);
}

status_t TBCL_BOutlineListView::GetValue(BArchivable *object, const char *propName, BString *value)
{
	TBCL_BListView::GetValue(object, propName, value);
	return B_OK;
}


// BMenuField -------------------------------------------------------------------------------

TBCL_BMenuField::TBCL_BMenuField(instantiation_info_t *info)
	:	TBCL_BView(info)
{
	bclBMenu = new TBCL_BMenu(info);
	SetControlStyle(!CS_ACCEPTS_CONTROLS & ControlStyle());
}

TBCL_BMenuField::~TBCL_BMenuField()
{
	delete bclBMenu;
}

BArchivable *TBCL_BMenuField::Instantiate()
{
	BMenuField *menuField = new BMenuField(BRect(0,0,100,0), 
					"New BMenuField", "New BMenuField", new BPopUpMenu("Menu"));
//	BMenuField *menuField = new BMenuField(BRect(0,0,100,0), 
//					"New BMenuField", "New BMenuField", new BMenu("Menu"));
	menuField->SetDivider(90);
	return menuField;
}

const char *TBCL_BMenuField::SupportedClass() const
{
	return typeid(BMenuField).name();
}

void TBCL_BMenuField::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	propertyRegistrar->AddObject(this, "Menu", bclBMenu);
	propertyRegistrar->AddProperty(this, "Label", B_STRING_TYPE);
	propertyRegistrar->AddProperty(this, "Divide", B_FLOAT_TYPE);
	TBCL_BView::PropertyList(propertyRegistrar);
}

status_t TBCL_BMenuField::SetValue(BArchivable *object, const char *propName, const char *value)
{
	_CAST(BMenuField, object, menuField);
	status_t status = B_OK;
	
	if (strcmp(propName, "Divide")==0) {
		menuField->SetDivider(atof(value));
		status = B_OK;
	} else if (strcmp(propName, "Label")==0) {
		menuField->SetLabel(value);
		status = B_OK;
	} else {
		status = TBCL_BView::SetValue(object, propName, value);
	}
	
	return status;
}

status_t TBCL_BMenuField::GetValue(BArchivable *object, const char *propName, BString *value)
{
	_CAST(BMenuField, object, menuField);
	
	if (strcmp(propName, "Divide")==0) {
		get_double_value(menuField->Divider(), value);
	} else if (strcmp(propName, "Label")==0) {
		value->SetTo(menuField->Label());
	} else {
		TBCL_BView::GetValue(object, propName, value);
	}
	
	return B_OK;
}

status_t TBCL_BMenuField::GetDefaultSize(float *width, float *height)
{
	return B_ERROR;
}

status_t TBCL_BMenuField::GetSuperObject(BArchivable *object, const char *propName, BArchivable **superObject)
{
	_CAST(BMenuField, object, menuField);
	
	if (strcmp(propName, "Menu")==0) {
		*superObject = menuField->Menu();
		return B_OK;
	} else {
		*superObject = NULL;
		return B_ERROR;
	}
}

// BTabView -----------------------------------------------------------------------------

TBCL_BTabView::TBCL_BTabView(instantiation_info_t *info)
	:	TBCL_BView(info)
{
}

TBCL_BTabView::~TBCL_BTabView()
{
}

BArchivable *TBCL_BTabView::Instantiate()
{
	BTabView *tabView = new BTabView(BRect(0,0,200,100),
		"New BTabView", B_WIDTH_FROM_LABEL, B_FOLLOW_TOP | B_FOLLOW_LEFT);
	BRect aRect = tabView->Bounds();
	aRect.bottom -= tabView->TabHeight();
	BView *view;
	
	view = new BView(aRect, "Tab1", B_FOLLOW_ALL, B_WILL_DRAW | B_FRAME_EVENTS);
	view->SetViewColor(216, 216, 216);
	tabView->AddTab(view);
	
	view = new BView(aRect, "Tab2", B_FOLLOW_ALL, B_WILL_DRAW | B_FRAME_EVENTS);
	view->SetViewColor(216, 216, 216);
	tabView->AddTab(view);
	
	return tabView;
}

const char *TBCL_BTabView::SupportedClass() const
{
	return typeid(BTabView).name();
}

void TBCL_BTabView::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	propertyRegistrar->AddProperty(this, "TabWidth", B_INT16_TYPE, NULL, &TABWIDTH_VALUES);
	propertyRegistrar->AddProperty(this, "TabHeight", B_FLOAT_TYPE);
	propertyRegistrar->AddProperty(this, "Tabs", B_RAW_TYPE, PROPEDITOR(TTabEditor));
	TBCL_BView::PropertyList(propertyRegistrar);
}

status_t TBCL_BTabView::SetValue(BArchivable *object, const char *propName, const char *value)
{
	_CAST(BTabView, object, tabView);
	status_t status = B_ERROR;
	
	if (strcmp(propName, "TabWidth")==0) {
		status = B_OK;
		TStringEx str(value);
		str.Trim();
		if (str.Compare("B_WIDTH_FROM_WIDEST")==0) { tabView->SetTabWidth(B_WIDTH_FROM_WIDEST); } else
		if (str.Compare("B_WIDTH_AS_USUAL")==0) { tabView->SetTabWidth(B_WIDTH_AS_USUAL); } else
		if (str.Compare("B_WIDTH_FROM_LABEL")==0) { tabView->SetTabWidth(B_WIDTH_FROM_LABEL); } else
			status = B_ERROR;
	} else if (strcmp(propName, "TabHeight")==0) {
		tabView->SetTabHeight(atof(value));
		status = B_OK;
	} else {
		status = TBCL_BView::SetValue(object, propName, value);
	}
	
	return status;
}

status_t TBCL_BTabView::GetValue(BArchivable *object, const char *propName, BString *value)
{
	_CAST(BTabView, object, tabView);
	
	if (strcmp(propName, "TabWidth")==0) {
		switch(tabView->TabWidth()) {
			case B_WIDTH_FROM_WIDEST: value->SetTo("B_WIDTH_FROM_WIDEST"); break;
			case B_WIDTH_AS_USUAL: value->SetTo("B_WIDTH_AS_USUAL"); break;
			case B_WIDTH_FROM_LABEL: value->SetTo("B_WIDTH_FROM_LABEL"); break;
		}
	} else if (strcmp(propName, "TabHeight")==0) {
		get_double_value(tabView->TabHeight(), value);
	} else if (strcmp(propName, "Tabs")==0) {
		value->SetTo("(Tabs)");
	} else {
		TBCL_BView::GetValue(object, propName, value);
	}
	
	return B_OK;
}

status_t TBCL_BTabView::GetDefaultSize(float *width, float *height)
{
//	*width = 200;
//	*height = 100;
//	return B_OK;
	return B_ERROR;
}

// BStringView -----------------------------------------------------------------------------

TBCL_BStringView::TBCL_BStringView(instantiation_info_t *info)
	:	TBCL_BView(info)
{
	SetControlStyle(!CS_ACCEPTS_CONTROLS & ControlStyle());
}

TBCL_BStringView::~TBCL_BStringView()
{
}

BArchivable *TBCL_BStringView::Instantiate()
{
	return new BStringView(BRect(0,0,100,20), "New BStringView", "New BStringView");
}

const char *TBCL_BStringView::SupportedClass() const
{
	return typeid(BStringView).name();
}

void TBCL_BStringView::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	propertyRegistrar->AddProperty(this, "Alignment", B_INT32_TYPE, NULL, &ALIGN_VALUES);
	propertyRegistrar->AddProperty(this, "Text", B_STRING_TYPE);
	TBCL_BView::PropertyList(propertyRegistrar);
}

status_t TBCL_BStringView::SetValue(BArchivable *object, const char *propName, const char *value)
{
	_CAST(BStringView, object, stringView);
	status_t status = B_ERROR;
	
	if (strcmp(propName, "Alignment")==0) {
		status = B_OK;
		TStringEx str(value);
		str.Trim();
		if (str.Compare("B_ALIGN_LEFT")==0) { stringView->SetAlignment(B_ALIGN_LEFT); } else
		if (str.Compare("B_ALIGN_RIGHT")==0) { stringView->SetAlignment(B_ALIGN_RIGHT); } else
		if (str.Compare("B_ALIGN_CENTER")==0) { stringView->SetAlignment(B_ALIGN_CENTER); } else
			status = B_ERROR;
	} else if (strcmp(propName, "Text")==0) {
		stringView->SetText(value);
		status = B_OK;
	} else {
		status = TBCL_BView::SetValue(object, propName, value);
	}
	
	return status;
}

status_t TBCL_BStringView::GetValue(BArchivable *object, const char *propName, BString *value)
{
	_CAST(BStringView, object, stringView);
	
	if (strcmp(propName, "Alignment")==0) {
		switch(stringView->Alignment()) {
			case B_ALIGN_LEFT: value->SetTo("B_ALIGN_LEFT"); break;
			case B_ALIGN_RIGHT: value->SetTo("B_ALIGN_RIGHT"); break;
			case B_ALIGN_CENTER: value->SetTo("B_ALIGN_CENTER"); break;
		}
	} else if (strcmp(propName, "Text")==0) {
		value->SetTo(stringView->Text());
	} else {
		TBCL_BView::GetValue(object, propName, value);
	}
	
	return B_OK;
}

status_t TBCL_BStringView::GetDefaultSize(float *width, float *height)
{
	*width = 100;
	*height = 20;
	return B_OK;
}


// BTextView -----------------------------------------------------------------------------

TBCL_BTextView::TBCL_BTextView(instantiation_info_t *info)
	:	TBCL_BView(info)
{
	SetControlStyle(!CS_ACCEPTS_CONTROLS & ControlStyle());
}

TBCL_BTextView::~TBCL_BTextView()
{
}

BArchivable *TBCL_BTextView::Instantiate()
{
	return new BTextView(BRect(0,0,200,100), "New BTextView", BRect(0,0,200,100), B_FOLLOW_NONE, B_WILL_DRAW | B_FRAME_EVENTS);
}

const char *TBCL_BTextView::SupportedClass() const
{
	return typeid(BTextView).name();
}

void TBCL_BTextView::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	propertyRegistrar->AddProperty(this, "Alignment", B_INT32_TYPE, NULL, &ALIGN_VALUES);
	propertyRegistrar->AddProperty(this, "Text", B_RAW_TYPE, PROPEDITOR(TTextViewEditor));
//	propertyRegistrar->AddProperty(this, "HideTyping", B_BOOL_TYPE, NULL, &BOOLEAN_VALUES);
	TBCL_BView::PropertyList(propertyRegistrar);
}

status_t TBCL_BTextView::SetValue(BArchivable *object, const char *propName, const char *value)
{
	_CAST(BTextView, object, textView);
	status_t status = B_ERROR;
	
	if (strcmp(propName, "Alignment")==0) {
		status = B_OK;
		TStringEx str(value);
		str.Trim();
		if (str.Compare("B_ALIGN_LEFT")==0) { textView->SetAlignment(B_ALIGN_LEFT); } else
		if (str.Compare("B_ALIGN_RIGHT")==0) { textView->SetAlignment(B_ALIGN_RIGHT); } else
		if (str.Compare("B_ALIGN_CENTER")==0) { textView->SetAlignment(B_ALIGN_CENTER); } else
			status = B_ERROR;
	} else if (strcmp(propName, "Text")==0) {
		status = B_OK;
//	} else if (strcmp(propName, "HideTyping")==0) {
//		bool enabled;
//		status = get_bool(value, &enabled);
//		if (status == B_OK) textView->HideTyping(enabled);
	} else {
		status = TBCL_BView::SetValue(object, propName, value);
	}
	
	return status;
}

status_t TBCL_BTextView::GetValue(BArchivable *object, const char *propName, BString *value)
{
	_CAST(BTextView, object, textView);
	
	if (strcmp(propName, "Alignment")==0) {
		switch(textView->Alignment()) {
			case B_ALIGN_LEFT: value->SetTo("B_ALIGN_LEFT"); break;
			case B_ALIGN_RIGHT: value->SetTo("B_ALIGN_RIGHT"); break;
			case B_ALIGN_CENTER: value->SetTo("B_ALIGN_CENTER"); break;
		}
	} else if (strcmp(propName, "Text")==0) {
		value->SetTo("(text)");
//	} else if (strcmp(propName, "HideTyping")==0) {
//		value->SetTo(textView->IsTypingHidden() ? "True" : "False");
	} else {
		TBCL_BView::GetValue(object, propName, value);
	}
	
	return B_OK;
}

status_t TBCL_BTextView::GetDefaultSize(float *width, float *height)
{
	*width = 200;
	*height = 100;
	return B_OK;
}

// BStatusBar -----------------------------------------------------------------------------

TBCL_BStatusBar::TBCL_BStatusBar(instantiation_info_t *info)
	:	TBCL_BView(info)
{
	SetControlStyle(!CS_ACCEPTS_CONTROLS & ControlStyle());
}

TBCL_BStatusBar::~TBCL_BStatusBar()
{
}

BArchivable *TBCL_BStatusBar::Instantiate()
{
	return new BStatusBar(BRect(0,0,200,32), "New BStatusBar", "New BStatusBar");
}

const char *TBCL_BStatusBar::SupportedClass() const
{
	return typeid(BStatusBar).name();
}

void TBCL_BStatusBar::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	propertyRegistrar->AddProperty(this, "BarColor", B_INT32_TYPE, PROPEDITOR(TColorEditor));
	propertyRegistrar->AddProperty(this, "BarHeight", B_FLOAT_TYPE);
	propertyRegistrar->AddProperty(this, "CurrentValue", B_FLOAT_TYPE);
	propertyRegistrar->AddProperty(this, "MaxValue", B_FLOAT_TYPE);
	propertyRegistrar->AddProperty(this, "Text", B_STRING_TYPE);
	propertyRegistrar->AddProperty(this, "TrailingText", B_STRING_TYPE);
	propertyRegistrar->AddProperty(this, "Label", B_STRING_TYPE);
	propertyRegistrar->AddProperty(this, "TrailingLabel", B_STRING_TYPE);
	TBCL_BView::PropertyList(propertyRegistrar);
}

status_t TBCL_BStatusBar::SetValue(BArchivable *object, const char *propName, const char *value)
{
	_CAST(BStatusBar, object, statusBar);
	status_t status = B_ERROR;
	BString label(statusBar->Label());
	BString trailingLabel(statusBar->TrailingLabel());
	BString text(statusBar->Text());
	BString trailingText(statusBar->TrailingText());
	
	if (strcmp(propName, "BarColor")==0) {
		rgb_color rgb;
		status = get_rgb_color(value, &rgb);
		if (status == B_OK) statusBar->SetBarColor(rgb);
	} else if (strcmp(propName, "BarHeight")==0) {
		statusBar->SetBarHeight(atof(value));
		status = B_OK;
	} else if (strcmp(propName, "CurrentValue")==0) {
		statusBar->Update(atof(value) - statusBar->CurrentValue(), text.String(), trailingText.String());
		status = B_OK;
	} else if (strcmp(propName, "MaxValue")==0) {
		statusBar->SetMaxValue(atof(value));
		status = B_OK;
	} else if (strcmp(propName, "Text")==0) {
		statusBar->SetText(value);
		status = B_OK;
	} else if (strcmp(propName, "TrailingText")==0) {
		statusBar->SetTrailingText(value);
		status = B_OK;
	} else if (strcmp(propName, "Label")==0) {
		float currentValue = statusBar->CurrentValue();
		statusBar->Reset(value, trailingLabel.String());
		statusBar->Update(currentValue, text.String(), trailingText.String());
		status = B_OK;
	} else if (strcmp(propName, "TrailingLabel")==0) {
		statusBar->Reset(label.String(), value);
		statusBar->Update(0, text.String(), trailingText.String());
		status = B_OK;
	} else {
		status = TBCL_BView::SetValue(object, propName, value);
	}
	
	return status;
}

status_t TBCL_BStatusBar::GetValue(BArchivable *object, const char *propName, BString *value)
{
	_CAST(BStatusBar, object, statusBar);
	
	if (strcmp(propName, "BarColor")==0) {
		get_rgb_color_value(statusBar->BarColor(), value);
	} else if (strcmp(propName, "BarHeight")==0) {
		get_double_value(statusBar->BarHeight(), value);
	} else if (strcmp(propName, "CurrentValue")==0) {
		get_double_value(statusBar->CurrentValue(), value);
	} else if (strcmp(propName, "MaxValue")==0) {
		get_double_value(statusBar->MaxValue(), value);
	} else if (strcmp(propName, "Text")==0) {
		value->SetTo(statusBar->Text());
	} else if (strcmp(propName, "TrailingText")==0) {
		value->SetTo(statusBar->TrailingText());
	} else if (strcmp(propName, "Label")==0) {
		value->SetTo(statusBar->Label());
	} else if (strcmp(propName, "TrailingLabel")==0) {
		value->SetTo(statusBar->TrailingLabel());
	} else {
		TBCL_BView::GetValue(object, propName, value);
	}
	
	return B_OK;
}

status_t TBCL_BStatusBar::GetDefaultSize(float *width, float *height)
{
	*width = 300;
	*height = 33;
	return B_OK;
}


// BScrollBar ---------------------------------------------------------------------

TBCL_BScrollBarH::TBCL_BScrollBarH(instantiation_info_t *info)
	:	TBCL_BView(info)
{
	SetControlStyle(!CS_ACCEPTS_CONTROLS & ControlStyle());
}

TBCL_BScrollBarH::~TBCL_BScrollBarH()
{
}

BArchivable *TBCL_BScrollBarH::Instantiate()
{
	BScrollBar *scrollBar = new BScrollBar(BRect(0,0,199,B_H_SCROLL_BAR_HEIGHT),
						"New BScrollBar", NULL, 0, 100, B_HORIZONTAL);
	scrollBar->SetResizingMode(B_FOLLOW_NONE);
	return scrollBar;
}

const char *TBCL_BScrollBarH::SupportedClass() const
{
	return "BScrollBar (Horizontal)";
}

void TBCL_BScrollBarH::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
}

status_t TBCL_BScrollBarH::SetValue(BArchivable *object, const char *propName, const char *value)
{
	return B_ERROR;
}

status_t TBCL_BScrollBarH::GetValue(BArchivable *object, const char *propName, BString *value)
{
	return B_ERROR;
}

status_t TBCL_BScrollBarH::GetDefaultSize(float *width, float *height)
{
	return B_ERROR;
}

TBCL_BScrollBar::TBCL_BScrollBar(instantiation_info_t *info)
	:	TBCL_BView(info)
{
	SetControlStyle(!CS_ACCEPTS_CONTROLS & ControlStyle());
}

TBCL_BScrollBar::~TBCL_BScrollBar()
{
}

BArchivable *TBCL_BScrollBar::Instantiate()
{
	BScrollBar *scrollBar = new BScrollBar(BRect(0,0,B_V_SCROLL_BAR_WIDTH,199),
						"New BScrollBar", NULL, 0, 100, B_VERTICAL);
	scrollBar->SetResizingMode(B_FOLLOW_NONE);
	return scrollBar;
}

const char *TBCL_BScrollBar::SupportedClass() const
{
	return typeid(BScrollBar).name();
}

void TBCL_BScrollBar::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	propertyRegistrar->AddProperty(this, "MinRange", B_INT32_TYPE);
	propertyRegistrar->AddProperty(this, "MaxRange", B_INT32_TYPE);
	propertyRegistrar->AddProperty(this, "SmallStep", B_FLOAT_TYPE);
	propertyRegistrar->AddProperty(this, "LargeStep", B_FLOAT_TYPE);
	propertyRegistrar->AddProperty(this, "Value", B_FLOAT_TYPE);
	propertyRegistrar->AddProperty(this, "Propotion", B_FLOAT_TYPE);
//	propertyRegistrar->AddProperty(this, "Orientation", B_INT32_TYPE, NULL, &ORIENTATION_VALUES);
	TBCL_BView::PropertyList(propertyRegistrar);
}

status_t TBCL_BScrollBar::SetValue(BArchivable *object, const char *propName, const char *value)
{
	_CAST(BScrollBar, object, scrollBar);
	status_t status = B_ERROR;
	float min, max, mins, maxs;
	scrollBar->GetRange(&min, &max);
	scrollBar->GetSteps(&mins, &maxs);
	
	if (strcmp(propName, "MinRange")==0) {
		scrollBar->SetRange(atof(value), max);
		status = B_OK;
	} else if (strcmp(propName, "MaxRange")==0) {
		scrollBar->SetRange(min, atof(value));
		status = B_OK;
	} else if (strcmp(propName, "SmallStep")==0) {
		scrollBar->SetSteps(atof(value), maxs);
		status = B_OK;
	} else if (strcmp(propName, "LargeStep")==0) {
		scrollBar->SetSteps(mins, atof(value));
		status = B_OK;
	} else if (strcmp(propName, "Value")==0) {
		scrollBar->SetValue(atof(value));
		status = B_OK;
	} else if (strcmp(propName, "Propotion")==0) {
		scrollBar->SetProportion(atof(value));
		status = B_OK;
/*	} else if (strcmp(propName, "Orientation")==0) {
		status = B_OK;
		TStringEx str(value);
		str.Trim();
		if (str.Compare("B_HORIZONTAL")==0) { fSetOrientation(scrollBar, B_HORIZONTAL); } else
		if (str.Compare("B_VERTICAL")==0) { fSetOrientation(scrollBar, B_VERTICAL); } else
			status = B_ERROR;*/
	} else {
		status = TBCL_BView::SetValue(object, propName, value);
	}
	
	return status;
}

status_t TBCL_BScrollBar::GetValue(BArchivable *object, const char *propName, BString *value)
{
	_CAST(BScrollBar, object, scrollBar);
	float min, max, mins, maxs;
	scrollBar->GetRange(&min, &max);
	scrollBar->GetSteps(&mins, &maxs);
	
	if (strcmp(propName, "MinRange")==0) {
		get_double_value(min, value);
	} else if (strcmp(propName, "MaxRange")==0) {
		get_double_value(max, value);
	} else if (strcmp(propName, "SmallStep")==0) {
		get_double_value(mins, value);
	} else if (strcmp(propName, "LargeStep")==0) {
		get_double_value(maxs, value);
	} else if (strcmp(propName, "Value")==0) {
		get_double_value(scrollBar->Value(), value);
	} else if (strcmp(propName, "Propotion")==0) {
		get_double_value(scrollBar->Proportion(), value);
	} else if (strcmp(propName, "Orientation")==0) {
		switch(scrollBar->Orientation()) {
			case B_HORIZONTAL: value->SetTo("B_HORIZONTAL"); break;
			case B_VERTICAL: value->SetTo("B_VERTICAL"); break;
		}
	} else {
		TBCL_BView::GetValue(object, propName, value);
	}
	
	return B_OK;
}

status_t TBCL_BScrollBar::GetDefaultSize(float *width, float *height)
{
	return B_ERROR;
}
/*
BScrollBar *TBCL_BScrollBar::fSetOrientation(BScrollBar *scrollBar, orientation which)
{
//	scrollBar->fOrientation = which;
	BView *parent = scrollBar->Parent();
	parent->RemoveChild(scrollBar);
	parent->AddChild(scrollBar);
	
	float width, height;
	scrollBar->GetPreferredSize(&width, &height);
	switch(scrollBar->Orientation()) {
		case B_VERTICAL: 
			scrollBar->ResizeTo(width, scrollBar->Frame().Width());
			break;
		case B_HORIZONTAL: 
			scrollBar->ResizeTo(scrollBar->Frame().Height(), height);
			break;
	}
	
	return scrollBar;
}
*/

// BScrollView -------------------------------------------------------------------------

TBCL_BScrollView::TBCL_BScrollView(instantiation_info_t *info)
	:	TBCL_BView(info)
{
	SetControlStyle(!CS_ACCEPTS_CONTROLS & ControlStyle());
}

TBCL_BScrollView::~TBCL_BScrollView()
{
}

BArchivable *TBCL_BScrollView::Instantiate()
{
	BView *target = new BView(BRect(0,0,200,100), "New BView",
		B_FOLLOW_ALL, B_WILL_DRAW | B_FRAME_EVENTS);
	BScrollView *scrollView = new BScrollView("New BScrollView",
		target, B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW | B_FRAME_EVENTS, true, true);
	return scrollView;
}

const char *TBCL_BScrollView::SupportedClass() const
{
	return typeid(BScrollView).name();
}

void TBCL_BScrollView::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	propertyRegistrar->AddProperty(this, "Style", B_INT32_TYPE, NULL, &BORDER_STYLE_VALUES);
//	propertyRegistrar->AddProperty(this, "Vertical", B_BOOL_TYPE);
//	propertyRegistrar->AddProperty(this, "Horizontal", B_BOOL_TYPE);
	TBCL_BView::PropertyList(propertyRegistrar);
}

status_t TBCL_BScrollView::SetValue(BArchivable *object, const char *propName, const char *value)
{
	_CAST(BScrollView, object, scrollView);
	status_t status = B_ERROR;
	
	if (strcmp(propName, "Style")==0) {
		status = B_OK;
		TStringEx str(value);
		str.Trim();
		if (str.Compare("B_PLAIN_BORDER")==0) { scrollView->SetBorder(B_PLAIN_BORDER); } else
		if (str.Compare("B_FANCY_BORDER")==0) { scrollView->SetBorder(B_FANCY_BORDER); } else
		if (str.Compare("B_NO_BORDER")==0) { scrollView->SetBorder(B_NO_BORDER); } else
		status = B_ERROR;
	} else if (strcmp(propName, "Vertical")==0) {
		status = B_OK;
	} else if (strcmp(propName, "Horizontal")==0) {
		status = B_OK;
	} else {
		status = TBCL_BView::SetValue(object, propName, value);
	}
	
	return status;
}

status_t TBCL_BScrollView::GetValue(BArchivable *object, const char *propName, BString *value)
{
	_CAST(BScrollView, object, scrollView);
	
	if (strcmp(propName, "Style")==0) {
		switch(scrollView->Border()) {
			case B_PLAIN_BORDER: value->SetTo("B_PLAIN_BORDER"); break;
			case B_FANCY_BORDER: value->SetTo("B_FANCY_BORDER"); break;
			case B_NO_BORDER: value->SetTo("B_NO_BORDER"); break;
		}
	} else if (strcmp(propName, "Vertical")==0) {
		
	} else if (strcmp(propName, "Horizontal")==0) {
		
	} else {
		TBCL_BView::GetValue(object, propName, value);
	}
	
	return B_OK;
}

status_t TBCL_BScrollView::GetDefaultSize(float *width, float *height)
{
	return B_ERROR;
}

// BScrollView (BListView) ------------------------------------------------------------------

TBCL_BScrollView_BListView::TBCL_BScrollView_BListView(instantiation_info_t *info)
	:	TBCL_BView(info)
{
	SetControlStyle(!CS_ACCEPTS_CONTROLS & ControlStyle());
}

TBCL_BScrollView_BListView::~TBCL_BScrollView_BListView()
{
}

BArchivable *TBCL_BScrollView_BListView::Instantiate()
{
	BListView *target = new BListView(BRect(0,0,200,100), "New BListView",
		B_SINGLE_SELECTION_LIST, B_FOLLOW_ALL_SIDES);
	BScrollView *scrollView = new BScrollView("New BScrollView",
		target, B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW | B_FRAME_EVENTS, false, true);
	return scrollView;
}

const char *TBCL_BScrollView_BListView::SupportedClass() const
{
	return "BScrollView (BListView)";
}

void TBCL_BScrollView_BListView::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
	propertyRegistrar->AddProperty(this, "Style", B_INT32_TYPE, NULL, &BORDER_STYLE_VALUES);
//	propertyRegistrar->AddProperty(this, "Vertical", B_BOOL_TYPE);
//	propertyRegistrar->AddProperty(this, "Horizontal", B_BOOL_TYPE);
	TBCL_BView::PropertyList(propertyRegistrar);
}

status_t TBCL_BScrollView_BListView::SetValue(BArchivable *object, const char *propName, const char *value)
{
	_CAST(BScrollView, object, scrollView);
	status_t status = B_ERROR;
	
	if (strcmp(propName, "Style")==0) {
		status = B_OK;
		TStringEx str(value);
		str.Trim();
		if (str.Compare("B_PLAIN_BORDER")==0) { scrollView->SetBorder(B_PLAIN_BORDER); } else
		if (str.Compare("B_FANCY_BORDER")==0) { scrollView->SetBorder(B_FANCY_BORDER); } else
		if (str.Compare("B_NO_BORDER")==0) { scrollView->SetBorder(B_NO_BORDER); } else
		status = B_ERROR;
	} else if (strcmp(propName, "Vertical")==0) {
		status = B_OK;
	} else if (strcmp(propName, "Horizontal")==0) {
		status = B_OK;
	} else {
		status = TBCL_BView::SetValue(object, propName, value);
	}
	
	return status;
}

status_t TBCL_BScrollView_BListView::GetValue(BArchivable *object, const char *propName, BString *value)
{
	_CAST(BScrollView, object, scrollView);
	
	if (strcmp(propName, "Style")==0) {
		switch(scrollView->Border()) {
			case B_PLAIN_BORDER: value->SetTo("B_PLAIN_BORDER"); break;
			case B_FANCY_BORDER: value->SetTo("B_FANCY_BORDER"); break;
			case B_NO_BORDER: value->SetTo("B_NO_BORDER"); break;
		}
	} else if (strcmp(propName, "Vertical")==0) {
		
	} else if (strcmp(propName, "Horizontal")==0) {
		
	} else {
		TBCL_BView::GetValue(object, propName, value);
	}
	
	return B_OK;
}

status_t TBCL_BScrollView_BListView::GetDefaultSize(float *width, float *height)
{
	return B_ERROR;
}
