#include <stdlib.h>
#include "BCL.h"
#include "BePropEditors.h"


// TColorEditor -----------------------------------------------------

TColorEditor::TColorEditor(propeditor_info_t *peInfo)
	:	TMultiPropEditor(peInfo, BRect(200, 100, 480, 195), "Color",
						B_FLOATING_WINDOW_LOOK, B_MODAL_APP_WINDOW_FEEL,
						B_NOT_RESIZABLE | B_NOT_ZOOMABLE)	
{
	BView *bg = new BView(Bounds(), "bgView", 
		B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP_BOTTOM,
		B_WILL_DRAW | B_FRAME_EVENTS);
	bg->SetViewColor(216, 216, 216, 0);
	AddChild(bg);
	
	fColorControl = new BColorControl(BPoint(5,5), B_CELLS_32x8, 0,
		"colorControl", new BMessage(COLORCONTROL_CHANGED));
	bg->AddChild(fColorControl);
	
	BButton *proof = new BButton(BRect(5, 65, 65, 85), "proof", "Proof", new BMessage(PROOF_BTN_CLICKED));
	bg->AddChild(proof);
	
	fAutoProof = new BCheckBox(BRect(70, 68, 140, 85), "autoProof", "Auto proof", NULL);
	fAutoProof->SetValue(1);
	bg->AddChild(fAutoProof);
	
	BButton *cancel = new BButton(BRect(150, 65, 210, 85), "cancel", "Cancel", new BMessage(CANCEL_BTN_CLICKED));
	bg->AddChild(cancel);
	
	BButton *ok = new BButton(BRect(215, 65, 275, 85), "ok", "OK", new BMessage(OK_BTN_CLICKED));
	bg->AddChild(ok);
}

TColorEditor::~TColorEditor()
{
}

void TColorEditor::MessageReceived(BMessage *msg)
{
	switch(msg->what)	{
		case PROOF_BTN_CLICKED:
			fUpdate();
			break;
		case OK_BTN_CLICKED: {
			fUpdate();
			BMessenger msgr(this);
			msgr.SendMessage(B_QUIT_REQUESTED);
			break;
		}
		case CANCEL_BTN_CLICKED: {
			fUndo();
			BMessenger msgr(this);
			msgr.SendMessage(B_QUIT_REQUESTED);
			break;
		}
		case COLORCONTROL_CHANGED:
			if (fAutoProof->Value() != 0) fUpdate();
			break;
	}
}

status_t TColorEditor::SetValue(const char *value)
{
	fUndoValue.SetTo(value);
	rgb_color rgb;
	status_t status = get_rgb_color(value, &rgb);
	if (status == B_OK) fColorControl->SetValue(rgb);
	return status;
}

void TColorEditor::fUpdate()
{
	char str[3];
	rgb_color color = fColorControl->ValueAsColor();
	BString c("");
	sprintf(str, "%2.2X", color.blue);
	c << str;
	sprintf(str, "%2.2X", color.green);
	c << str;
	sprintf(str, "%2.2X", color.red);
	c << str;
//	sprintf(str, "%2.2X", color.alpha);
//	c << str;
	Update(c.String());
}

void TColorEditor::fUndo()
{
	Update(fUndoValue.String());
}


// TFontEditor -----------------------------------------------------

TFontEditor::TFontEditor(propeditor_info_t *peInfo)
	:	TMultiPropEditor(peInfo, BRect(200, 100, 480, 195), "Font",
						B_FLOATING_WINDOW_LOOK, B_MODAL_APP_WINDOW_FEEL,
						B_NOT_RESIZABLE | B_NOT_ZOOMABLE)	
{
	BView *bg = new BView(Bounds(), "bgView", 
		B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP_BOTTOM,
		B_WILL_DRAW | B_FRAME_EVENTS);
	bg->SetViewColor(216, 216, 216, 0);
	AddChild(bg);
	
	fStyleItem = NULL;
	fFamilyItem = NULL;
	fFontMenu = new BPopUpMenu("");
	font_family family;
	font_style style;
	BMenuItem *first = NULL;
	
	for(int32 i=0; i < count_font_families(); i++) {
		get_font_family(i, &family);
		BMenu *styleMenu = new BMenu(family);
		for(int32 j=0; j < count_font_styles(family); j++) {
			get_font_style(family, j, &style);
			BMessage *msg = new BMessage(FONT_CHANGED);
			msg->AddString("font_family", family);
			msg->AddString("font_style", style);
			BMenuItem *styleItem = new BMenuItem(style, msg);
			styleMenu->AddItem(styleItem);
			if (j==0) first = styleItem;
		}
		fFontMenu->AddItem(styleMenu);
		BMenuItem *item = fFontMenu->FindItem(family);
		BMessage *msg = new BMessage(FONT_CHANGED);
		msg->AddPointer("menu_item", first);
		item->SetMessage(msg);
	}
	
	fFontMenuField = new BMenuField(BRect(5, 5, 120, 15), "FontMenuField", "Font:", fFontMenu);
	fFontMenuField->SetDivider(35);
	bg->AddChild(fFontMenuField);
	
	fSizeEdit = new BTextControl(BRect(5, 30, 80, 50), "SizeEdit", "Size:", "9", new BMessage(FONTSIZE_CHANGED));
	fSizeEdit->SetDivider(35);
	bg->AddChild(fSizeEdit);
	
	BButton *proof = new BButton(BRect(5, 65, 65, 85), "proof", "Proof", new BMessage(PROOF_BTN_CLICKED));
	bg->AddChild(proof);
	
	fAutoProof = new BCheckBox(BRect(70, 68, 140, 85), "autoProof", "Auto proof", NULL);
	fAutoProof->SetValue(1);
	bg->AddChild(fAutoProof);
	
	BButton *cancel = new BButton(BRect(150, 65, 210, 85), "cancel", "Cancel", new BMessage(CANCEL_BTN_CLICKED));
	bg->AddChild(cancel);
	
	BButton *ok = new BButton(BRect(215, 65, 275, 85), "ok", "OK", new BMessage(OK_BTN_CLICKED));
	bg->AddChild(ok);
}

TFontEditor::~TFontEditor()
{
}

void TFontEditor::MessageReceived(BMessage *msg)
{
	switch(msg->what)	{
		case PROOF_BTN_CLICKED:
			fUpdate();
			break;
		case OK_BTN_CLICKED: {
			fUpdate();
			BMessenger msgr(this);
			msgr.SendMessage(B_QUIT_REQUESTED);
			break;
		}
		case CANCEL_BTN_CLICKED: {
			fUndo();
			BMessenger msgr(this);
			msgr.SendMessage(B_QUIT_REQUESTED);
			break;
		}
		case FONT_CHANGED: {
			void *p, *p1;
			const char *family, *style;
			BMenuItem *source, *parent;
			if ((msg->FindPointer("source", &p)==B_OK) &&
			    (msg->FindString("font_family", &family)==B_OK) &&
			    (msg->FindString("font_style", &style)==B_OK)) {
				source = reinterpret_cast<BMenuItem *>(p);
				parent = fFontMenu->FindItem(family);
			} else if ((msg->FindPointer("menu_item", &p)==B_OK) &&
			           (msg->FindPointer("source", &p1)==B_OK)) {
				source = reinterpret_cast<BMenuItem *>(p);
				parent = reinterpret_cast<BMenuItem *>(p1);
			} else {
				return;
			}
			if (fStyleItem != NULL) fStyleItem->SetMarked(false);
			if (fFamilyItem != NULL) fFamilyItem->SetMarked(false);
			source->SetMarked(true);
			parent->SetMarked(true);
			fStyleItem = source;
			fFamilyItem = parent;
			if (fAutoProof->Value() != 0) fUpdate();
			break;
		}
		case FONTSIZE_CHANGED:
			fUpdateSizeEdit();
			if (fAutoProof->Value() != 0) fUpdate();
			break;
	}
}

status_t TFontEditor::SetValue(const char *value)
{
	TStringEx family, style, size;
	fUndoValue.SetTo(value);
	TStrTok strTok(value, ",", 0);
	if (strTok.CountTokens() != 3) {
		font_family _family;
		font_style _style;
		be_plain_font->GetFamilyAndStyle(&_family, &_style);
		family.SetTo(_family);
		style.SetTo(_style);
		size << be_plain_font->Size();
	} else {
		family.SetTo(strTok.Token(0));
		style.SetTo(strTok.Token(1));
		size.SetTo(strTok.Token(2));
		family.Trim();
		style.Trim();
		size.Trim();
	}
	
	fFamilyItem = fFontMenu->FindItem(family.String());
	if (fFamilyItem != NULL) {
		fStyleItem = fFamilyItem->Submenu()->FindItem(style.String());
		if (fStyleItem != NULL) {
			fFamilyItem->SetMarked(true);
			fStyleItem->SetMarked(true);
			fSizeEdit->SetText(size.String());
		}
	}
	return B_OK;
}

void TFontEditor::fUpdate()
{
	BString value;
	fUpdateSizeEdit();
	value << fFamilyItem->Label() << "," << fStyleItem->Label() << "," << fSizeEdit->Text();
	Update(value.String());
}

void TFontEditor::fUndo()
{
	Update(fUndoValue.String());
}

void TFontEditor::fUpdateSizeEdit()
{
	BString value;
	float size = atof(fSizeEdit->Text());
	if (size < 0) size = 0;
	value << size;
	fSizeEdit->SetText(value.String());
	fSizeEdit->TextView()->SelectAll();
}


// TMenuEditor -----------------------------------------------------


// TMenuStringItem

TMenuStringItem::TMenuStringItem(const char *text, BMenuItem *menuItem, uint32 level, bool expanded)
	:	BStringItem(text, level, expanded)
{
	fMenuItem = menuItem;
	Bold = false;
}

TMenuStringItem::~TMenuStringItem()
{
}

void TMenuStringItem::DrawItem(BView *owner, BRect frame, bool complete)
{
	BFont afont;
	owner->PushState();
	owner->GetFont(&afont);
	font_family family;
	font_style style;
	afont.GetFamilyAndStyle(&family, &style);
	afont.SetFamilyAndStyle(family, (Bold) ? "Bold" : "Regular");
	owner->SetFont(&afont, B_FONT_FAMILY_AND_STYLE);
	BStringItem::DrawItem(owner, frame, complete);
	owner->PopState();
}

BMenuItem *TMenuStringItem::MenuItem() const
{
	return fMenuItem;
}

// TMenuListView

TMenuListView::TMenuListView(BRect rect, TMenuEditor *owner)
	:	BOutlineListView(rect, "OutlineListView", B_SINGLE_SELECTION_LIST, B_FOLLOW_ALL_SIDES)
{
	fOwner = owner;
	fLastDrawedIndex = -1;
}

TMenuListView::~TMenuListView()
{
}

TMenuStringItem *TMenuListView::Parent(int32 index) const
{
	BListItem *item = ItemAt(index);
	if (item == NULL) return NULL;
	TMenuStringItem *uitem;
	
	while(true) {
		index--;
		uitem = (TMenuStringItem *)ItemAt(index);
		if (uitem == NULL) return NULL;
		if (uitem->OutlineLevel() > item->OutlineLevel()) return NULL;
		if (uitem->OutlineLevel() < item->OutlineLevel()) {
			return uitem;
		}
	}
}

bool TMenuListView::IsParent(int32 index, TMenuStringItem *parent) const
{
	while(true) {
		TMenuStringItem *item = Parent(index);
		if (item == NULL) return false;
		if (item == parent) return true;
		index = IndexOf(item);
	}
}

void TMenuListView::MessageReceived(BMessage *msg)
{
	if (msg->WasDropped() && (msg->what == MENU_DRAG)) {
		Invalidate();
		TMenuStringItem *destItem = (TMenuStringItem *)ItemAt(fLastDrawedIndex);
		BMenuItem *srcMenuItem = fDragSource->MenuItem();
		if (destItem != NULL) {
			if (destItem == fDragSource) return;
			BMenuItem *destMenuItem = destItem->MenuItem();
			BMenu *destMenu = destMenuItem->Menu();
			int32 index = destMenu->IndexOf(destMenuItem);
			srcMenuItem->Menu()->RemoveItem(srcMenuItem);
			destMenu->AddItem(srcMenuItem, index);
		} else {
			BMenu *destMenu = (BMenu *)((TMenuEditor *)Window())->Object();
			srcMenuItem->Menu()->RemoveItem(srcMenuItem);
			destMenu->AddItem(srcMenuItem);
		}
		((TMenuEditor *)Window())->Changed();
	}
}

enum {
	INSERT_ITEM,
	INSERT_SEPARATOR,
	DELETE,
	INSERT_SUBMENU,
	ADD_ITEM,
	ADD_SEPARATOR,
	ADD_SUBMENU
};

void TMenuListView::MouseDown(BPoint point)
{
	BOutlineListView::MouseDown(point);
	
	int32 buttons;
	this->Window()->CurrentMessage()->FindInt32("buttons", &buttons);
	if ((buttons & B_SECONDARY_MOUSE_BUTTON)!= 0) {
		BPopUpMenu *popupMenu = new BPopUpMenu("popupMenu");
		popupMenu->AddItem(new BMenuItem("Insert Item", new BMessage(INSERT_ITEM)));
		popupMenu->AddItem(new BMenuItem("Insert Separator", new BMessage(INSERT_SEPARATOR)));
		popupMenu->AddItem(new BMenuItem("Delete", new BMessage(DELETE)));
		popupMenu->AddSeparatorItem();
		popupMenu->AddItem(new BMenuItem("Insert Submenu", new BMessage(INSERT_SUBMENU)));
		TMenuStringItem *item = (TMenuStringItem *)ItemAt(CurrentSelection());
		if (item!=NULL)
		if (item->MenuItem()->Submenu() != NULL) {
			popupMenu->AddSeparatorItem();
			popupMenu->AddItem(new BMenuItem("Add Item", new BMessage(ADD_ITEM)));
			popupMenu->AddItem(new BMenuItem("Add Separator", new BMessage(ADD_SEPARATOR)));
			popupMenu->AddItem(new BMenuItem("Add Submenu", new BMessage(ADD_SUBMENU)));
		}
		
		ConvertToScreen(&point);
		BMenuItem *selected = popupMenu->Go(point, false, true, BRect(point.x, point.y, point.x, point.y));
		if (selected != NULL) {
			switch(selected->Command()) {
				case INSERT_ITEM: fOwner->Insert(); break;
				case INSERT_SEPARATOR: fOwner->InsertSeparator(); break;
				case DELETE: fOwner->Delete(); break;
				case INSERT_SUBMENU: fOwner->InsertSubmenu(); break;
				case ADD_ITEM: fOwner->AddItem(item); break;
				case ADD_SEPARATOR: fOwner->AddSeparator(item); break;
				case ADD_SUBMENU: fOwner->AddSubmenu(item); break;
			}
		}
	}
	
}

void TMenuListView::MouseMoved(BPoint point, uint32 transit, const BMessage *msg)
{
	if (!msg || msg->what != MENU_DRAG) {
		BOutlineListView::MouseMoved(point, transit, msg);
		return;
	}
	
	int32 index = IndexOf(point);
	if (index < 0) index = CountItems();
	switch(transit) {
		case B_EXITED_VIEW:
		case B_OUTSIDE_VIEW:
			Invalidate();
			break;
		case B_INSIDE_VIEW:
		case B_ENTERED_VIEW:
			if (!IsParent(index, fDragSource)) DrawUnderline(index);
			break;
	}
}

void TMenuListView::DrawUnderline(int32 index)
{
	if (fLastDrawedIndex != index) {
		InvalidateItem(fLastDrawedIndex);
		fLastDrawedIndex = index;
	}
	/*
	PushState();
	SetDrawingMode(B_OP_COPY);
	BRect rect = ItemFrame(index);
	rect.top = rect.bottom - 1;
	SetHighColor(216, 0, 0);
	FillRect(rect);
	PopState();
	*/
}

bool TMenuListView::InitiateDrag(BPoint point, int32 index, bool wasSelected)
{
	BMessage msg(MENU_DRAG);
	fDragSource = (TMenuStringItem *)ItemAt(index);
	BRect rect = ItemFrame(index);
	DragMessage(&msg, rect);
	return true;
}

// TMenuEditor

TMenuEditor::TMenuEditor(propeditor_info_t *peInfo)
	:	TSinglePropEditor(peInfo, BRect(260, 150, 460, 470), "Menu", B_DOCUMENT_WINDOW, 0)
{
	fView = new BView(Bounds(), "View", B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FRAME_EVENTS);
	fView->SetViewColor(216, 216, 216);
	this->AddChild(fView);
	
	BRect rect = fView->Bounds();
	rect.right -= B_V_SCROLL_BAR_WIDTH;
	rect.bottom -= B_H_SCROLL_BAR_HEIGHT;
	fMenuListView = new TMenuListView(rect, this);
	BFont font;
	fMenuListView->GetFont(&font);
	font.SetSize(14);
	fMenuListView->SetFont(&font, B_FONT_SIZE);
	fMenuListView->SetSelectionMessage(new BMessage(SELECTION_CHANGED));
	fScrollView = new BScrollView("ScrollView", fMenuListView,
									B_FOLLOW_ALL_SIDES,
									B_WILL_DRAW | B_FRAME_EVENTS, true, true);
	fView->AddChild(fScrollView);
}

TMenuEditor::~TMenuEditor()
{
}

void TMenuEditor::Changed()
{
	Clear();
	BMenu *menu = (BMenu *)Object();
	if ((menu->CountItems() > 0) && (menu != NULL)) BuildOutlineListView(menu, 0);
}

void TMenuEditor::Update()
{
	BMenuItem *menuItem = fSelectedStringItem->MenuItem();
	BSeparatorItem *sepItem = dynamic_cast<BSeparatorItem *>(menuItem);
	fSelectedStringItem->SetText((sepItem == NULL) ? menuItem->Label() : "--- separator ---");
	fMenuListView->Invalidate();
}

void TMenuEditor::Clear()
{
	fMenuListView->MakeEmpty();
}

void TMenuEditor::BuildOutlineListView(BMenu *menu, uint32 level)
{
	for(int32 i=0; i < menu->CountItems(); i++) {
		BMenuItem *menuItem = menu->ItemAt(i);
		TMenuStringItem *stItem;
		BSeparatorItem *sepItem = dynamic_cast<BSeparatorItem *>(menuItem);
		if (sepItem == NULL) {
			stItem = new TMenuStringItem(menuItem->Label(), menuItem, level);
			if (menuItem->Submenu() != NULL) {
				stItem->Bold = true;
			}
		} else {
			stItem = new TMenuStringItem("--- separator ---", sepItem, level);
		}
		fMenuListView->AddItem(stItem);
		BMenu *subMenu = menuItem->Submenu();
		if (subMenu !=NULL) {
			level++;
			BuildOutlineListView(subMenu, level);
			level--;
		}
	}
}

void TMenuEditor::MessageReceived(BMessage *msg)
{
	switch(msg->what){
		case SELECTION_CHANGED: {
			int32 index;
			if (msg->FindInt32("index", &index) == B_OK) {
				if (index >= 0) {
					fSelectedStringItem = (TMenuStringItem *)fMenuListView->ItemAt(index);
					if (fSelectedStringItem == NULL) break;
					BMenuItem *menuItem = fSelectedStringItem->MenuItem();
					if (menuItem->Submenu() == NULL) {
						this->SetObjectInspector(menuItem);
					} else {
						this->SetObjectInspector(menuItem->Submenu());
					}
				} else {
					this->SetObjectInspector(NULL);
				}
			}
			break;
		}
	}
}

int32 TMenuEditor::GetCurrentSelectionMenu(BMenuItem **item, BMenu **menu)
{
	int32 index = fMenuListView->CurrentSelection();
	TMenuStringItem *sItem = (TMenuStringItem *)fMenuListView->ItemAt(index);
	if (sItem == NULL) return -1;
	*item = sItem->MenuItem();
	if (*item == NULL) return -1;
	*menu = (*item)->Menu();
	return index;
}

void TMenuEditor::Insert()
{
	BMenuItem *item;
	BMenu *menu;
	int32 index = GetCurrentSelectionMenu(&item, &menu);
	BMenuItem *newitem = new BMenuItem("New BMenuItem", NULL);
	if (index >= 0) {
		menu->AddItem(newitem, menu->IndexOf(item));
	} else {
		menu = (BMenu *)Object();	
		menu->AddItem(newitem);
	}
	Changed();
}

void TMenuEditor::InsertSeparator()
{
	BMenuItem *item;
	BMenu *menu;
	int32 index = GetCurrentSelectionMenu(&item, &menu);
	BSeparatorItem *newitem = new BSeparatorItem();
	if (index >= 0) {
		menu->AddItem(newitem, menu->IndexOf(item));
//		TMenuStringItem *sitem = (TMenuStringItem *)fMenuListView->ItemAt(index);
//		fMenuListView->AddItem(new TMenuStringItem("New BMenuItem", newitem, sitem->OutlineLevel()), index);
	} else {
		menu = (BMenu *)Object();	
		menu->AddItem(newitem);
//		fMenuListView->AddItem(new TMenuStringItem("New BMenuItem", newitem));
	}
	Changed();
}

void TMenuEditor::Delete()
{
	BMenuItem *item;
	BMenu *menu;
	int32 index = GetCurrentSelectionMenu(&item, &menu);
	if (index >= 0) {
		menu->RemoveItem(menu->IndexOf(item));
		Changed();
	}
}

void TMenuEditor::InsertSubmenu()
{
	BMenuItem *item;
	BMenu *menu;
	int32 index = GetCurrentSelectionMenu(&item, &menu);
	BMenu *newMenu = new BMenu("New BMenu");
	if (index >= 0) {
		menu->AddItem(newMenu, menu->IndexOf(item));
	} else {
		menu = (BMenu *)Object();	
		menu->AddItem(newMenu);
	}
	Changed();
}

void TMenuEditor::AddItem(TMenuStringItem *item)
{
	BMenu *menu = item->MenuItem()->Submenu();
	BMenuItem *newitem = new BMenuItem("New BMenuItem", NULL);
	menu->AddItem(newitem);
	Changed();
}

void TMenuEditor::AddSeparator(TMenuStringItem *item)
{
	BMenu *menu = item->MenuItem()->Submenu();
	BSeparatorItem *newitem = new BSeparatorItem();
	menu->AddItem(newitem);
	Changed();
}

void TMenuEditor::AddSubmenu(TMenuStringItem *item)
{
	BMenu *menu = item->MenuItem()->Submenu();
	BMenu *newitem = new BMenu("New BMenu");
	menu->AddItem(newitem);
	Changed();
}


// TTabEditor ---------------------------------------------------------------------

TTabListView::TTabListView(BRect rect)
	:	BListView(rect, "ListView")
{
	fLastDrawedIndex = -1;
}

TTabListView::~TTabListView()
{
}

void TTabListView::MessageReceived(BMessage *msg)
{
	if (msg->WasDropped() && (msg->what == MENU_DRAG)) {
		Invalidate();
		((TTabEditor *)Window())->Move(IndexOf(fDragSource), fLastDrawedIndex);
	}
}

void TTabListView::MouseMoved(BPoint point, uint32 transit, const BMessage *msg)
{
	if (!msg || msg->what != MENU_DRAG) {
		BListView::MouseMoved(point, transit, msg);
		return;
	}
	
	int32 index = IndexOf(point);
	if (index < 0) index = CountItems();
	switch(transit) {
		case B_EXITED_VIEW:
		case B_OUTSIDE_VIEW:
			Invalidate();
			break;
		case B_INSIDE_VIEW:
		case B_ENTERED_VIEW:
			DrawUnderline(index);
			break;
	}
}

void TTabListView::DrawUnderline(int32 index)
{
	if (fLastDrawedIndex != index) {
		InvalidateItem(fLastDrawedIndex);
		fLastDrawedIndex = index;
	}
	PushState();
	SetDrawingMode(B_OP_COPY);
	BRect rect = ItemFrame(index);
	rect.top = rect.bottom - 1;
	SetHighColor(216, 0, 0);
	FillRect(rect);
	PopState();
}

bool TTabListView::InitiateDrag(BPoint point, int32 index, bool wasSelected)
{
	BMessage msg(MENU_DRAG);
	fDragSource = (BStringItem *)ItemAt(index);
	BRect rect = ItemFrame(index);
	DragMessage(&msg, rect);
	return true;
}

TTabEditor::TTabEditor(propeditor_info_t *peInfo)
	:	TSinglePropEditor(peInfo, BRect(260, 150, 530, 300), "Tab", B_TITLED_WINDOW, B_NOT_RESIZABLE | B_NOT_ZOOMABLE)
{
	BView *bgView = new BView(Bounds(), "bgView", B_FOLLOW_ALL, B_WILL_DRAW | B_FRAME_EVENTS);
	bgView->SetViewColor(216, 216, 216);
	this->AddChild(bgView);
	
	fListView = new TTabListView(BRect(7, 7, 190, 117));
	fListView->SetSelectionMessage(new BMessage(SELECTION_CHANGED));
	BScrollView *scrollView = new BScrollView("scrollView", fListView,
						B_FOLLOW_LEFT | B_FOLLOW_TOP,
						B_WILL_DRAW | B_FRAME_EVENTS,
						false, true);
	
	bgView->AddChild(scrollView);

	fAddButton = new BButton(BRect(215, 10, 263, 20), "Add", "Add", new BMessage(ADD_CLICKED));
	bgView->AddChild(fAddButton);
	fDeleteButton = new BButton(BRect(215, 40, 263, 50), "Delete", "Delete", new BMessage(DELETE_CLICKED));
	bgView->AddChild(fDeleteButton);
	
	fLabel = new BTextControl(BRect(7, 125, 205, 135), "Label", "Label", "", new BMessage(LABEL_CHANGED));
	fLabel->SetDivider(30);
	bgView->AddChild(fLabel);
}

TTabEditor::~TTabEditor()
{
}

void TTabEditor::Changed()
{
	Clear();
	BuildListView();
	
	BTabView *tabView = (BTabView *)Object();
	if (tabView == NULL) return;
	for(int32 i=0; i < tabView->CountTabs(); i++) {
		BTab *tab = tabView->TabAt(i);
		LockView(tab->View());
	}
}

void TTabEditor::BuildListView()
{
	BTabView *tabView = (BTabView *)Object();
	if (tabView == NULL) return;
	
	tabView->Window()->Lock();
	for(int32 i=0; i < tabView->CountTabs(); i++) {
		BTab *tab = tabView->TabAt(i);
		BStringItem *item = new BStringItem(tab->Label());
		fListView->AddItem(item);
	}
	fListView->Select(tabView->Selection());
	tabView->Window()->Unlock();
}

void TTabEditor::Update()
{
}

void TTabEditor::MessageReceived(BMessage *msg)
{
	switch(msg->what){
		case SELECTION_CHANGED:
			SelectionChanged();
			break;
		case ADD_CLICKED:
			AddClicked();
			break;
		case DELETE_CLICKED:
			DeleteClicked();
			break;
		case LABEL_CHANGED:
			LabelChanged();
			break;
	}
}

void TTabEditor::Move(int32 srcIndex, int32 destIndex)
{
	if (srcIndex == destIndex) return;
	
	BMessage srcTabArc, srcViewArc;
	BMessage *bakTabArc, *bakViewArc;
	BTab *srcTab;
	BView *srcView;
	BList bakTabList, bakViewList;
	
	BTabView *tabView = (BTabView *)Object();
	tabView->Window()->Lock();
	
	srcTab = tabView->TabAt(srcIndex);
	srcView = srcTab->View();
	srcTab->Archive(&srcTabArc);
	srcView->Archive(&srcViewArc);
	tabView->RemoveTab(srcIndex);
	
	for(int32 i = destIndex; i < tabView->CountTabs(); ) {
		bakTabArc = new BMessage();
		bakViewArc = new BMessage();
		BTab *bakTab = tabView->TabAt(i);
		BView *bakView = bakTab->View();
		bakTab->Archive(bakTabArc);
		bakView->Archive(bakViewArc);
		bakTabList.AddItem(bakTabArc);
		bakViewList.AddItem(bakViewArc);
		UnlockView(bakView);
		tabView->RemoveTab(i);
	}
	
	tabView->AddTab(new BView(&srcViewArc), new BTab(&srcTabArc));
	for(int32 i=0; i < bakTabList.CountItems(); i++) {
		bakTabArc = (BMessage *)bakTabList.ItemAt(i);
		bakViewArc = (BMessage *)bakViewList.ItemAt(i);
		tabView->AddTab(new BView(bakViewArc), new BTab(bakTabArc));
		delete bakTabArc;
		delete bakViewArc;
	}
	
	tabView->Select(destIndex);
	tabView->Invalidate();
	tabView->Window()->Unlock();
	Changed();
}

void TTabEditor::Clear()
{
	fListView->MakeEmpty();
	fLabel->SetText("");
}

void TTabEditor::SelectionChanged()
{
	int32 idx = fListView->CurrentSelection();
	if (idx < 0) {
		fLabel->SetText("");
		fCurrentSelection = -1;
		return;
	}
	
	BTabView *tabView = (BTabView *)Object();
	tabView->Window()->Lock();
	BTab *tab = tabView->TabAt(idx);
	fLabel->SetText(tab->Label());
	tabView->Select(idx);
	BView *parentView = tab->View()->Parent();
	tab->View()->ResizeTo(parentView->Frame().Width(), parentView->Frame().Height());
	tab->View()->MoveTo(0, 0);
	tabView->Invalidate();
	tabView->Window()->Unlock();
	
	fCurrentSelection = idx;
}

void TTabEditor::AddClicked()
{
	BTabView *tabView = (BTabView *)Object();
	tabView->Window()->Lock();
	BRect aRect = tabView->Bounds();
	aRect.bottom -= tabView->TabHeight();
	BView *view;
	view = new BView(aRect, "New Tab", B_FOLLOW_ALL, B_WILL_DRAW | B_FRAME_EVENTS);
	view->SetViewColor(216, 216, 216);
	tabView->AddTab(view);
	tabView->Window()->Unlock();
	Changed();
	fListView->Select(fListView->CountItems() - 1);
}

void TTabEditor::DeleteClicked()
{
	int32 idx = fListView->CurrentSelection();
	if (idx < 0) {
		return;
	}
	
	BTabView *tabView = (BTabView *)Object();
	tabView->Window()->Lock();
	BTab *tab = tabView->TabAt(idx);
	UnlockView(tab->View());
	tabView->RemoveTab(idx);
	tabView->Window()->Unlock();
	Changed();
}

void TTabEditor::LabelChanged()
{
	int32 idx = fListView->CurrentSelection();
	if (fCurrentSelection != idx) idx = fCurrentSelection;
	if (idx < 0) {
		fLabel->SetText("");
		return;
	}
	
	Looper()->CurrentMessage()->PrintToStream();
	
	BTabView *tabView = (BTabView *)Object();
	BTab *tab = tabView->TabAt(idx);
	tab->SetLabel(fLabel->Text());
	BStringItem *item = (BStringItem *)fListView->ItemAt(idx);
	item->SetText(tab->Label());
	fListView->InvalidateItem(idx);
	tabView->Window()->Lock();
	tabView->Invalidate();
	tabView->Window()->Unlock();
	fLabel->TextView()->SelectAll();
}


// TTextViewEditor -----------------------------------------------------

TTextEditor::TTextEditor(BRect frame, const char *name, BRect textRect,
						uint32 resizingMode, uint32 flags)
	:	BTextView(frame, name, textRect, resizingMode, flags)
{
	fTargetTextView = NULL;
}

void TTextEditor::FrameResized(float width, float height)
{
	if (this->DoesWordWrap()) {
		BRect rect(this->TextRect());
		rect.right = width;
		this->SetTextRect(rect);
	}
	this->BTextView::FrameResized(width, height);
}

void TTextEditor::SetTargetTextView(BTextView *target)
{
	fTargetTextView = target;
	
	fTargetTextView->Window()->Lock();
	this->SetText(fTargetTextView->Text());
	this->SetAlignment(fTargetTextView->Alignment());
	fTargetTextView->Window()->Unlock();
}

void TTextEditor::InsertText(const char *text, int32 length, int32 offset, const text_run_array *runs)
{
	if (fTargetTextView != NULL) {
		fTargetTextView->Window()->Lock();
		fTargetTextView->Insert(offset, text, length, runs);
		fTargetTextView->Window()->Unlock();
	}
	BTextView::InsertText(text, length, offset, runs);
}

void TTextEditor::DeleteText(int32 start, int32 finish)
{
	if (fTargetTextView != NULL) {
		fTargetTextView->Window()->Lock();
		fTargetTextView->Delete(start, finish);
		fTargetTextView->Window()->Unlock();
	}
	BTextView::DeleteText(start, finish);
}


TTextViewEditor::TTextViewEditor(propeditor_info_t *peInfo)
	:	TSinglePropEditor(peInfo, BRect(260, 150, 530, 300), "Text", B_DOCUMENT_WINDOW, 0)
{
	BRect rect = Bounds();
	rect.right -= B_V_SCROLL_BAR_WIDTH;
	rect.bottom -= B_H_SCROLL_BAR_HEIGHT;
	fTextEditor = new TTextEditor(rect, "TextEditor", Bounds(), B_FOLLOW_ALL, B_WILL_DRAW | B_FRAME_EVENTS);
	fScrollView = new BScrollView("ScrollView", fTextEditor, B_FOLLOW_ALL, B_WILL_DRAW, true, true);
	this->AddChild(fScrollView);
}

TTextViewEditor::~TTextViewEditor()
{
}

void TTextViewEditor::Changed()
{
	BTextView *textView = (BTextView *)Object();
	fTextEditor->SetTargetTextView(textView);
}

void TTextViewEditor::Update()
{
}
