#include "PropEditor.h"


TPropEditor::TPropEditor(propeditor_info_t *peInfo,
				BRect frame, const char *title,
				window_type type, uint32 flags,
				uint32 workspaces)
	:	BWindow(frame, title, type, flags, workspaces)
{
	fPeInfo = *peInfo;
}

TPropEditor::TPropEditor(propeditor_info_t *peInfo,
				BRect frame, const char *title,
				window_look look, window_feel feel, uint32 flags,
				uint32 workspaces)
	:	BWindow(frame, title, look, feel, flags, workspaces)
{
	fPeInfo = *peInfo;
}

TPropEditor::~TPropEditor()
{
}

const propeditor_info_t *TPropEditor::PeInfo() const
{
	return &fPeInfo;
}

void TPropEditor::LockView(BView *view)
{
	fPeInfo.locked_list->AddView(view);
}

void TPropEditor::UnlockView(BView *view)
{
	fPeInfo.locked_list->RemoveView(view);
}


// TSPEMessageFilter ---------------------------------------------------------

TSPEMessageFilter::TSPEMessageFilter(TSinglePropEditor *owner)
	:	BMessageFilter(B_ANY_DELIVERY, B_ANY_SOURCE)
{
	fOwner = owner;
}

TSPEMessageFilter::~TSPEMessageFilter()
{
}

filter_result TSPEMessageFilter::Filter(BMessage *msg, BHandler **target)
{
	switch(msg->what) {
		case BERRY_VIEW_INVALIDATE: {
			fOwner->Update();
			return B_SKIP_MESSAGE;
		}
	}
	return B_DISPATCH_MESSAGE;
}


// TSinglePropEditor ---------------------------------------------------------

TSinglePropEditor::TSinglePropEditor(propeditor_info_t *peInfo,
				BRect frame, const char *title,
				window_type type, uint32 flags,
				uint32 workspaces)
	:	TPropEditor(peInfo, frame, title, type, flags, workspaces)
{
	fInitialization();
}

TSinglePropEditor::TSinglePropEditor(propeditor_info_t *peInfo,
				BRect frame, const char *title,
				window_look look, window_feel feel, uint32 flags,
				uint32 workspaces)
	:	TPropEditor(peInfo, frame, title, look, feel, flags, workspaces)
{
	fInitialization();
}

TSinglePropEditor::~TSinglePropEditor()
{
}

void TSinglePropEditor::fInitialization()
{
	AddFilter(new TSPEMessageFilter(this));
}

BArchivable *TSinglePropEditor::Object()
{
	return fObject;
}

void TSinglePropEditor::SetObject(BArchivable *object)
{
	fObject = object;
	Changed();
}

void TSinglePropEditor::SetObjectInspector(BArchivable *object) const
{
	if (fMessenger.IsValid()) {
		BMessage msg(BERRY_OBJECT_SELECTED);
		msg.AddPointer("object", object);
		BMessenger msgr(this);
		msg.AddMessenger("messenger", msgr);
		fMessenger.SendMessage(&msg);
	}
}


// TMultiPropEditor ---------------------------------------------------------

TMultiPropEditor::TMultiPropEditor(propeditor_info_t *peInfo,
				BRect frame, const char *title,
				window_type type, uint32 flags,
				uint32 workspaces)
	:	TPropEditor(peInfo, frame, title, type, flags, workspaces)
{
}

TMultiPropEditor::TMultiPropEditor(propeditor_info_t *peInfo,
				BRect frame, const char *title,
				window_look look, window_feel feel, uint32 flags,
				uint32 workspaces)
	:	TPropEditor(peInfo, frame, title, look, feel, flags, workspaces)
{
}

TMultiPropEditor::~TMultiPropEditor()
{
}

status_t TMultiPropEditor::Update(const char *value)
{
	return PeInfo()->updaterfunc(value);
}
