#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include "BCL.h"


// TBerryComponentLibrary

TBerryComponentLibrary::TBerryComponentLibrary(instantiation_info_t *info)
{
	fInfo = *info;
	fControlStyle = 0;
}

TBerryComponentLibrary::~TBerryComponentLibrary()
{
}

void TBerryComponentLibrary::PropertyList(TPropertyRegistrar *propertyRegistrar) const
{
}

status_t TBerryComponentLibrary::SetValue(BArchivable *view, const char *propName, const char *value)
{
	return B_NAME_NOT_FOUND;
}

status_t TBerryComponentLibrary::GetValue(BArchivable *view, const char *propName, BString *value)
{
	return B_NAME_NOT_FOUND;
}

status_t TBerryComponentLibrary::GetDefaultSize(float *width, float *height)
{
	return B_ERROR;
}

status_t TBerryComponentLibrary::GetSuperObject(BArchivable *object, const char *propName, BArchivable **superObject)
{
	*superObject = NULL;
	return B_ERROR;
}

image_id TBerryComponentLibrary::Image() const
{
	return fInfo.id;
}

uint32 TBerryComponentLibrary::ControlStyle() const
{
	return fControlStyle;
}

void TBerryComponentLibrary::SetControlStyle(uint32 style)
{
	fControlStyle = style;
}

// TPropertyRegistrar

TPropertyRegistrar::TPropertyRegistrar(bool multi)
{
	fLevel = 0;
	fMulti = multi;
}

TPropertyRegistrar::~TPropertyRegistrar()
{
}

void TPropertyRegistrar::AddProperty(const TBerryComponentLibrary *bcl,
							const char *propertyName,
							type_code type,
							const type_info *info,
							const selectable_values *selectableValues)
{
	TPropertyItem *item;
	item = (TPropertyItem *)malloc(sizeof(TPropertyItem));
	item->Name = propertyName;
	item->BCL = bcl;
	fBCL = bcl;
	item->SuperItem = false;
	item->Level = fLevel;
	item->TypeCode = type;
	item->TypeInfo = info;
	item->SelectableValues = selectableValues;
	fPropList->AddItem(item);
}

void TPropertyRegistrar::AddObject(const TBerryComponentLibrary *bcl,
							const char *superName,
							TBerryComponentLibrary *subbcl)
{
	if (fMulti) return;
	TPropertyItem *item;
	item = (TPropertyItem *)malloc(sizeof(TPropertyItem));
	item->Name = superName;
	item->SuperItem = true;
	item->BCL = bcl;
	item->Level = fLevel;
	fPropList->AddItem(item);
	fLevel++;
	subbcl->PropertyList(this);
	fLevel--;
}


// TStrTok

TStrTok::TStrTok(const char *string, const char *delim, int flags)
{
	bool countNullstr = (flags & TOKEN_COUNT_NULLSTR) > 0;
	bool singleDelim = (flags & TOKEN_SINGLE_DELIM) > 0;
	
	const char *str = string;
	const char *strd;
	int pitch;
	
	if (singleDelim) {
		pitch = 1;
	} else {
		pitch = strlen(delim);
	}
	
	for(;;) {
		if (singleDelim) {
			strd = strpbrk(str, delim);
		} else {
			strd = strstr(str, delim);
		}
		
		if (str == strd) {	// delim で挟まれている(文字長 0)
			if (countNullstr) fList.AddItem(new BString(""));
		} else {
			if (strd == NULL) {	// 文字列の終端に達した
				if (strlen(str)==0) {
					if (countNullstr) fList.AddItem(new BString(""));
				} else {
					fList.AddItem(new BString(str));
				}
				break;
			} else {
				fList.AddItem(new BString(str, strd - str));
			}
		}
		str = strd + pitch;
	}		
}

const char *TStrTok::Token(int index)
{
	BString *str = (BString *)fList.ItemAt(index);
	if (str!=NULL) return str->String();
	return NULL;
}


// TStringEx

TStringEx::TStringEx()
	:	BString()
{
}

TStringEx::TStringEx(const char *string)
	:	BString(string)
{
}

TStringEx::TStringEx(const char *string, int32 maxLength)
	:	BString(string, maxLength)
{
}

TStringEx::TStringEx(const BString &string)
	:	BString(string)
{
}

TStringEx::~TStringEx()
{
}

void TStringEx::TrimLeft()
{
	const char *str = this->String();
	int len = this->Length();
	int i;
	for(i=0; str[i]==' ' && i<len; i++);
	if (i<=0) return;
	
	char *blk = (char *)malloc(i+1);
	blk[i] = 0;
	for(; i>0; i--) blk[i-1]=' ';
	
	this->RemoveFirst(blk);
	free(blk);
}

void TStringEx::TrimRight()
{
	const char *str = this->String();
	int len = this->Length();
	int i, j = 0;
	for(i=len-1; str[i]==' ' && i>=0; i--) j++;
	if (j<=0) return;
	
	char *blk = (char *)malloc(j+1);
	blk[j] = 0;
	for(; j>0; j--) blk[j-1]=' ';
	
	this->RemoveLast(blk);
	free(blk);
}

void TStringEx::Trim()
{
	TrimLeft();
	TrimRight();
}


//

bool hex_to_int(int32 *result, const char *hex)
{
	int i, a = 1;
	char h;
	*result = 0;
	for(i = strlen(hex)-1; i >= 0; i--){
		if (isxdigit(hex[i]) != 0) {
			h = toupper(hex[i]);
			switch (h) {
				case '0': break;
				case '1': *result += 1 * a;
						break;
				case '2': *result += 2 * a;
						break;
				case '3': *result += 3 * a;
						break;
				case '4': *result += 4 * a;
						break;
				case '5': *result += 5 * a;
						break;
				case '6': *result += 6 * a;
						break;
				case '7': *result += 7 * a;
						break;
				case '8': *result += 8 * a;
						break;
				case '9': *result += 9 * a;
						break;
				case 'A': *result += 0xa * a;
						break;
				case 'B': *result += 0xb * a;
						break;
				case 'C': *result += 0xc * a;
						break;
				case 'D': *result += 0xd * a;
						break;
				case 'E': *result += 0xe * a;
						break;
				case 'F': *result += 0xf * a;
						break;
				default:  break;
			}
		} else {
			return false;
		}
		a = a * 0x10;
	}
	return true;
}

status_t get_bool(const char *s, bool *value)
{
	BString v(s);
	if (v.ICompare("TRUE")==0) { *value = true; } else
	if (v.ICompare("FALSE")==0) { *value = false; } else
		return B_ERROR;
	return B_OK;
}

status_t get_int_value(int32 ivalue, BString *value)
{
	char str[100];
	sprintf(str, "%d", (int)ivalue);
	value->SetTo(str);
	return B_OK;
}

status_t get_double_value(double ivalue, BString *value)
{
	char str[100];
	sprintf(str, "%f", ivalue);
	value->SetTo(str);
	return B_OK;
}

status_t get_rgb_color(const char *rgb_value, rgb_color *rgb)
{
	return hex_to_int((int32 *)rgb, rgb_value) ? B_OK : B_ERROR;
}

void get_rgb_color_value(rgb_color color, BString *value)
{
	char str[3];
	value->SetTo("");
	sprintf(str, "%2.2X", color.blue);
	*value << str;
	sprintf(str, "%2.2X", color.green);
	*value << str;
	sprintf(str, "%2.2X", color.red);
	*value << str;
}
