#include <Application.h>
#include <Roster.h>
#include <File.h>
#include <Resources.h>
#include "HelloWindow.h"


// 自身の実行ファイルから GuiBerry リソースを抽出して BGView に再生する
status_t GetBGView(BView **BGView)
{
	status_t status;
	
	// 実行ファイルの情報(app_info)を取得
	app_info appinfo;
	be_app->GetAppInfo(&appinfo);
	
	// 実行ファイル(appinfo.ref)をオープン
	BFile file(&appinfo.ref, B_READ_ONLY);
	status = file.InitCheck();
	if (status != B_OK) {
		return B_ERROR;
	}
	
	// file からリソースを抽出
	BResources rsrc(&file);
	size_t size;
	char *data = (char *)rsrc.LoadResource('ARCV', "HelloWorld", &size);
	if (data == NULL) {
		return B_ERROR;
	}
	
	// GuiBerry リソースは BMessage をアーカイブしたものなので
	// それを archive に展開(Unflatten)
	BMessage archive;
	status = archive.Unflatten(data);
	if (status != B_OK) {
		return B_ERROR;
	}
	
	// Unflatten されたデータを instantiate_object() に喰わせると
	// 最上位層の BView を取得できる。(それは BGView のはず)
	// instantiate_object() は、また同時にすべての階層の(子、孫、曾孫・・・の）
	// View を再生してくれる。
	// あとは、それを Window に AddChild() するだけで再現される。
	*BGView = dynamic_cast<BView *>(instantiate_object(&archive));
	if (BGView == NULL) {
		return B_ERROR;
	}
	
	return B_OK;
}

THelloWindow::THelloWindow(BRect frame)
	:	BWindow(frame, "Hello", B_TITLED_WINDOW, B_NOT_RESIZABLE | B_NOT_ZOOMABLE )
{
	// 自身の実行ファイルから GuiBerry リソースを抽出して BGView に再生する
	// ( 上の GetBGView() を参照 )
	if (GetBGView(&BGView) != B_OK) {
		return;
	}
	AddChild(BGView);
	
	// リソースから展開した中から、"HelloButton" という名前の BButton を探しだす。
	// 以後は HelloButton でアクセスできる。
	HelloButton = dynamic_cast<BButton *>(FindView("HelloButton"));
	if (HelloButton == NULL) {
		return;
	}
	
	// 同様に HelloStringView を探す
	HelloStringView = dynamic_cast<BStringView *>(FindView("HelloStringView"));
	if (HelloStringView == NULL) {
		return;
	}
	
	// 最初は HelloStringView を非表示にしておき、
	HelloStringView->Hide();
	
	// HelloButton をクリックすると HelloStringView を表示するようにする。
	// 下の MessageReceived() も参照
	HelloButton->SetMessage(new BMessage('helo'));
}

bool THelloWindow::QuitRequested()
{
	be_app_messenger.SendMessage(B_QUIT_REQUESTED);
	return true;
}

void THelloWindow::MessageReceived(BMessage *msg)
{
	switch(msg->what) {
		case 'helo':
			HelloStringView->Show();
			break;
		default: BWindow::MessageReceived(msg);
	}
}

/*
従来の、各クラスのコンストラクタで View オブジェクトを new して形成していくのと、
GuiBerry によって作られたリソースから形成するのとでは、コード自体は異なるが、
本質的には、やってることは一緒なのである。
*/
