#ifndef _BCL_H
#define _BCL_H

#include <typeinfo>
#include <image.h>
#include <Archivable.h>
#include <String.h>
#include <Errors.h>
#include <List.h>
#include <string.h>
#include <Control.h>

#include "LockedList.h"


enum {
	BERRY_VIEW_INVALIDATE = 'bviv',
	BERRY_OBJECT_SELECTED = 'bvsl',
	BERRY_OBJECT_DELETED = 'bodl',
	BERRY_CLOSE_ALL = 'bcal',
	BERRY_BCL_DROPPED = 'bbdr',
};

// ControlStyle flags
enum {
	CS_ACCEPTS_CONTROLS = 0x1,
	
};

#define _CAST(_class, object, out) 													\
	if (object == NULL) return B_ERROR;												\
	if (dynamic_cast<_class *>(object)==NULL) return B_ERROR;						\
	_class *out = (_class *)object;													\

typedef struct instantiation_info_t {
	image_id id;
	TLockedList *locked_list;
} instantiation_info_t;

typedef struct selectable_values {
	const char **SelectableStrings;
	bool FlagMode;
	const char *Separator;
} selectable_values;

bool hex_to_int(int32 *result, const char *hex);
status_t get_bool(const char *s, bool *value);
status_t get_int_value(int32 ivalue, BString *value);
status_t get_double_value(double ivalue, BString *value);
status_t get_rgb_color(const char *rgb_value, rgb_color *rgb);
void get_rgb_color_value(rgb_color color, BString *value);

class TPropertyRegistrar;
class TBerryComponentLibrary
{
public:
	TBerryComponentLibrary(instantiation_info_t *info);
	virtual ~TBerryComponentLibrary();
	virtual BArchivable *Instantiate() = 0;
	virtual const char *SupportedClass() const = 0;
	virtual void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	virtual status_t SetValue(BArchivable *view, const char *propName, const char *value);
	virtual status_t GetValue(BArchivable *view, const char *propName, BString *value);
	virtual status_t GetDefaultSize(float *width, float *height);
	virtual status_t GetSuperObject(BArchivable *object, const char *propName, BArchivable **superObject);
	image_id Image() const;
	uint32 ControlStyle() const;
	void SetControlStyle(uint32 style);
private:
	uint32 fControlStyle;
	instantiation_info_t fInfo;
};

class TPropertyRegistrar
{
public:
	TPropertyRegistrar(bool multi);
	~TPropertyRegistrar();
	void AddProperty(const TBerryComponentLibrary *bcl,
					const char *propertyName,
					type_code type,
					const type_info *info = NULL,
					const selectable_values *selectableValues = NULL);
	void AddObject(const TBerryComponentLibrary *bcl,
					const char *superName,
					TBerryComponentLibrary *subbcl);
	
private:
	friend class TObjectInspectorView;
	BList *fPropList;
	int32 fLevel;
	BList fBCLStack;
	const TBerryComponentLibrary *fBCL;
	bool fMulti;
};

typedef void register_component(const char *page, TBerryComponentLibrary *bcl);

enum {
	TOKEN_COUNT_NULLSTR = 1,
	TOKEN_SINGLE_DELIM = 2
};

class TStrTok
{
public:
	TStrTok(const char *string, const char *delim, int flags);
	const char *Token(int index);
	int CountTokens() { return fList.CountItems(); };
private:
	BList fList;
};

class TStringEx : public BString
{
public:
	TStringEx();
	TStringEx(const char *string);
	TStringEx(const char *string, int32 maxLength);
	TStringEx(const BString &string);
	~TStringEx();
	
	void TrimLeft();
	void TrimRight();
	void Trim();
};

typedef struct TPropertyItem {
	const char *Name;
	bool SuperItem;
	const TBerryComponentLibrary *BCL;
	int32 Level;
	type_code TypeCode;
	const type_info *TypeInfo;
	const selectable_values *SelectableValues;
} TPropertyItem;


#endif
