#ifndef _BECOMPONENTS_H
#define _BECOMPONENTS_H

#include <List.h>
#include <Archivable.h>
#include <Handler.h>
#include <InterfaceKit.h>
#include "BCL.h"

#define _CAST(_class, object, out) 													\
	if (object == NULL) return B_ERROR;												\
	if (dynamic_cast<_class *>(object)==NULL) return B_ERROR;						\
	_class *out = (_class *)object;													\

static const char *BOOLEAN_STRINGS[] = { "True", "False", NULL };
static const selectable_values BOOLEAN_VALUES = {
	BOOLEAN_STRINGS, false, NULL
};

static const char *BVIEW_FLAG_STRINGS[] = {
	"B_WILL_DRAW", "B_FULL_UPDATE_ON_RESIZE", "B_FRAME_EVENTS", "B_PULSE_NEEDED",
	"B_NAVIGABLE", "B_NAVIGABLE_JUMP", "B_SUBPIXEL_PRECISE", NULL
};
static const selectable_values BVIEW_FLAG_VALUES = {
	BVIEW_FLAG_STRINGS, true, " | "
};

static const char *BORDER_STRINGS[] = {
	"B_BORDER_FRAME", "B_BORDER_CONTENTS", "B_BORDER_EACH_ITEM", "B_BORDER_OFF", NULL
};
static const selectable_values BORDER_VALUES = {
	BORDER_STRINGS, false, NULL
};

static const char *BEHAVE_STRINGS[] = { "B_ONE_STATE_BUTTON", "B_TWO_STATE_BUTTON", NULL };
static const selectable_values BEHAVE_VALUES = {
	BEHAVE_STRINGS, false, NULL
};

static const char *ALIGN_STRINGS[] = {
	"B_ALIGN_LEFT", "B_ALIGN_RIGHT", "B_ALIGN_CENTER", NULL
};
static const selectable_values ALIGN_VALUES = {
	ALIGN_STRINGS, false, NULL
};

static const char *STYLE_STRINGS[] = { "B_BLOCK_THUMB", "B_TRIANGLE_THUMB", NULL };
static const selectable_values STYLE_VALUES = {
	STYLE_STRINGS, false, NULL
};

static const char *ORIENTATION_STRINGS[] = { "B_HORIZONTAL", "B_VERTICAL", NULL };
static const selectable_values ORIENTATION_VALUES = {
	ORIENTATION_STRINGS, false, NULL
};

static const char *HASH_MARKS_STRINGS[] = {
	"B_HASH_MARKS_NONE", "B_HASH_MARKS_TOP", "B_HASH_MARKS_BOTTOM", 
	"B_HASH_MARKS_BOTH", "B_HASH_MARKS_LEFT", "B_HASH_MARKS_RIGHT", NULL
};
static const selectable_values HASH_MARKS_VALUES = {
	HASH_MARKS_STRINGS, false, NULL
};

static const char *BORDER_STYLE_STRINGS[] = {
	"B_PLAIN_BORDER", "B_FANCY_BORDER", "B_NO_BORDER", NULL
};
static const selectable_values BORDER_STYLE_VALUES = {
	BORDER_STYLE_STRINGS, false, NULL
};

static const char *LIST_TYPE_STRINGS[] = {
	"B_SINGLE_SELECTION_LIST", "B_MULTIPLE_SELECTION_LIST", NULL
};
static const selectable_values LIST_TYPE_VALUES = {
	LIST_TYPE_STRINGS, false, NULL
};

static const char *TABWIDTH_STRINGS[] = { "B_WIDTH_FROM_WIDEST", "B_WIDTH_AS_USUAL", "B_WIDTH_FROM_LABEL", NULL };
static const selectable_values TABWIDTH_VALUES = {
	TABWIDTH_STRINGS, false, NULL
};

static const char *H_RESIZING_STRINGS[] = {"None", "Left", "Right", "Left&Right", "HCenter", NULL};
static const selectable_values H_RESIZING_VALUES = {
	H_RESIZING_STRINGS, false, NULL
};

static const char *V_RESIZING_STRINGS[] = {"None", "Top", "Bottom", "Top&Bottom", "VCenter", NULL};
static const selectable_values V_RESIZING_VALUES = {
	V_RESIZING_STRINGS, false, NULL
};


// BArchivable
class TBCL_BArchivable : public TBerryComponentLibrary
{
public:
	TBCL_BArchivable(instantiation_info_t *info);
	~TBCL_BArchivable();
	BArchivable *Instantiate();
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
};

// BHandler
class TBCL_BHandler : public TBCL_BArchivable
{
public:
	TBCL_BHandler(instantiation_info_t *info);
	~TBCL_BHandler();
	BArchivable *Instantiate();
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
	
private:
	status_t fSetName(BHandler *object, const char *name);
	const char *fGetName(BHandler *object);
};

// BView
class TBCL_BView : public TBCL_BHandler
{
public:
	TBCL_BView(instantiation_info_t *info);
	~TBCL_BView();
	BArchivable *Instantiate();
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
	status_t GetDefaultSize(float *width, float *height);
	
private:
	void fGetResizingMode(uint32 resizingMode, BString *value);
	status_t fSetResizingMode(BView *view, const char *value);
	void fGetFollowH(uint32 resizingMode, BString *value);
	status_t fSetFollowH(BView *view, const char *value);
	void fGetFollowV(uint32 resizingMode, BString *value);
	status_t fSetFollowV(BView *view, const char *value);
	void fGetFlags(uint32 flags, BString *value);
	status_t fSetFlags(BView *view, const char *value);
	void fGetFont(BFont *font, BString *value);
	status_t fSetFont(BView *view, const char *value);
};

// BControl
class TBCL_BControl : public TBCL_BView
{
public:
	TBCL_BControl(instantiation_info_t *info);
	~TBCL_BControl();
	BArchivable *Instantiate();
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
	status_t GetDefaultSize(float *width, float *height);
};

// BButton
class TBCL_BButton : public TBCL_BControl
{
public:
	TBCL_BButton(instantiation_info_t *info);
	~TBCL_BButton();
	BArchivable *Instantiate();
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
	status_t GetDefaultSize(float *width, float *height);
};

// BMenu
class TBCL_BMenu : public TBCL_BView
{
public:
	TBCL_BMenu(instantiation_info_t *info);
	~TBCL_BMenu();
	BArchivable *Instantiate();
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
	status_t GetDefaultSize(float *width, float *height);
};

// BMenuItem

class TBCL_BMenuItem : public TBCL_BArchivable
{
public:
	TBCL_BMenuItem(instantiation_info_t *info);
	~TBCL_BMenuItem();
	BArchivable *Instantiate();
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
};

// BSeparatorItem

class TBCL_BSeparatorItem : public TBCL_BMenuItem
{
public:
	TBCL_BSeparatorItem(instantiation_info_t *info);
	~TBCL_BSeparatorItem();
	BArchivable *Instantiate();
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
};

// BMenuBar

class TBCL_BMenuBar : public TBCL_BMenu
{
public:
	TBCL_BMenuBar(instantiation_info_t *info);
	~TBCL_BMenuBar();
	BArchivable *Instantiate();
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
	
private:
	status_t fSetEnabled(BMenu *menu, const char *value);
};

// BBox
class TBCL_BBox : public TBCL_BView
{
public:
	TBCL_BBox(instantiation_info_t *info);
	~TBCL_BBox();
	BArchivable *Instantiate();
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
	status_t GetDefaultSize(float *width, float *height);
	
private:
	
};

// BPictureButton
class TBCL_BPictureButton : public TBCL_BControl
{
public:
	TBCL_BPictureButton(instantiation_info_t *info);
	~TBCL_BPictureButton();
	BArchivable *Instantiate();
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
	
private:
	
};

// BCheckBox
class TBCL_BCheckBox : public TBCL_BControl
{
public:
	TBCL_BCheckBox(instantiation_info_t *info);
	~TBCL_BCheckBox();
	BArchivable *Instantiate();
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
	status_t GetDefaultSize(float *width, float *height);
	
private:
	
};

// BColorControl
class TBCL_BColorControl : public TBCL_BControl
{
public:
	TBCL_BColorControl(instantiation_info_t *info);
	~TBCL_BColorControl();
	BArchivable *Instantiate();
//	void Instantiated(BArchivable *target);
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
	status_t GetDefaultSize(float *width, float *height);
	
private:
	
};

// BTextControl
class TBCL_BTextControl : public TBCL_BControl
{
public:
	TBCL_BTextControl(instantiation_info_t *info);
	~TBCL_BTextControl();
	BArchivable *Instantiate();
//	void Instantiated(BArchivable *target);
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
	status_t GetDefaultSize(float *width, float *height);
	
private:
	status_t fGetAlignment(const char *value, alignment *al);
};

// BRadioButton
class TBCL_BRadioButton : public TBCL_BControl
{
public:
	TBCL_BRadioButton(instantiation_info_t *info);
	~TBCL_BRadioButton();
	BArchivable *Instantiate();
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
	status_t GetDefaultSize(float *width, float *height);
};

// BSlider
class TBCL_BSlider : public TBCL_BControl
{
public:
	TBCL_BSlider(instantiation_info_t *info);
	~TBCL_BSlider();
	BArchivable *Instantiate();
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
private:
	status_t fSetUseFillColor(BSlider *slider, const char *value);
};

// BListView
class TBCL_BListView : public TBCL_BView
{
public:
	TBCL_BListView(instantiation_info_t *info);
	~TBCL_BListView();
	BArchivable *Instantiate();
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
};

// BOutlineListView
class TBCL_BOutlineListView : public TBCL_BListView
{
public:
	TBCL_BOutlineListView(instantiation_info_t *info);
	~TBCL_BOutlineListView();
	BArchivable *Instantiate();
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
};

// BMenuField
class TBCL_BMenuField : public TBCL_BView
{
public:
	TBCL_BMenuField(instantiation_info_t *info);
	~TBCL_BMenuField();
	BArchivable *Instantiate();
//	void Instantiated(BArchivable *target);
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
	status_t GetDefaultSize(float *width, float *height);
	status_t GetSuperObject(BArchivable *object, const char *propName, BArchivable **superObject);
private:
	TBCL_BMenu *bclBMenu;
};

// BTabView
class TBCL_BTabView : public TBCL_BView
{
public:
	TBCL_BTabView(instantiation_info_t *info);
	~TBCL_BTabView();
	BArchivable *Instantiate();
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
	status_t GetDefaultSize(float *width, float *height);
};


// BStringView
class TBCL_BStringView : public TBCL_BView
{
public:
	TBCL_BStringView(instantiation_info_t *info);
	~TBCL_BStringView();
	BArchivable *Instantiate();
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
	status_t GetDefaultSize(float *width, float *height);
};


// BTextView
class TBCL_BTextView : public TBCL_BView
{
public:
	TBCL_BTextView(instantiation_info_t *info);
	~TBCL_BTextView();
	BArchivable *Instantiate();
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
	status_t GetDefaultSize(float *width, float *height);
};

// BStatusBar
class TBCL_BStatusBar : public TBCL_BView
{
public:
	TBCL_BStatusBar(instantiation_info_t *info);
	~TBCL_BStatusBar();
	BArchivable *Instantiate();
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
	status_t GetDefaultSize(float *width, float *height);
};

// BScrollBar
class TBCL_BScrollBarH : public TBCL_BView
{
public:
	TBCL_BScrollBarH(instantiation_info_t *info);
	~TBCL_BScrollBarH();
	BArchivable *Instantiate();
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
	status_t GetDefaultSize(float *width, float *height);
};

class TBCL_BScrollBar : public TBCL_BView
{
public:
	TBCL_BScrollBar(instantiation_info_t *info);
	~TBCL_BScrollBar();
	BArchivable *Instantiate();
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
	status_t GetDefaultSize(float *width, float *height);
private:
//	BScrollBar *fSetOrientation(BScrollBar *scrollBar, orientation which);
};

// BScrollView
class TBCL_BScrollView : public TBCL_BView
{
public:
	TBCL_BScrollView(instantiation_info_t *info);
	~TBCL_BScrollView();
	BArchivable *Instantiate();
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
	status_t GetDefaultSize(float *width, float *height);
};

// BScrollView
class TBCL_BScrollView_BListView : public TBCL_BView
{
public:
	TBCL_BScrollView_BListView(instantiation_info_t *info);
	~TBCL_BScrollView_BListView();
	BArchivable *Instantiate();
	const char *SupportedClass() const;
	void PropertyList(TPropertyRegistrar *propertyRegistrar) const;
	status_t SetValue(BArchivable *object, const char *propName, const char *value);
	status_t GetValue(BArchivable *object, const char *propName, BString *value);
	status_t GetDefaultSize(float *width, float *height);
};

#endif
