#ifndef _BE_PROP_EDITORS_H
#define _BE_PROP_EDITORS_H

#include <Window.h>
#include <View.h>
#include <Button.h>
#include <CheckBox.h>
#include <ColorControl.h>
#include <Font.h>
#include <MenuField.h>
#include <MenuItem.h>
#include <PopUpMenu.h>
#include <TextControl.h>
#include <MenuBar.h>
#include <MessageFilter.h>
#include <OutlineListView.h>
#include <ListItem.h>
#include <ScrollView.h>
#include <PopUpMenu.h>
#include <MenuField.h>
#include <interface/TabView.h>
#include <TextView.h>
#include "PropEditor.h"


enum {
	OK_BTN_CLICKED = 'abcd',
	CANCEL_BTN_CLICKED = 'cbcd',
	PROOF_BTN_CLICKED = 'pbcd',
	
	COLORCONTROL_CHANGED = 'cccd',
	FONT_CHANGED = 'fcgd',
	FONTSIZE_CHANGED = 'fscg',
	SELECTION_CHANGED = 'scgd',
	MENU_DRAG = 'mdrg',
	
	ADD_CLICKED = 'aclk',
	DELETE_CLICKED = 'delk',
	LABEL_CHANGED = 'lbch',
};


// TColorEditor --------------------------------------------------

class TColorEditor : public TMultiPropEditor
{
public:
	TColorEditor(propeditor_info_t *peInfo);
	~TColorEditor();
	void MessageReceived(BMessage *msg);
	status_t SetValue(const char *value);
	
private:
	BString fUndoValue;
	BColorControl *fColorControl;
	BCheckBox *fAutoProof;
	void fUpdate();
	void fUndo();
};


// TFontEditor --------------------------------------------------

class TFontEditor : public TMultiPropEditor
{
public:
	TFontEditor(propeditor_info_t *peInfo);
	~TFontEditor();
	void MessageReceived(BMessage *msg);
	status_t SetValue(const char *value);
	
private:
	BString fUndoValue;
	BMenuField *fFontMenuField;
	BMenuField *fSizeMenuField;
	BPopUpMenu *fFontMenu;
	BTextControl *fSizeEdit;
	BMenuItem *fStyleItem;
	BMenuItem *fFamilyItem;
	BCheckBox *fAutoProof;
	void fUpdate();
	void fUndo();
	void fUpdateSizeEdit();
};


// TMenuEditor -----------------------------------------------------

class TMenuEditor;

class TMenuStringItem : public BStringItem
{
public:
	TMenuStringItem(const char *text, BMenuItem *menuItem, uint32 level = 0, bool expanded = true);
	~TMenuStringItem();
	void DrawItem(BView *owner, BRect frame, bool complete = false);
	BMenuItem *MenuItem() const;
	bool Bold;
private:
	BMenuItem *fMenuItem;
};

class TMenuListView : public BOutlineListView
{
public:
	TMenuListView(BRect rect, TMenuEditor *owner);
	~TMenuListView();
	TMenuStringItem *Parent(int32 index) const;
	bool IsParent(int32 index, TMenuStringItem *parent) const;
	void MessageReceived(BMessage *msg);
	void MouseDown(BPoint point);
	void MouseMoved(BPoint point, uint32 transit, const BMessage *msg);
	void DrawUnderline(int32 index);
	bool InitiateDrag(BPoint point, int32 index, bool wasSelected);
private:
	TMenuEditor *fOwner;
	int32 fLastDrawedIndex;
	TMenuStringItem *fDragSource;
};

class TMenuEditor : public TSinglePropEditor
{
public:
	TMenuEditor(propeditor_info_t *peInfo);
	~TMenuEditor();
	
	void Changed();
	void Update();
	void MessageReceived(BMessage *msg);
	
	int32 GetCurrentSelectionMenu(BMenuItem **item, BMenu **menu);
	void Insert();
	void InsertSeparator();
	void Delete();
	void InsertSubmenu();
	void AddItem(TMenuStringItem *item);
	void AddSeparator(TMenuStringItem *item);
	void AddSubmenu(TMenuStringItem *item);
	
private:
	void Clear();
	void BuildOutlineListView(BMenu *menu, uint32 level);
	BScrollView *fScrollView;
	TMenuListView *fMenuListView;
	BView *fView;
	
	TMenuStringItem *fSelectedStringItem;
};

// TTabEditor -----------------------------------------------------

class TTabListView : public BListView
{
public:
	TTabListView(BRect rect);
	~TTabListView();
	void MessageReceived(BMessage *msg);
	void MouseMoved(BPoint point, uint32 transit, const BMessage *msg);
	void DrawUnderline(int32 index);
	bool InitiateDrag(BPoint point, int32 index, bool wasSelected);
private:
	int32 fLastDrawedIndex;
	BStringItem *fDragSource;
};

class TTabEditor : public TSinglePropEditor
{
public:
	TTabEditor(propeditor_info_t *peInfo);
	~TTabEditor();
	void Changed();
	void Update();
	void MessageReceived(BMessage *msg);
	void Move(int32 srcIndex, int32 destIndex);
private:
	void Clear();
	void BuildListView();
	void SelectionChanged();
	void AddClicked();
	void DeleteClicked();
	void LabelChanged();
	TTabListView *fListView;
	BButton *fAddButton;
	BButton *fDeleteButton;
	BTextControl *fLabel;
	int32 fCurrentSelection;
};


// TTextViewEditor -----------------------------------------------------

class TTextEditor : public BTextView
{
public:
	TTextEditor(BRect frame, const char *name, BRect textRect,
				uint32 resizingMode, uint32 flags);
	void FrameResized(float width, float height);
	void SetTargetTextView(BTextView *target);
protected:
	void InsertText(const char *text, int32 length, int32 offset, const text_run_array *runs);
	void DeleteText(int32 start, int32 finish);
private:
	BTextView *fTargetTextView;
	
};

class TTextViewEditor : public TSinglePropEditor
{
public:
	TTextViewEditor(propeditor_info_t *peInfo);
	~TTextViewEditor();
	void Changed();
	void Update();
	
private:
	BScrollView *fScrollView;
	TTextEditor *fTextEditor;
	
};


#endif
