#ifndef _PROP_EDITOR_H
#define _PROP_EDITOR_H

#include <List.h>
#include <stdio.h>
#include <Archivable.h>
#include <Window.h>
#include <MessageFilter.h>

#include "BCL.h"

//#define REGISTER_PROPEDITOR(type, info, pinfo)	if (info == &typeid(type)) return new type(pinfo);
#define REGISTER_PROPEDITOR(type, info, pinfo)	if (strcmp(info->name(), typeid(type).name()) == 0) return new type(pinfo);
#define PROPEDITOR(type) &typeid(type)

typedef status_t updater_func(const char *value);


typedef struct propeditor_info_t {
	updater_func *updaterfunc;
	TLockedList *locked_list;
} propeditor_info_t;


// TPropEditor

class TPropEditor : public BWindow
{
public:
	TPropEditor(propeditor_info_t *peInfo,
				BRect frame, const char *title,
				window_type type, uint32 flags,
				uint32 workspaces = B_CURRENT_WORKSPACE);
	TPropEditor(propeditor_info_t *peInfo,
				BRect frame, const char *title,
				window_look look, window_feel feel, uint32 flags,
				uint32 workspaces = B_CURRENT_WORKSPACE);
	~TPropEditor();
	const propeditor_info_t *PeInfo() const;
	void LockView(BView *view);
	void UnlockView(BView *view);
private:
	propeditor_info_t fPeInfo;
};

class TSinglePropEditor;

// TSPEMessageFilter

class TSPEMessageFilter : public BMessageFilter
{
public:
	TSPEMessageFilter(TSinglePropEditor *owner);
	~TSPEMessageFilter();
	filter_result Filter(BMessage *msg, BHandler **target);
	
private:
	TSinglePropEditor *fOwner;
};

// TSinglePropEditor

class TSinglePropEditor : public TPropEditor
{
public:
	TSinglePropEditor(propeditor_info_t *peInfo,
				BRect frame, const char *title,
				window_type type, uint32 flags,
				uint32 workspaces = B_CURRENT_WORKSPACE);
	TSinglePropEditor(propeditor_info_t *peInfo,
				BRect frame, const char *title,
				window_look look, window_feel feel, uint32 flags,
				uint32 workspaces = B_CURRENT_WORKSPACE);
	~TSinglePropEditor();
	virtual void Changed() = 0;
	virtual void Update() = 0;
	BArchivable *Object();
	
protected:
	void SetObjectInspector(BArchivable *object) const;
	virtual void SetObject(BArchivable *object);
	
private:
	friend class TObjectInspectorView;
	void fInitialization();
	BArchivable *fObject;
	BMessenger fMessenger;
	BString fClassName;
};

// TMultiPropEditor

class TMultiPropEditor : public TPropEditor
{
public:
	TMultiPropEditor(propeditor_info_t *peInfo,
				BRect frame, const char *title,
				window_type type, uint32 flags,
				uint32 workspaces = B_CURRENT_WORKSPACE);
	TMultiPropEditor(propeditor_info_t *peInfo,
				BRect frame, const char *title,
				window_look look, window_feel feel, uint32 flags,
				uint32 workspaces = B_CURRENT_WORKSPACE);
	~TMultiPropEditor();
	virtual status_t SetValue(const char *value) = 0;
	
protected:
	status_t Update(const char *value);
};


#endif
