#include <stdio.h>
#include <stdlib.h>
#include "AppfEditorWindow.h"


enum {
	EW_APPLY = 'ewap',
	EW_CANCEL = 'ewca',
};

// TAppfEditorView

TAppfEditorView::TAppfEditorView(BRect frame, const char *name)
	:	BView(frame, name, B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FRAME_EVENTS)
{
}

TAppfEditorView::~TAppfEditorView()
{
}


// TAppfEditorWindow

TAppfEditorWindow::TAppfEditorWindow(BRect frame, const char *title,
			type_code type, TResAddon *addon)
	:	TResEditorWindow(frame, title,
			B_TITLED_WINDOW, B_NOT_ZOOMABLE | B_NOT_RESIZABLE)
{
	fType = type;
	fAddon = addon;
	
	fAppfEditorView = new TAppfEditorView(Bounds(), "EditorView");
	fAppfEditorView->SetViewColor(216, 216, 216);
	AddChild(fAppfEditorView);
	
	fSingleLaunch = new BRadioButton(BRect(10, 10, 115, 20),
			"SingleLaunch", "Single Launch", NULL);
	fAppfEditorView->AddChild(fSingleLaunch);
	
	fMultipleLaunch = new BRadioButton(BRect(10, 30, 115, 40),
			"MultipleLaunch", "Multiple Launch", NULL);
	fAppfEditorView->AddChild(fMultipleLaunch);
	
	fExclusiveLaunch = new BRadioButton(BRect(10, 50, 115, 60),
			"ExclusiveLaunch", "Exclusive Launch", NULL);
	fAppfEditorView->AddChild(fExclusiveLaunch);
	
	fArgvOnly = new BCheckBox(BRect(130, 10, 250, 20), "ArgvOnly", "Argv Only", NULL);
	fAppfEditorView->AddChild(fArgvOnly);
	
	fBackgroundApp = new BCheckBox(BRect(130, 30, 250, 40), "BackgroundApp", "Background App", NULL);
	fAppfEditorView->AddChild(fBackgroundApp);
	
	
	fApplyButton = new BButton(BRect(100, 80, 160, 95),
			"Apply", "Apply", new BMessage(EW_APPLY));
	fApplyButton->MakeDefault(true);
	fAppfEditorView->AddChild(fApplyButton);
	fCancelButton = new BButton(BRect(170, 80, 230, 95),
			"Cancel", "Cancel", new BMessage(EW_CANCEL));
	fAppfEditorView->AddChild(fCancelButton);
}

TAppfEditorWindow::~TAppfEditorWindow()
{
}

void TAppfEditorWindow::MessageReceived(BMessage *msg)
{
	switch(msg->what) {
		case EW_APPLY:
			SendData();
			break;
		case EW_CANCEL:
			Quit();
			break;
		default: BWindow::MessageReceived(msg);
	}
}

status_t TAppfEditorWindow::InstantiateFromData(const void *data,
		ssize_t numBytes)
{
	switch(fType){
		case 'APPF': {
			uint32 flag = *(uint32 *)data;
			switch(flag & 3) {
				case 0: fSingleLaunch->SetValue(1); break;
				case 1: fMultipleLaunch->SetValue(1); break;
				case 2: fExclusiveLaunch->SetValue(1); break;
			}
			fArgvOnly->SetValue(flag & 8);
			fBackgroundApp->SetValue(flag & 4);
			break;
		}
	}
	return B_OK;
}

void TAppfEditorWindow::GetData(BMallocIO *data)
{
	data->SetSize(0);
	switch(fType) {
		case 'APPF': {
			uint32 flag = 0;
			if (fSingleLaunch->Value()) flag = 0; else
			if (fMultipleLaunch->Value()) flag = 1; else
			if (fExclusiveLaunch->Value()) flag = 2;
			if (fArgvOnly->Value()) flag |= 8;
			if (fBackgroundApp->Value()) flag |= 4;
			data->Write(&flag, sizeof(flag));
			break;
		}
		default: ;
	}
}
