#include <stdio.h>
#include <String.h>
#include <Looper.h>
#include <Screen.h>
#include <File.h>
#include <Directory.h>
#include <Resources.h>
#include <Entry.h>
#include <Window.h>
#include <Archivable.h>
#include <List.h>
#include <Clipboard.h>
#include <Application.h>
#include <MessageQueue.h>

#include "BerryUtils.h"
#include "Berry.h"
#include "ContainerWindow.h"
#include "ContainerView.h"
#include "BCLPictureButton.h"
#include "MainView.h"
#include "LockedList.h"
#include "ResManager.h"
#include "ObjectInspectorView.h"
#include "Locale.h"
#include "GGServer.h"
#include "GGShared.h"


extern TBCLPictureButton *selected_bcl_button;		// 選択中のBCL


#define _RECT(x,y) x-2,y-2,x+2,y+2

enum {
	TOP_LEFT = 1,
	BTM_LEFT = 2,
	TOP_RIGHT = 3,
	BTM_RIGHT = 4,
	TOP_LEFT2 = 5,
	BTM_LEFT2 = 6,
	TOP2_LEFT = 7,
	TOP2_RIGHT = 8
};

enum {
	ALIGN_LEFT_SIDES = 'alsd',
	ALIGN_H_CENTERS = 'ahcs',
	ALIGN_RIGHT_SIDES = 'arsd',
	ALIGN_H_SPACE_EQUALLY = 'ahse',
	ALIGN_H_CENTER_IN_WINDOW = 'ahcw',
	ALIGN_TOPS = 'atps',
	ALIGN_V_CENTERS = 'avcs',
	ALIGN_BOTTOMS = 'abms',
	ALIGN_V_SPACE_EQUALLY = 'avse',
	ALIGN_V_CENTER_IN_WINDOW = 'avcw'
};

int compareHFunc(const void *item1, const void *item2);
int compareVFunc(const void *item1, const void *item2);


// TContainerView ---------------------------------------------------------------------------

TContainerView::TContainerView(BRect frame, const char *name)
	:	BView(frame, name, B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP_BOTTOM,
			B_WILL_DRAW | B_FRAME_EVENTS)
{
	BGView = new BView(Bounds(), "BGView", B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FRAME_EVENTS);
	UIView = new TUIView(Bounds(), "UIView", BGView, this);
	MaskView = new TMaskView(Bounds(), "MaskView", UIView);
	
	MaskView->SetViewColor(B_TRANSPARENT_32_BIT);
	BGView->SetViewColor(216, 216, 216);
	UIView->SetViewColor(B_TRANSPARENT_32_BIT);
	
	this->AddChild(MaskView);
	this->AddChild(BGView);
	this->AddChild(UIView);
	
	LockedList.AddView(BGView);
}

TContainerView::~TContainerView()
{
}

void TContainerView::MessageReceived(BMessage *msg)
{
	switch(msg->what) {
		case BERRY_VIEW_INVALIDATE: {
			UIView->Invalidate();
			this->SetDirty();
			BGView->SetResizingMode(0);
			Window()->ResizeTo(BGView->Frame().Width(), BGView->Frame().Height());
			BGView->SetResizingMode(B_FOLLOW_ALL_SIDES);
			
			// GenesTation へ送信
			void *object;
			BString name, old_value, new_value;
			msg->FindPointer("object", &object);
			msg->FindString("name", &name);
			msg->FindString("old_value", &old_value);
			msg->FindString("new_value", &new_value);
			if (name == "Name") {
				BView *view = dynamic_cast<BView *>((BView *)object);
				if (view != NULL) {
					entry_ref ref;
					int32 id;
					((TResEditorWindow *)Window())->GetParameter(NULL, NULL, NULL, &id, &ref);
					BMessage archive;
					view->Archive(&archive);
					BMessage msg(GG_COMPONENT_NAME_CHANGED);
					msg.AddRef("refs", &ref);
					msg.AddInt32(GG_MSG_NAME_RESOURCE_ID, id);
					msg.AddString(GG_MSG_NAME_COMPONENT_NAME, old_value);
					msg.AddString(GG_MSG_NAME_COMPONENT_NEW_NAME, new_value);
					msg.AddMessage(GG_MSG_NAME_COMPONENT, &archive);
					BMessenger(gg_server).SendMessage(msg);
				}
			}
				
			break;
		}
	}
}

status_t TContainerView::InstantiateFromData(const char *data, ssize_t numBytes)
{
	status_t status;
	
	BMessage archive;
	status = archive.Unflatten(data);
	if (status != B_OK) {
		return status;
	}
	
	BArchivable *arc = instantiate_object(&archive);
	if (arc == NULL) {
		return B_ERROR;
	}
	
	BView *newView = dynamic_cast<BView *>(arc);
	if (newView == NULL) {
		return B_ERROR;
	}
	
	for(int32 i=0; i < be_app->CountWindows(); i++) {
		BWindow *window = be_app->WindowAt(i);
		TSinglePropEditor *singleEditor = dynamic_cast<TSinglePropEditor *>(window);
		if (singleEditor != NULL) {
			BMessenger(singleEditor).SendMessage(B_QUIT_REQUESTED);
		} else {
			TMultiPropEditor *multiEditor = dynamic_cast<TMultiPropEditor *>(window);
			if (multiEditor != NULL) {
				BMessenger(singleEditor).SendMessage(B_QUIT_REQUESTED);
			}
		}
	}
	UIView->ClearSelectedList();
	
	this->RemoveChild(UIView);
	this->RemoveChild(BGView);
	LockedList.RemoveView(BGView);
	delete BGView;
	BGView = newView;
	Window()->ResizeTo(BGView->Frame().Width(), BGView->Frame().Height());
	UIView->ResizeTo(BGView->Frame().Width(), BGView->Frame().Height());
	BGView->SetResizingMode(B_FOLLOW_ALL_SIDES);
	BGView->SetFlags(B_WILL_DRAW | B_FRAME_EVENTS);
	this->AddChild(BGView);
	this->AddChild(UIView);
	UIView->SetUIView(BGView);
	UIView->MoveTo(0, 0);
	UIView->SetContainerView(this);
	
	BMessage instantiatedMsg;
	instantiatedMsg.AddPointer("instantiated_object", BGView);
	UIView->MakeInstantiatedMsg(BGView, &instantiatedMsg);
	UIView->SendToObjectInspector(&instantiatedMsg);
	
	Invalidate();
	return status;
}

void TContainerView::SetDirty()
{
	((TContainerWindow *)Window())->SetDirty();
	((TContainerWindow *)Window())->SendData();
}


// TUIView -----------------------------------------------------------------------------------

TUIView::TUIView(BRect frame, const char *name, BView *bgView, TContainerView *containerView)
	:	BView(frame, name, B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FRAME_EVENTS)
{
	BGView = bgView;
	ContainerView = containerView;
	fDragging = false;
	
	fPopUpMenu = new BPopUpMenu("PopUpMenu");
	fPopUpMenu->SetRadioMode(false);
	
	fAlignMenu = new BMenu(GET_STRING("Align", "Align..."));
	
	fHorizontalMenu = new BMenu(GET_STRING("Horizontal", "Horizontal"));
	fAlignMenu->AddItem(fHorizontalMenu);
	fHorizontalMenu->AddItem(new BMenuItem(GET_STRING("Left_sides", "Left sides"), new BMessage(ALIGN_LEFT_SIDES)));
	fHorizontalMenu->AddItem(new BMenuItem(GET_STRING("Centers", "Centers"), new BMessage(ALIGN_H_CENTERS)));
	fHorizontalMenu->AddItem(new BMenuItem(GET_STRING("Right_sides", "Right sides"), new BMessage(ALIGN_RIGHT_SIDES)));
	fHorizontalMenu->AddItem(new BMenuItem(GET_STRING("Space_equally", "Space equally"), new BMessage(ALIGN_H_SPACE_EQUALLY)));
//	fHorizontalMenu->AddItem(new BMenuItem(GET_STRING("Center_in_window", ""), new BMessage(ALIGN_H_CENTER_IN_WINDOW)));
	
	fVerticalMenu = new BMenu(GET_STRING("Vertical", "Vertical"));
	fAlignMenu->AddItem(fVerticalMenu);
	fVerticalMenu->AddItem(new BMenuItem(GET_STRING("Tops", "Tops"), new BMessage(ALIGN_TOPS)));
	fVerticalMenu->AddItem(new BMenuItem(GET_STRING("Centers", "Centers"), new BMessage(ALIGN_V_CENTERS)));
	fVerticalMenu->AddItem(new BMenuItem(GET_STRING("Bottoms", "Bottoms"), new BMessage(ALIGN_BOTTOMS)));
	fVerticalMenu->AddItem(new BMenuItem(GET_STRING("Space_equally", "Space equally"), new BMessage(ALIGN_V_SPACE_EQUALLY)));
//	fVerticalMenu->AddItem(new BMenuItem(GET_STRING("Center_in_window", ""), new BMessage(ALIGN_V_CENTER_IN_WINDOW)));
	
	fPopUpMenu->AddItem(fAlignMenu);
	fPopUpMenu->AddItem(new BSeparatorItem());
	
	fCutMenuItem = new BMenuItem(GET_STRING("Cut", "Cut"), new BMessage(B_CUT), 'X', 0);
	fPopUpMenu->AddItem(fCutMenuItem);
	fCopyMenuItem = new BMenuItem(GET_STRING("Copy", "Copy"), new BMessage(B_COPY), 'C', 0);
	fPopUpMenu->AddItem(fCopyMenuItem);
	fPasteMenuItem = new BMenuItem(GET_STRING("Paste", "Paste"), new BMessage(B_PASTE), 'V', 0);
	fPopUpMenu->AddItem(fPasteMenuItem);
	fDeleteMenuItem= new BMenuItem(GET_STRING("Delete", "Delete"), new BMessage(B_DELETE));
	fPopUpMenu->AddItem(fDeleteMenuItem);
	
	fPopUpMenu->AddItem(new BSeparatorItem());
	
	fSelectAllMenuItem = new BMenuItem(GET_STRING("Select_All", "Select All"), new BMessage(B_SELECT_ALL), 'A', 0);
	fPopUpMenu->AddItem(fSelectAllMenuItem);
}

TUIView::~TUIView()
{
	delete fPopUpMenu;
}

void TUIView::DetachedFromWindow()
{
	BMessage msg(BERRY_OBJECT_SELECTED);
	SendToObjectInspector(&msg);
	
	msg.what = BERRY_OBJECT_DELETED;
	MakeDeleteMsg(BGView, &msg);
	SendToObjectInspector(&msg);
}

void TUIView::MessageReceived(BMessage *msg)
{
	switch(msg->what) {
		case B_CUT: Cut(); break;
		case B_COPY: Copy(); break;
		case B_PASTE: Paste(); break;
		case B_DELETE: DeleteSelection(); break;
		case B_SELECT_ALL: SelectAll(); break;
		case BERRY_BCL_DROPPED: {
			BPoint drop_point;
			void *p;
			if ((msg->FindPoint("_drop_point_", &drop_point) == B_OK) &&
			    (msg->FindPointer("bcl", &p) == B_OK)) {
			    ConvertFromScreen(&drop_point);
				TBerryComponentLibrary *bcl = reinterpret_cast<TBerryComponentLibrary *>(p);
				PutObject(drop_point, bcl);
			}
			break;
		}
	}
}

// point 上の View を再帰的に階層を辿りながら特定する。
// ただし、CS_ACCEPTS_CONTROLS 指定されていない View 以降は探さない。
BView *TUIView::FindView(BPoint point, bool deep, BView *view) const
{
	TBerryComponentLibrary *bcl;
	
	for(int32 i = view->CountChildren()-1; i >= 0; i--) {
		if (FindSupportBCL(view, &bcl) != B_OK) break;
		if (!(bcl->ControlStyle() & CS_ACCEPTS_CONTROLS)) break;
		BView *child = view->ChildAt(i);
		if (child->Frame().Contains(point)) {
			if (deep) {
				BPoint _point = child->ConvertFromParent(point);
				BView *grandchild = this->FindView(_point, true, child);
				if (grandchild==NULL) {
					return child;
				} else {
					return grandchild;
				}
			} else {
				return child;
			}
		}
	}
	return view;
}

void TUIView::Draw(BRect updateRect)
{
	rgb_color normal = {0xff, 0, 0, 0};
	rgb_color locked = {0, 0, 0xff, 0};
	
	// 選択されている view をフォーカス
	SetDrawingMode(B_OP_COPY);
	SetPenSize(0);
	if (fSelectedList.CountItems() == 1) {
		BView *view = (BView *)fSelectedList.ItemAt(0);
		if (view->Window() == NULL) return;
		SetHighColor(LockedList.HasView(view) ? locked : normal);
 		SetHighColor(normal);
		BRect rect = TopViewFrame(view);
		StrokeRect(rect);
		DrawHandlerBox(BPoint(rect.left, rect.top));
		DrawHandlerBox(BPoint(rect.left, rect.bottom));
		DrawHandlerBox(BPoint(rect.right, rect.top));
		DrawHandlerBox(BPoint(rect.right, rect.bottom));
		DrawHandlerBox(BPoint(rect.left + rect.Width() / 2, rect.top));
		DrawHandlerBox(BPoint(rect.left + rect.Width() / 2, rect.bottom));
		DrawHandlerBox(BPoint(rect.left, rect.top + rect.Height() / 2));
		DrawHandlerBox(BPoint(rect.right, rect.top + rect.Height() / 2));
	} else {
		for(int32 i=0; i<fSelectedList.CountItems(); i++) {
			BView *view = (BView *)fSelectedList.ItemAt(i);
			if (view->Window() == NULL) continue;
			SetHighColor(LockedList.HasView(view) ? locked : normal);
			SetHighColor(normal);
			StrokeRect(TopViewFrame(view));
		}
	}
}

void TUIView::FrameResized(float width, float height)
{
	if (fSelectedList.CountItems()==1) InvalidateSelection();
	BView::FrameResized(width, height);
	UpdateObjectInspector();
	ContainerView->SetDirty();
}

bool TUIView::DrawSelecting(BRect *rect, BPoint cursor)
{
	BPoint currentPos, lastPos;
	uint32 buttons;
	key_info info;
	bool canceled = false;
	
	SetDrawingMode(B_OP_INVERT);
	SetPenSize(0);
	GetMouse(&currentPos, &buttons);
	lastPos = currentPos;
	StrokeRect(BRect(cursor.x, cursor.y, currentPos.x, currentPos.y));
	while(buttons==1 && !canceled){
		if (lastPos!=currentPos) {
			StrokeRect(BRect(cursor.x, cursor.y, lastPos.x, lastPos.y));
			StrokeRect(BRect(cursor.x, cursor.y, currentPos.x, currentPos.y));
			lastPos = currentPos;
		}
		snooze(10000);
//		ProcessMessages();
		GetMouse(&currentPos, &buttons);
		get_key_info(&info);
		canceled = info.key_states[0] & 0x40;	// ESC キー押下でキャンセル
	}
	*rect = BRect(min_c(cursor.x, lastPos.x), min_c(cursor.y, lastPos.y),
			max_c(cursor.x, lastPos.x), max_c(cursor.y, lastPos.y));
	StrokeRect(*rect);
	
	return !canceled && buttons==0;
}

void TUIView::Selecting(BView *bgView, BPoint cursor, bool multi)
{
	
	BRect selRect;
	if (!DrawSelecting(&selRect, cursor)) return;
	
	ClearSelectedList();
	// 選択範囲中の view を検索して fSelectedList に追加
	// (multi == true で fSelectedList に既に存在するなら削除)
	BView *child = NULL;
	for(int i = 0; i < bgView->CountChildren(); i++) {
		child = bgView->ChildAt(i);
		if (TopViewFrame(child).Intersects(selRect)) {
			if (fSelectedList.HasItem(child)) {
				if (multi) {
					fSelectedList.RemoveItem(child);
				}
			} else {
				AddSelection(child);
			}
		}
	}
	if ((fSelectedList.CountItems()==0) && (!multi)) {
		AddSelection(bgView);
	}
	UpdateObjectInspector();
}

void TUIView::InvalidateObject(BView *view)
{
	BRect rect = TopViewFrame(view);
	rect.left -= 2;
	rect.right += 2;
	rect.top -= 2;
	rect.bottom += 2;
	Invalidate(rect);
}

void TUIView::AddSelection(BView *view)
{
	fSelectedList.AddItem(view);
	InvalidateObject(view);
	SendGGMessage(view, GG_COMPONENT_SELECTION_CHANGED);
}

void TUIView::InvalidateSelection()
{
	for(int32 i=0; i < fSelectedList.CountItems(); i++) {
		InvalidateObject((BView *)fSelectedList.ItemAt(i));
	}
}

void TUIView::PutObject(BPoint point, TBerryComponentLibrary *t_bcl)
{
	BView *destView = NULL;
	BRect rect;
	BPoint destPoint, topleft;
	BString className;
	float width, height;
	TBerryComponentLibrary *bcl, *new_bcl;
	
	Window()->Activate(true);
	// AddChild すべき BView を決める
	destView = this->FindView(point, true, BGView);
	
	// ただし CS_ACCEPTS_CONTROLS が指定されていない View の上には配置できないようにする
	if (FindSupportBCL(destView, &bcl) != B_OK) {
		destView = destView->Parent();
	} else {
		if (!(bcl->ControlStyle() & CS_ACCEPTS_CONTROLS)) destView = destView->Parent();
	}
	
	if (!DrawSelecting(&rect, point)) return;
	if (t_bcl == NULL) {
		new_bcl = selected_bcl_button->BCL();
	} else {
		new_bcl = t_bcl;
		rect.right = rect.left;
		rect.bottom = rect.top;
	}
	
	topleft.x = (point.x == rect.left) ? point.x : rect.left;
	topleft.y = (point.y == rect.top) ? point.y : rect.top;
	
	if (destView == NULL) {
		destView = BGView;
		destPoint = topleft;
	} else {
		BRect frame = TopViewFrame(destView);
		destPoint = BPoint(topleft.x - frame.left, topleft.y - frame.top);
	}
	
	ClearSelectedList();
	
	const char *rawName = new_bcl->SupportedClass();
	className.Append(rawName, GetNumber(rawName));
	
	BArchivable *object = new_bcl->Instantiate();
	BView *newObject = dynamic_cast<BView *>(object);
	BString name("New ");
	name << className;
	if (newObject != NULL) {
		// オブジェクト名が重複しないようにする
		uint32 suf = 1;
		BString baseName(name);
		while(Window()->FindView(name.String()) != NULL) {
			name.SetTo(baseName);
			name << " " << suf;
			suf++;
		}
		
		newObject->SetName(name.String());
		if (dynamic_cast<BControl *>(newObject) != NULL) ((BControl *)newObject)->SetLabel(name.String());
		newObject->MoveTo(destPoint);
		if (new_bcl->GetDefaultSize(&width, &height) == B_OK) {
			if (rect.Width() > 2 && rect.Height() > 2) {
				width = rect.Width();
				height = rect.Height();
			}
			newObject->ResizeTo(width, height);
		}
		
		destView->AddChild(newObject);
		
		BMessage instantiatedMsg;
		instantiatedMsg.AddPointer("instantiated_object", newObject);
		MakeInstantiatedMsg(newObject, &instantiatedMsg);
		SendToObjectInspector(&instantiatedMsg);
		
		SendGGMessage(newObject, GG_COMPONENT_ADDED);
		AddSelection(newObject);
		UpdateObjectInspector();
	}
	
	// SHIFT キーを押していると連続で配置できるようにする
	if (!(modifiers() & B_SHIFT_KEY)) {
		main_view->Looper()->Lock();
		main_view->DeselectAll();
		main_view->Looper()->Unlock();
	}
	
	ContainerView->SetDirty();
	
	return;
}

void TUIView::SendToObjectInspector(BMessage *msg)
{
	BMessenger((const BHandler *)ObjectInspectorView).SendMessage(msg);
}

void TUIView::MakeInstantiatedMsg(BView *parent, BMessage *instantiatedMsg)
{
//	instantiatedMsg->what = BERRY_OBJECT_INSTANTIATED;
	
	for(int32 i=0; i < parent->CountChildren(); i++) {
		BView *child = (BView *)parent->ChildAt(i);
		instantiatedMsg->AddPointer("instantiated_object", child);
		MakeInstantiatedMsg(child, instantiatedMsg);
	}
}

void TUIView::MouseDownLeft(BPoint point)
{
	uint32 key = modifiers();
	bool shift = key & B_SHIFT_KEY;
	bool command = key & B_COMMAND_KEY;
//	bool control = key & B_CONTROL_KEY;
//	bool option = key & B_OPTION_KEY;
	
	if (selected_bcl_button != NULL) {
		PutObject(point);
		return;
	}
	
	// ダブルクリック検出
	int32 clicks;
	BMessage *msg = Looper()->CurrentMessage();
	if (msg->FindInt32("clicks", &clicks) == B_OK && clicks == 2) {
		ViewInvoked();
//		return;
	}
	
	BView *view = this->FindView(point, true, BGView);
	if (shift) {
		if (command) {
			if (fSelectedList.CountItems() == 1) {
				BView *bgView = (BView *)fSelectedList.ItemAt(0);
				Selecting(bgView, point, true);
			}
		} else {
			if (view == BGView) {
				Selecting(view, point, true);
			} else {
				if (fSelectedList.HasItem(view)) {
					fSelectedList.RemoveItem(view);
				} else {
					if (fSelectedList.CountItems()>0) {
						BView *parent = ((BView *)fSelectedList.ItemAt(0))->Parent();
						if (view->Parent() != parent) return;
					}
					AddSelection(view);
					InvalidateSelection();
				}
				UpdateObjectInspector();
			}
		}
	} else {
		// 押した座標がリサイズポイントか？
		int resizingMode = 0;
		if (fSelectedList.CountItems() == 1) {
			BView *view = (BView *)fSelectedList.ItemAt(0);
			if (view != BGView) {
				BRect rect = TopViewFrame(view);
				if (BRect(_RECT(rect.right, rect.bottom)).Contains(point)) {resizingMode = BTM_RIGHT;} else
				if (BRect(_RECT(rect.left, rect.bottom)).Contains(point)) {resizingMode = BTM_LEFT;} else
				if (BRect(_RECT(rect.right, rect.top)).Contains(point)) {resizingMode = TOP_RIGHT;} else
				if (BRect(_RECT(rect.left + rect.Width() / 2, rect.bottom)).Contains(point)) {resizingMode = BTM_LEFT2;} else
				if (BRect(_RECT(rect.left + rect.Width() / 2, rect.top)).Contains(point)) {resizingMode = TOP_LEFT2;} else
				if (BRect(_RECT(rect.right, rect.top + rect.Height() / 2)).Contains(point)) {resizingMode = TOP2_RIGHT;} else
				if (BRect(_RECT(rect.left, rect.top + rect.Height() / 2)).Contains(point)) {resizingMode = TOP2_LEFT;} else
				if (BRect(_RECT(rect.left, rect.top)).Contains(point)) {resizingMode = TOP_LEFT;}
			}
		}
		if (resizingMode > 0) {
			// 押した座標がリサイズポイントならリサイズモード開始
			Resizing(resizingMode);
		} else {
			bool moving = false;
			if (command) {
				// (はぐれView救済用)
				// 押した座標が fSelectedList から一つでも重なっていたら移動モード開始
				for (int32 i=0; i < fSelectedList.CountItems() && !moving; i++) {
					BView *sview = (BView *)fSelectedList.ItemAt(i);
					if (sview == BGView) continue;
					moving = TopViewFrame(sview).Contains(point);
				}
			} else {
				// クリックした view が fSelectedList 中にあれば移動モード開始
				if (view != BGView) {
					for(int32 i=0;  i < fSelectedList.CountItems() && !moving; i++) {
						BView *sview = (BView *)fSelectedList.ItemAt(i);
						if (sview == view) {
							moving = true;
							break;
						}
					}
				}
			}
			
			if (moving) {
				// moving (BGView は背景なので動かさない)
				Moving(point);
			} else {
				// selecting
				if (view == BGView) {
					Selecting(view, point, false);
				} else {
					ClearSelectedList();
					AddSelection(view);
					UpdateObjectInspector();
				}
			}
		}
	}
//	Invalidate();
}

void TUIView::MouseDownRight(BPoint point)
{
	bool moving = false;
	BView *view = this->FindView(point, true, BGView);
	for(int32 i=0;  i < fSelectedList.CountItems() && !moving; i++) {
		BView *sview = (BView *)fSelectedList.ItemAt(i);
		if (sview == view) {
			moving = true;
			break;
		}
	}
	if (!moving) {
		ClearSelectedList();
		AddSelection(view);
		UpdateObjectInspector();
	}
	
	ConvertToScreen(&point);
	uint32 command;
	BMenuItem *selected = fPopUpMenu->Go(point, false, true, BRect(point.x, point.y, point.x, point.y));
	if (selected == NULL) return;
	command = selected->Command();
	switch(command){
		case B_CUT:
			Cut();
			break;
		case B_COPY:
			Copy();
			break;
		case B_PASTE:
			Paste();
			break;
		case B_DELETE:
			DeleteSelection();
			break;
		case B_SELECT_ALL:
			SelectAll();
			break;
		case ALIGN_LEFT_SIDES:
		case ALIGN_H_CENTERS:
		case ALIGN_RIGHT_SIDES:
		case ALIGN_TOPS:
		case ALIGN_V_CENTERS:
		case ALIGN_BOTTOMS:
			Align(command);
			break;
		case ALIGN_H_SPACE_EQUALLY:
			AlignHSpaceEqually();
			break;
		case ALIGN_V_SPACE_EQUALLY:
			AlignVSpaceEqually();
			break;
		case ALIGN_H_CENTER_IN_WINDOW:
			break;
		case ALIGN_V_CENTER_IN_WINDOW:
			break;
	}
}

void TUIView::Cut()
{
	Copy();
	DeleteSelection();
}

void TUIView::Copy()
{
	if (be_clipboard->Lock()) {
		be_clipboard->Clear();
		BMessage *clip = be_clipboard->Data();
		if (clip) {
			for(int32 i=0; i < fSelectedList.CountItems(); i++) {
				BArchivable *object = (BArchivable *)fSelectedList.ItemAt(i);
				if (object == BGView) break;
				BMessage aclip;
				status_t status = object->Archive(&aclip, true);
				clip->AddMessage(BERRY_CLIP_NAME, &aclip);
				if (status != B_OK) fprintf(stderr, APP_NAME ": Copy() error  %d.", (int)status);
			}
		}
		be_clipboard->Unlock();
	}
}

void TUIView::Paste()
{
	BView *baseView;
	if ((fSelectedList.CountItems() > 1) || (fSelectedList.CountItems() == 0)) {
		baseView = BGView;
	} else {
		baseView = (BView *)fSelectedList.ItemAt(0);
	}
	
	if (be_clipboard->Lock()) {
		BMessage *clip = be_clipboard->Data();
		if (clip) {
			ClearSelectedList();
			type_code typeCode;
			int32 count;
			clip->GetInfo(BERRY_CLIP_NAME, &typeCode, &count);
			BMessage aclip;
			for(int32 i=0; i < count; i++) {
				if (clip->FindMessage(BERRY_CLIP_NAME, i, &aclip) == B_OK) {
					BArchivable *object = instantiate_object(&aclip);
					BView *view = dynamic_cast<BView *>(object);
					if (view != NULL) {
						baseView->AddChild(view);
						fSelectedList.AddItem(view);
						SendGGMessage(view, GG_COMPONENT_ADDED);
						SendGGMessage(view, GG_COMPONENT_SELECTION_CHANGED);
					}
				}
			}
		}
		be_clipboard->Unlock();
	}
	UpdateObjectInspector();
	Invalidate();
}

void TUIView::SelectAll()
{
	BView *baseView;
	if ((fSelectedList.CountItems() > 1) || (fSelectedList.CountItems() == 0)) {
		baseView = BGView;
	} else {
		baseView = (BView *)fSelectedList.ItemAt(0);
	}
	ClearSelectedList();
	for(int i=0; i < baseView->CountChildren(); i++) {
		fSelectedList.AddItem(baseView->ChildAt(i));
	}
	UpdateObjectInspector();
	Invalidate();
}

void TUIView::Align(uint32 alignCommand)
{
	if (fSelectedList.CountItems() < 2) return;
	BRect baseRect = ((BView *)fSelectedList.ItemAt(0))->Frame();
	for(int32 i=1; i < fSelectedList.CountItems(); i++) {
		BView *view = (BView *)fSelectedList.ItemAt(i);
		BRect frame = view->Frame();
		switch(alignCommand){
			case ALIGN_LEFT_SIDES:
				view->MoveTo(baseRect.left, frame.top);
				break;
			case ALIGN_H_CENTERS:{
				float center = baseRect.left + (baseRect.Width() / 2);
				view->MoveTo(center - (frame.Width() / 2), frame.top);
				break;
			}
			case ALIGN_RIGHT_SIDES:
				view->MoveTo(baseRect.right - frame.Width(), frame.top);
				break;
			case ALIGN_H_CENTER_IN_WINDOW:
				break;
			case ALIGN_TOPS:
				view->MoveTo(frame.left, baseRect.top);
				break;
			case ALIGN_V_CENTERS:{
				float center = baseRect.top + (baseRect.Height() / 2);
				view->MoveTo(frame.left, center - (frame.Height() / 2));
				break;
			}
			case ALIGN_BOTTOMS:
				view->MoveTo(frame.left, baseRect.bottom - frame.Height());
				break;
			case ALIGN_V_CENTER_IN_WINDOW:
				break;
		}
	}
	Invalidate();
	ContainerView->SetDirty();
	UpdateObjectInspector();
}

int compareHFunc(const void *item1, const void *item2)
{
	const BView *p1 = (const BView *)*(const void**)item1;
	const BView *p2 = (const BView *)*(const void**)item2;
	if (p1->Frame().left > p2->Frame().left) { return 1; } else
	if (p1->Frame().left < p2->Frame().left) { return -1; }
	return 0;
}

void TUIView::AlignHSpaceEqually()
{
	if (fSelectedList.CountItems() < 3) return;
	
	BList list(fSelectedList);
	list.SortItems(&compareHFunc);
	
	BView *lview = (BView *)list.ItemAt(0);
	BView *rview = (BView *)list.ItemAt(list.CountItems() - 1);
	float d = (rview->Frame().left - lview->Frame().left) / (list.CountItems() - 1);
	float x = lview->Frame().left;
	for(int32 i=1; i < list.CountItems() - 1; i++) {
		BView *view = (BView *)list.ItemAt(i);
		x += d;
		view->MoveTo(x, view->Frame().top);
	}
	Invalidate();
	ContainerView->SetDirty();
	UpdateObjectInspector();
}

int compareVFunc(const void *item1, const void *item2)
{
	const BView *p1 = (const BView *)*(const void**)item1;
	const BView *p2 = (const BView *)*(const void**)item2;
	if (p1->Frame().top > p2->Frame().top) { return 1; } else
	if (p1->Frame().top < p2->Frame().top) { return -1; }
	return 0;
}

void TUIView::AlignVSpaceEqually()
{
	if (fSelectedList.CountItems() < 3) return;
	
	BList list(fSelectedList);
	list.SortItems(&compareVFunc);
	
	BView *tview = (BView *)list.ItemAt(0);
	BView *bview = (BView *)list.ItemAt(list.CountItems() - 1);
	float d = (bview->Frame().top - tview->Frame().top) / (list.CountItems() - 1);
	float y = tview->Frame().top;
	for(int32 i=1; i < list.CountItems() - 1; i++) {
		BView *view = (BView *)list.ItemAt(i);
		y += d;
		view->MoveTo(view->Frame().left, y);
	}
	Invalidate();
	ContainerView->SetDirty();
	UpdateObjectInspector();
}

void TUIView::ClearSelectedList()
{
	for(int32 i=0; i < fSelectedList.CountItems(); i++) {
		BView *view = (BView *)fSelectedList.ItemAt(i);
		InvalidateObject(view);
	}
	fSelectedList.MakeEmpty();
}

void TUIView::UpdateObjectInspector()
{
	BMessage msg(BERRY_OBJECT_SELECTED);
	for(int i=0; i<fSelectedList.CountItems(); i++) {
		msg.AddPointer("object", fSelectedList.ItemAt(i));
	}
	BMessenger msgr(ContainerView);
	msg.AddMessenger("messenger", msgr);
	SendToObjectInspector(&msg);
}

BRect TUIView::TopViewFrame(BView *child) const
{
	BRect rect = child->Frame();
	BView *parent = child->Parent();
	while(parent!=NULL){
		rect = parent->ConvertToParent(rect);
		child = parent;
		parent = child->Parent();
	}
	return rect;
}

void TUIView::DrawHandlerBox(BPoint point)
{
	BRect rect(_RECT(point.x, point.y));
	FillRect(rect);
}

void TUIView::Resizing(int resizingMode)
{
	if (LockedList.CountHasViews(&fSelectedList) > 0) return;
	
	SetDrawingMode(B_OP_INVERT);
	SetPenSize(0);
	
	BView *view = (BView *)fSelectedList.ItemAt(0);
	BRect rect = TopViewFrame(view);
	BRect rect0 = rect;
	BRect lastRect = rect;
	bool canceled = false;
	BPoint startPos, currentPos, lastPos, sub;
	uint32 buttons;
	key_info info;
	GetMouse(&currentPos, &buttons);
	startPos = currentPos;
	lastPos = currentPos;
	while(buttons==1 && !canceled){
		if (lastPos!=currentPos) {
			sub = currentPos - lastPos;
			StrokeRect(rect);
			switch(resizingMode) {
				case TOP_LEFT:
					rect.SetLeftTop(rect.LeftTop() + sub);
					break;
				case BTM_LEFT:
					rect.SetLeftBottom(rect.LeftBottom() + sub);
					break;
				case TOP_RIGHT:
					rect.SetRightTop(rect.RightTop() + sub);
					break;
				case BTM_RIGHT:
					rect.SetRightBottom(rect.RightBottom() + sub);
					break;
				case TOP_LEFT2:
					sub.x = 0;
					rect.SetLeftTop(rect.LeftTop() + sub);
					break;
				case BTM_LEFT2:
					sub.x = 0;
					rect.SetLeftBottom(rect.LeftBottom() + sub);
					break;
				case TOP2_LEFT:
					sub.y = 0;
					rect.SetLeftTop(rect.LeftTop() + sub);
					break;
				case TOP2_RIGHT:
					sub.y = 0;
					rect.SetRightTop(rect.RightTop() + sub);
					break;
			}
			if ((rect.Width() < 0) || (rect.Height() < 0)) {
				rect = lastRect;
			}
			lastRect = rect;
			StrokeRect(rect);
			lastPos = currentPos;
		}
		snooze(10000);
//		ProcessMessages();
		GetMouse(&currentPos, &buttons);
		// ESC キー押下でキャンセル
		get_key_info(&info);
		canceled = info.key_states[0] & 0x40;
	}
	
	if (!canceled && buttons==0) {
		BPoint point, point1;
		InvalidateObject(view);
		switch(resizingMode) {
			case BTM_RIGHT:
			case BTM_LEFT2:
			case TOP2_RIGHT:
				point = rect.RightBottom() - rect0.RightBottom();
				view->ResizeBy(point.x, point.y);
				break;
			case TOP_LEFT:
			case TOP_LEFT2:
			case TOP2_LEFT:
				point = rect.LeftTop() - rect0.LeftTop();
				view->MoveBy(point.x, point.y);
				point1 = rect0.RightBottom() - TopViewFrame(view).RightBottom();
				view->ResizeBy(point1.x, point1.y);
				break;
			case BTM_LEFT:
				view->MoveBy(rect.LeftTop().x - rect0.LeftTop().x, 0);
				view->ResizeBy(rect0.RightBottom().x - TopViewFrame(view).RightBottom().x, rect.bottom - rect0.bottom);
				break;
			case TOP_RIGHT:
				view->MoveBy(0, rect.LeftTop().y - rect0.LeftTop().y);
				view->ResizeBy(rect.right - rect0.right, rect0.RightBottom().y - TopViewFrame(view).RightBottom().y);
				break;
		}
		UpdateObjectInspector();
		InvalidateObject(view);
		ContainerView->SetDirty();
	} else {
		Invalidate();
	}
}

// 選択中の View を移動
void TUIView::Moving(BPoint cursor)
{
	if (LockedList.CountHasViews(&fSelectedList) > 0) return;
	
	SetDrawingMode(B_OP_INVERT);
	SetPenSize(0);
	
	BList rectList;
	for(int32 i=0; i < fSelectedList.CountItems(); i++) {
		BView *view = (BView *)fSelectedList.ItemAt(i);
		if (view == BGView) continue;
		rectList.AddItem(new BRect(TopViewFrame(view)));
	}
	
	bool canceled = false;
	BPoint startPos, currentPos, lastPos, sub;
	uint32 buttons;
	key_info info;
	GetMouse(&currentPos, &buttons);
	startPos = currentPos;
	lastPos = currentPos;
	while(buttons==1 && !canceled){
		if (lastPos!=currentPos) {
			sub = currentPos - lastPos;
			for(int32 i=0; i < rectList.CountItems(); i++) {
				BRect *rect = (BRect *)rectList.ItemAt(i);
				StrokeRect(*rect);
				rect->OffsetBy(sub.x, sub.y);
				StrokeRect(*rect);
			}
			lastPos = currentPos;
		}
		snooze(10000);
//		ProcessMessages();
		GetMouse(&currentPos, &buttons);
		// ESC キー押下でキャンセル
		get_key_info(&info);
		canceled = info.key_states[0] & 0x40;
	}
	
	if (!canceled && buttons==0) {
		BPoint point = lastPos - startPos;
		for(int32 i=0; i < rectList.CountItems(); i++) {
			BView *view = (BView *)fSelectedList.ItemAt(i);
			if (view == BGView) continue;
			InvalidateObject(view);
			view->MoveBy(point.x, point.y);
			InvalidateObject(view);
		}
		UpdateObjectInspector();
		ContainerView->SetDirty();
	} else {
		Invalidate();
	}
	
	for(int32 i=0; i < rectList.CountItems(); i++) {
		BRect *rect = (BRect *)rectList.ItemAt(i);
		delete rect;
	}
}

void TUIView::MakeDeleteMsg(BView *view, BMessage *delmsg)
{
	delmsg->AddPointer("object", view);
	for (int32 i=0; i < view->CountChildren(); i++) {
		BView *child = view->ChildAt(i);
		MakeDeleteMsg(child, delmsg);
		delmsg->AddPointer("object", child);
	}
}

void TUIView::SendGGMessage(BView *view, uint32 what)
{
	entry_ref ref;
	int32 id;
	((TResEditorWindow *)Window())->GetParameter(NULL, NULL, NULL, &id, &ref);
	BMessage archive;
	view->Archive(&archive);
	BMessage msg(what);
	msg.AddRef("refs", &ref);
	msg.AddInt32(GG_MSG_NAME_RESOURCE_ID, id);
	msg.AddString(GG_MSG_NAME_COMPONENT_NAME, view->Name());
	msg.AddMessage(GG_MSG_NAME_COMPONENT, &archive);
	BMessenger(gg_server).SendMessage(msg);
}

void TUIView::DeleteSelection()
{
	if (LockedList.CountHasViews(&fSelectedList) > 0) return;
	
	BMessage msg(BERRY_OBJECT_DELETED);
	bool deleted = false;
	for(int32 i=0; i<fSelectedList.CountItems(); i++) {
		BView *view = (BView *)fSelectedList.ItemAt(i);
		if (view != BGView) {
			SendGGMessage(view, GG_COMPONENT_REMOVED);	// GenesTation へ通知
			MakeDeleteMsg(view, &msg);
			view->Parent()->RemoveChild(view);
			LockedList.RemoveView(view);
			delete view;
			deleted = true;
		}
	}
	if (deleted) {
		SendToObjectInspector(&msg);
		fSelectedList.MakeEmpty();
		UpdateObjectInspector();
		Invalidate();
		ContainerView->SetDirty();
	}
}

void TUIView::SelectedViewsResizeBy(float x, float y)
{
	if (LockedList.CountHasViews(&fSelectedList) > 0) return;
	for(int32 i=0; i < fSelectedList.CountItems(); i++) {
		BView *view = (BView *)fSelectedList.ItemAt(i);
		view->ResizeBy(x, y);
	}
	Invalidate();
	ContainerView->SetDirty();
	UpdateObjectInspector();
}

void TUIView::SelectedViewsMoveBy(float x, float y)
{
	if (LockedList.CountHasViews(&fSelectedList) > 0) return;
	for(int32 i=0; i < fSelectedList.CountItems(); i++) {
		BView *view = (BView *)fSelectedList.ItemAt(i);
		view->MoveBy(x, y);
	}
	Invalidate();
	ContainerView->SetDirty();
	UpdateObjectInspector();
}

void TUIView::KeyDown(const char *bytes, int32 numBytes)
{
	if (numBytes < 0) return;
	uint32 key = modifiers();
	bool shift = key & B_SHIFT_KEY;
	bool control = key & B_CONTROL_KEY;
	float x = 0, y = 0;
	switch(bytes[0]) {
		case B_ENTER: ViewInvoked(); break;
		case B_LEFT_ARROW: x = -1; break;
		case B_RIGHT_ARROW: x = 1; break;
		case B_UP_ARROW: y = -1; break;
		case B_DOWN_ARROW: y = 1; break;
	}
	
	if (shift) {
		this->SelectedViewsResizeBy(x, y);
	} else if (control) {
		this->SelectedViewsMoveBy(x, y);
	} else {
		switch(bytes[0]) {
			case B_DELETE: {
				this->DeleteSelection();
				break;
			}
		}
	}
}

void TUIView::ViewInvoked()
{
	if (fSelectedList.CountItems() <= 0) return;
	BView *view = (BView *)fSelectedList.ItemAt(0);
	SendGGMessage(view, GG_COMPONENT_INVOKED);
}

/*
void TUIView::ProcessMessages()
{
	BMessageQueue *q = Window()->MessageQueue();
	BMessage *msg;
	q->Lock();
	while((msg = q->NextMessage())) {
		Window()->Lock();
		Window()->DispatchMessage(msg, this);
		Window()->Unlock();
		q->RemoveMessage(msg);
		delete msg;
	}
	q->Unlock();
}
*/

// TMaskView ------------------------------------------------------------------------------

TMaskView::TMaskView(BRect frame, const char *name, TUIView *uiView)
	:	BView(frame, name, B_FOLLOW_ALL_SIDES, B_FRAME_EVENTS)
{
	UIView = uiView;
}

TMaskView::~TMaskView()
{
}

void TMaskView::MouseDown(BPoint point)
{
	BPoint currentPos;
	uint32 buttons;
	Window()->Activate(true);
	GetMouse(&currentPos, &buttons);
	switch(buttons) {
		case 1:
			UIView->MouseDownLeft(point);
			break;
		case 2:
			UIView->MouseDownRight(point);
			break;
	}
	UIView->MakeFocus();
	Window()->WindowActivated(true);
}

void TMaskView::MessageReceived(BMessage *msg)
{
	switch(msg->what) {
		case B_SIMPLE_DATA:
			be_app_messenger.SendMessage(msg);
			break;
		case BERRY_BCL_DROPPED:
			BMessenger(UIView).SendMessage(msg);
			break;
	}
}
