#ifndef _CONTAINERVIEW_H
#define _CONTAINERVIEW_H

#include <View.h>
#include <Bitmap.h>
#include <Messenger.h>
#include <List.h>
#include <Box.h>
#include <Button.h>
#include <MenuBar.h>
#include <Menu.h>
#include <MenuItem.h>
#include <PopUpMenu.h>
#include "BCL.h"

class TUIView;

class TContainerView : public BView
{
public:
	TContainerView(BRect frame, const char *name);
	~TContainerView();
	void MessageReceived(BMessage *msg);
	status_t InstantiateFromData(const char *data, ssize_t numBytes);
	
	TUIView *UIView;
	BView *MaskView;
	BView *BGView;
	
	void SetDirty();
};

class TUIView : public BView
{
public:
	TUIView(BRect frame, const char *name, BView *bgView, TContainerView *containerView);
	~TUIView();
	
	void DetachedFromWindow();
	void MessageReceived(BMessage *msg);
	void Draw(BRect updateRect);
	void FrameResized(float width, float height);
	void KeyDown(const char *bytes, int32 numBytes);
	void SetUIView(BView *view) { BGView = view; };
	void SetContainerView(TContainerView *view) { ContainerView = view; };
	bool DrawSelecting(BRect *rect, BPoint cursor);
	void Selecting(BView *bgView, BPoint cursor, bool multi);
	void PutObject(BPoint point, TBerryComponentLibrary *t_bcl = NULL);
	void SendToObjectInspector(BMessage *msg);
	void MakeInstantiatedMsg(BView *parent, BMessage *instantiatedMsg);
	void MouseDownLeft(BPoint point);
	void MouseDownRight(BPoint point);
	void DeleteSelection();
	BView *FindView(BPoint point, bool deep, BView *view) const;
	void ClearSelectedList();
	void InvalidateObject(BView *view);
	void AddSelection(BView *view);
	void InvalidateSelection();
	void UpdateObjectInspector();
//	void ProcessMessages();
	
	void Cut();
	void Copy();
	void Paste();
	void SelectAll();
	
	void Align(uint32 alignCommand);
	void AlignHSpaceEqually();
	void AlignVSpaceEqually();
	
	void SelectedViewsResizeBy(float x, float y);
	void SelectedViewsMoveBy(float x, float y);
	
	BView *BGView;
	
	void ViewInvoked();
	
private:
	void DrawHandlerBox(BPoint point);
	BRect TopViewFrame(BView *child) const;
	void Moving(BPoint cursor);
	void Resizing(int resizingMode);
	void MakeDeleteMsg(BView *view, BMessage *delmsg);
	
	TContainerView *ContainerView;
	
	bool fDragging;
	BList fSelectedList;
	
	BPopUpMenu *fPopUpMenu;
	BMenu *fAlignMenu;
	BMenu *fHorizontalMenu;
	BMenu *fVerticalMenu;
	BMenuItem *fCutMenuItem;
	BMenuItem *fCopyMenuItem;
	BMenuItem *fPasteMenuItem;
	BMenuItem *fDeleteMenuItem;
	BMenuItem *fSelectAllMenuItem;
	
	void SendGGMessage(BView *view, uint32 what);
};

class TMaskView : public BView
{
public:
	TMaskView(BRect frame, const char *name, TUIView *uiView);
	~TMaskView();
	
	void MouseDown(BPoint point);
	void MessageReceived(BMessage *msg);
private:
	TUIView *UIView;
	
};

#endif
