#include <Application.h>
#include <Alert.h>
#include <stdio.h>
#include <MenuField.h>
#include <Roster.h>
#include <NetPositive.h>
#include "Berry.h"
#include "ContainerWindow.h"
#include "Locale.h"


#define VIEWIT_SIGNATURE "application/x-vnd.Geb-ViewIt"

TContainerWindow *last_activated_container;
TTestWindow *test_window = NULL;


// TTestWindow

TTestWindow::TTestWindow(const char *title)
	:	BWindow(BRect(100,100,200,200), title, B_TITLED_WINDOW, 0)
{
}

TTestWindow::~TTestWindow()
{
}

bool TTestWindow::QuitRequested()
{
	test_window = NULL;
	return true;
}


// TContainerWindow

TContainerWindow::TContainerWindow(BRect frame, const char *title)
	:	TResEditorWindow(frame, title, B_TITLED_WINDOW,
				B_WILL_ACCEPT_FIRST_CLICK | B_NOT_ZOOMABLE)
{
	test_window = NULL;
	
	fContainerView = new TContainerView(Bounds(), "ContainerView");
	AddChild(fContainerView);
	
	AddShortcut('O', 0, new BMessage(GB_LOAD));
	AddShortcut('S', 0, new BMessage(GB_SAVE));
	AddShortcut('R', 0, new BMessage(GB_TEST_RUN));
	AddShortcut('B', 0, new BMessage(GB_OPEN_VIEWIT));
	
	AddShortcut('X', 0, new BMessage(B_CUT), fContainerView);
	AddShortcut('C', 0, new BMessage(B_COPY), fContainerView);
	AddShortcut('V', 0, new BMessage(B_PASTE), fContainerView);
	AddShortcut('A', 0, new BMessage(B_SELECT_ALL), fContainerView);
}

TContainerWindow::~TContainerWindow()
{
	
	if (last_activated_container == this) last_activated_container = NULL;
}

void TContainerWindow::Quit()
{
	SendData();
	BWindow::Quit();
}

void TContainerWindow::MessageReceived(BMessage *msg)
{
	switch(msg->what) {
		case B_SIMPLE_DATA:
			be_app_messenger.SendMessage(msg);
			break;
		case GB_TEST_RUN:
			TestRun();
			break;
		case GB_OPEN_VIEWIT:
			OpenViewIt();
			break;
		default: BWindow::MessageReceived(msg);
	}
}

void TContainerWindow::GetArchive(BMessage *archive)
{
	fContainerView->BGView->Archive(archive);
}

status_t TContainerWindow::InstantiateFromData(const void *data, ssize_t numBytes)
{
	return fContainerView->InstantiateFromData((const char *)data, numBytes);
}

void TContainerWindow::WindowActivated(bool active)
{
	if (active) {
		last_activated_container = this;
		fContainerView->UIView->UpdateObjectInspector();
	}
}

void TContainerWindow::TestRun()
{
	if (test_window != NULL) {
		BMessenger(test_window).SendMessage(B_QUIT_REQUESTED);
		test_window = NULL;
	}
	
	BMessage archive;
	last_activated_container->Lock();
	last_activated_container->GetArchive(&archive);
	last_activated_container->Unlock();
	
//	archive.PrintToStream();
	BArchivable *arc = instantiate_object(&archive);
	if (arc == NULL) return;
	
	BView *BGView = dynamic_cast<BView *>(arc);
	if (BGView == NULL) return;
	
	BString title(Title());
	title << " (TestRun)";
	test_window = new TTestWindow(title.String());
	test_window->ResizeTo(BGView->Frame().Width(), BGView->Frame().Height());
	test_window->AddChild(BGView);
	
	test_window->Show();
}

void TContainerWindow::OpenViewIt()
{
	if (last_activated_container == NULL) return;
	
	BMessage archive;
	last_activated_container->Lock();
	last_activated_container->GetArchive(&archive);
	last_activated_container->Unlock();
	
	if (be_roster->IsRunning(VIEWIT_SIGNATURE)) {
		BMessenger(VIEWIT_SIGNATURE).SendMessage(&archive);
		be_roster->ActivateApp(be_roster->TeamFor(VIEWIT_SIGNATURE));
	} else if (be_roster->Launch(VIEWIT_SIGNATURE, &archive) != B_OK) {
		BAlert *alert = new BAlert(APP_NAME,
							GET_STRING("UnableToOpenViewIt",
								"Unable to open ViewIt.\n"
								"Do you want to visit ViewIt website?"),
							GET_STRING("No", "No"),
							GET_STRING("VisitSite", "Visit Site"), NULL,
							B_WIDTH_AS_USUAL, B_IDEA_ALERT);
		if (alert->Go() == 1) {
			if(!be_roster->IsRunning(B_NETPOSITIVE_APP_SIGNATURE)) {
				be_roster->Launch(B_NETPOSITIVE_APP_SIGNATURE);
				while (!be_roster->IsRunning(B_NETPOSITIVE_APP_SIGNATURE)) { snooze(20000); }
			}
			BMessage msg(B_GET_PROPERTY), reply;
			BMessenger msgr(B_NETPOSITIVE_APP_SIGNATURE), wmsgr;
			msg.AddSpecifier("Messenger");
			msg.AddSpecifier("Window", (int32)0);
			msgr.SendMessage(&msg, &reply);
			if (reply.FindMessenger("result", &wmsgr) != B_OK) {
				(new BAlert(APP_NAME,
							GET_STRING("UnableToOpenNetP", "Unable to open NetPositive."),
							GET_STRING("Close", "Close"), NULL, NULL,
							B_WIDTH_AS_USUAL, B_STOP_ALERT))->Go();
				return;
			}
			
			msg.MakeEmpty();
			msg.what = B_NETPOSITIVE_OPEN_URL;
			msg.AddString("be:url", "http://www.bebits.com/app/315");
			wmsgr.SendMessage(&msg);
			be_roster->ActivateApp(be_roster->TeamFor(B_NETPOSITIVE_APP_SIGNATURE));
		}
	}
}

void TContainerWindow::GetData(BMallocIO *data)
{
	BMessage archive;
	GetArchive(&archive);
	data->SetSize(0);
	archive.Flatten(data);
}
