#include <stdio.h>
#include <stdlib.h>
#include "EditorWindow.h"


enum {
	EW_APPLY = 'ewap',
	EW_CANCEL = 'ewca',
};

// TEditorView

TEditorView::TEditorView(BRect frame, const char *name)
	:	BView(frame, name, B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FRAME_EVENTS)
{
}

TEditorView::~TEditorView()
{
}


// TEditorWindow

TEditorWindow::TEditorWindow(BRect frame, const char *title,
			type_code type, TResAddon *addon)
	:	TResEditorWindow(frame, title,
			B_TITLED_WINDOW, B_NOT_ZOOMABLE | B_NOT_RESIZABLE)
{
	fType = type;
	fAddon = addon;
	
	fEditorView = new TEditorView(Bounds(), "EditorView");
	fEditorView->SetViewColor(216, 216, 216);
	AddChild(fEditorView);
	
	fEditorTextControl = new BTextControl(BRect(10, 10, 220, 20),
			"EditorTextControl", "Value", "", NULL);
	fEditorTextControl->SetDivider(40);
	fEditorView->AddChild(fEditorTextControl);
	fApplyButton = new BButton(BRect(90, 40, 150, 55),
			"Apply", "Apply", new BMessage(EW_APPLY));
	fApplyButton->MakeDefault(true);
	fEditorView->AddChild(fApplyButton);
	fCancelButton = new BButton(BRect(160, 40, 220, 55),
			"Cancel", "Cancel", new BMessage(EW_CANCEL));
	fEditorView->AddChild(fCancelButton);
	
	fEditorTextControl->TextView()->MakeFocus();
	
	fData.SetTo("");
	printf("111\n");
}

TEditorWindow::~TEditorWindow()
{
}

void TEditorWindow::MessageReceived(BMessage *msg)
{
	switch(msg->what) {
		case EW_APPLY:
			fData.SetTo(fEditorTextControl->Text());
			SendData();
			break;
		case EW_CANCEL:
			Quit();
			break;
		default: BWindow::MessageReceived(msg);
	}
}

status_t TEditorWindow::InstantiateFromData(const void *data,
		ssize_t numBytes)
{
	BString str;
	switch(fType){
		case 'CHAR': {
			str << *(char *)data;
			break;
		}
		case 'FLOT': {
			str << *(float *)data;
			break;
		}
		case 'DBLE': {
			char s[64];
			sprintf(s, "%.16f", *(double *)data);
			str.SetTo(s);
			break;
		}
		case 'SHRT': {
			str << *(int16 *)data;
			break;
		}
		case 'USHT': {
			int32 d = *(uint16 *)data;
			str << d;
			break;
		}
		case 'LONG': {
			str << *(int32 *)data;
			break;
		}
		case 'ULNG': {
			str << *(uint32 *)data;
			break;
		}
		case 'LLNG': {
			str << *(int64 *)data;
			break;
		}
		case 'ULLG': {
			str << *(uint64 *)data;
			break;
		}
		case 'BYTE': {
			str << *(int8 *)data;
			break;
		}
		case 'UBYT': {
			int16 d = *(uint8 *)data;
			str << d;
			break;
		}
		case 'MIMS': {
			str.SetTo((const char *)data, numBytes);
			break;
		}
		case 'OFFT': {
			str << *(int64 *)data;
			break;
		}
		case 'PNTR': {
			str << *(uint32 *)data;
			break;
		}
		case 'SIZT': {
			str << *(size_t *)data;
			break;
		}
		case 'SSZT': {
			str << *(status_t *)data;
			break;
		}
		case 'TIME': {
			str << *(time_t *)data;
			break;
		}
		default: fEditorTextControl->SetText("");
	}
	fEditorTextControl->SetText(str.String());
	fData.SetTo(str.String());
	fEditorTextControl->TextView()->SelectAll();
}

void TEditorWindow::GetData(BMallocIO *data)
{
	data->SetSize(0);
	const char *text = fData.String();
	size_t length = fEditorTextControl->TextView()->TextLength();
	switch(fType) {
		case 'CHAR': {
			const char d = atoi(text);
			data->Write(&d, sizeof(d));
			break;
		}
		case 'FLOT': {
			const float d = atof(text);
			data->Write(&d, sizeof(d));
			break;
		}
		case 'DBLE': {
			const double d = atof(text);
			data->Write(&d, sizeof(d));
			break;
		}
		case 'SHRT': {
			const int16 d = atoi(text);
			data->Write(&d, sizeof(d));
			break;
		}
		case 'USHT': {
			const uint16 d = atoi(text);
			data->Write(&d, sizeof(d));
			break;
		}
		case 'LONG': {
			const int32 d = atoi(text);
			data->Write(&d, sizeof(d));
			break;
		}
		case 'ULNG': {
			const uint32 d = atoi(text);
			data->Write(&d, sizeof(d));
			break;
		}
		case 'LLNG': {
			const int64 d = atoi(text);
			data->Write(&d, sizeof(d));
			break;
		}
		case 'ULLG': {
			const uint64 d = atoi(text);
			data->Write(&d, sizeof(d));
			break;
		}
		case 'BYTE': {
			const int8 d = atoi(text);
			data->Write(&d, sizeof(d));
			break;
		}
		case 'UBYT': {
			const uint8 d = atoi(text);
			data->Write(&d, sizeof(d));
			break;
		}
		case 'MIMS': {
			data->Write(text, length);
			break;
		}
		case 'OFFT': {
			const int64 d = atoi(text);
			data->Write(&d, sizeof(d));
			break;
		}
		case 'PNTR': {
			const int d = atoi(text);
			data->Write(&d, sizeof(d));
			break;
		}
		case 'SIZT': {
			const size_t d = atoi(text);
			data->Write(&d, sizeof(d));
			break;
		}
		case 'SSZT': {
			const status_t d = atoi(text);
			data->Write(&d, sizeof(d));
			break;
		}
		case 'TIME': {
			const time_t d = atoi(text);
			data->Write(&d, sizeof(d));
			break;
		}
		default: ;
	}
	BString str;
	fAddon->String(data, &str);
	fEditorTextControl->SetText(str.String());
	fEditorTextControl->TextView()->SelectAll();
}
