#include <stdio.h>
#include <stdlib.h>
#include "GGServer.h"
#include "Berry.h"


TGGServer *gg_server;


TGGServer::TGGServer()
	:	BLooper()
{
	
}

TGGServer::~TGGServer()
{
	for(int32 i = 0; i < fGGList.CountItems(); i++) {
		delete ((TGGItem *)fGGList.ItemAt(i));
	}
}

void TGGServer::MessageReceived(BMessage *msg)
{
	switch(msg->what) {
		case GG_START_COMMUNICATION:
			fGGStartCommunication(msg);
			break;
		case GG_END_COMMUNICATION:
			fGGEndCommunication(msg);
			break;
		case GG_OPEN_VIEW_REQUESTED:
			fGGOpenViewRequested(msg);
			break;
		case GG_SEND_VIEW_REQUESTED:
			fGGSendViewRequested(msg);
			break;
		case GG_COMPONENT_ADDED:
		case GG_COMPONENT_REMOVED:
		case GG_COMPONENT_SELECTION_CHANGED:
		case GG_COMPONENT_INVOKED:
		case GG_COMPONENT_NAME_CHANGED:
			fGGComponentChanged(msg);
			break;
		case GG_SAVE_ALL_REQUESTED:
			ffGGSaveAllRequested(msg);
			break;
		default: BLooper::MessageReceived(msg);
	}
}

// GenesTation と接続開始
void TGGServer::fGGStartCommunication(BMessage *msg)
{
	// GenesTation 宛先取得
	msg->FindMessenger("Dest", &fGGMessenger);
	
	// 返信
	BMessage aReplyMsg(GG_START_COMMUNICATION_REPLY);
	aReplyMsg.AddInt32(GG_MSG_NAME_RESULT, GG_OK);
	aReplyMsg.AddMessenger(GG_MSG_NAME_DESTINATION, this);	// 次回からの BMessenger 送り先
	aReplyMsg.AddString(GG_MSG_NAME_VERSION, GUIBERRY_VERSION);
	msg->SendReply(&aReplyMsg);
}

void TGGServer::fGGEndCommunication(BMessage *msg)
{
}

TResManager *TGGServer::fFindManager(BEntry *entry)
{
	TResManager *manager;
	for(int32 i = 0; i < res_manager_list.CountItems(); i++) {
		manager = (TResManager *)res_manager_list.ItemAt(i);
		if (*manager->Entry() == *entry) return manager;
	}
	return NULL;
}

// GenesTation からリソースファイル指定が来た
void TGGServer::fGGOpenViewRequested(BMessage *msg)
{
	TResManager *manager;
	BMessage aReplyMsg(GG_OPEN_VIEW_REQUESTED_REPLY);
	entry_ref ref;
	int32 id;
	
	if ((msg->FindRef("refs", &ref) != B_OK) || (msg->FindInt32("RsrcID", &id))) {
		aReplyMsg.AddInt32(GG_MSG_NAME_RESULT, GG_ERR_GENERAL);
		msg->SendReply(&aReplyMsg);
		return;
	}
	
	BEntry entry(&ref);
	if (entry.InitCheck() != B_OK) {
		aReplyMsg.AddInt32(GG_MSG_NAME_RESULT, GG_ERR_ENTRY_NOT_FOUND);
		msg->SendReply(&aReplyMsg);
		return;
	}
	
	manager = fFindManager(&entry);
	
	if (!manager) {
		char name[B_FILE_NAME_LENGTH];
		entry.GetName(name);
		manager = new TResManager(BRect(260, 200, 760, 400), name);
		printf("GuiBerry: Opening '%s' for GenesTation.\n", name);
		if (manager->OpenResource(&entry) != B_OK) {
			delete manager;
			aReplyMsg.AddInt32(GG_MSG_NAME_RESULT, GG_ERR_GENERAL);
			msg->SendReply(&aReplyMsg);
			return;
		}
	}
	
	manager->Show();
	manager->Activate(true);
	if (manager->OpenEditor('ARCV', id) != B_OK) {
		aReplyMsg.AddInt32(GG_MSG_NAME_RESULT, GG_ERR_RESOURCE_ID_NOT_FOUND);
		msg->SendReply(&aReplyMsg);
		return;
	}
	
	aReplyMsg.AddInt32(GG_MSG_NAME_RESULT, GG_OK);
	msg->SendReply(&aReplyMsg);
	
	if (FindGGItem(&ref, id) < 0) {
		TGGItem *item = new TGGItem();
		item->gi_ref = ref;
		item->gi_id = id;
		fGGList.AddItem(item);
	}
	
}

void TGGServer::fGGSendViewRequested(BMessage *msg)
{
	TResManager *manager;
	BMessage aReplyMsg(GG_SEND_VIEW_REQUESTED_REPLY);
	entry_ref ref;
	int32 id;
	
	if ((msg->FindRef("refs", &ref) != B_OK) || (msg->FindInt32("RsrcID", &id))) {
		aReplyMsg.AddInt32(GG_MSG_NAME_RESULT, GG_ERR_GENERAL);
		msg->SendReply(&aReplyMsg);
		return;
	}
	
	BEntry entry(&ref);
	if (entry.InitCheck() != B_OK) {
		aReplyMsg.AddInt32(GG_MSG_NAME_RESULT, GG_ERR_ENTRY_NOT_FOUND);
		msg->SendReply(&aReplyMsg);
		return;
	}
	
	manager = fFindManager(&entry);
	
	if (!manager) {
		aReplyMsg.AddInt32(GG_MSG_NAME_RESULT, GG_ERR_ENTRY_NOT_FOUND);
		msg->SendReply(&aReplyMsg);
		return;
	}
	
	size_t size;
	const void *data;
	if (manager->GetData('ARCV', id, &data, &size) != B_OK) {
		aReplyMsg.AddInt32(GG_MSG_NAME_RESULT, GG_ERR_RESOURCE_ID_NOT_FOUND);
		msg->SendReply(&aReplyMsg);
		return;
	}
	
	BMessage archive;
	if (archive.Unflatten((const char *)data) != B_OK) {
		aReplyMsg.AddInt32(GG_MSG_NAME_RESULT, GG_ERR_RESOURCE_ID_NOT_FOUND);
		msg->SendReply(&aReplyMsg);
		return;
	}
	
	BString bgViewName;
	archive.FindString("_name", &bgViewName);
	
	aReplyMsg.AddInt32(GG_MSG_NAME_RESULT, GG_OK);
	aReplyMsg.AddString(GG_MSG_NAME_COMPONENT_NAME, bgViewName.String());
	aReplyMsg.AddMessage(GG_MSG_NAME_COMPONENT, &archive);
	msg->SendReply(&aReplyMsg);
}

void TGGServer::fGGComponentChanged(BMessage *msg)
{
	entry_ref ref;
	int32 id;
	if (msg->FindRef("refs", &ref) != B_OK) return;
	if (msg->FindInt32(GG_MSG_NAME_RESOURCE_ID, &id) != B_OK) return;
	if (FindGGItem(&ref, id) >= 0) fGGMessenger.SendMessage(msg);
}

int32 TGGServer::FindGGItem(entry_ref *ref, int32 id) const
{
	for(int32 i = 0; i < fGGList.CountItems(); i++) {
		TGGItem *item = (TGGItem *)fGGList.ItemAt(i);
		if ((item->gi_ref == *ref) && (item->gi_id == id))return i;
	}
	return -1;
}

void TGGServer::ffGGSaveAllRequested(BMessage *msg)
{
	TResManager *manager;
	for(int32 i = 0; i < fGGList.CountItems(); i++) {
		TGGItem *item = (TGGItem *)fGGList.ItemAt(i);
		BEntry entry(&item->gi_ref);
		manager = fFindManager(&entry);
		if (manager) {
			manager->Lock();
			manager->Save();
			manager->Unlock();
		}
	}
	
	BMessage aReplyMsg(GG_SAVE_ALL_REQUESTED_REPLY);
	aReplyMsg.AddInt32(GG_MSG_NAME_RESULT, GG_OK);
	msg->SendReply(&aReplyMsg);
}
