#include <Screen.h>
#include <Alert.h>
#include <Directory.h>
#include <Entry.h>
#include <stdlib.h>
#include <stdio.h>
#include <image.h>
#include <DataIO.h>
#include <Mime.h>
#include <Path.h>

#include "BerryUtils.h"
#include "GuiBerry.h"
#include "Locale.h"
#include "GGShared.h"


BList AddonList;
BList BCLList;

int main()
{
	TGuiBerry app;
	app.Run();
	return 0;
}


TGuiBerry::TGuiBerry() : BApplication("application/x-vnd.momoziro-guiberry")
{
#if B_BEOS_VERSION < 0x0520
	be_locale = new TZetaLocale("Language/Dictionaries/GuiBerry", "jaJP");
#endif

	BString addonPath;
	GetAddonDir("bcl", &addonPath);
	fLoadAddons(addonPath.String(), &AddonList);
	
	// Main Window
	BScreen screen;
	fMainWindow = new TMainWindow(BRect(260, 90, screen.Frame().right * 0.8, 162), "GuiBerry");
	fMainWindow->Show();
	
	// Object Inspector Window
	float bottom = max_c(400, screen.Frame().bottom * 0.7);
	fObjectInspectorWindow = new TObjectInspectorWindow(BRect(5, 90, 245, bottom), "Object Inspector");
	fObjectInspectorWindow->Show();
	
	// Load Panel
	fLoadPanel = new BFilePanel(B_OPEN_PANEL, new BMessenger(this), NULL, 0, false);
	
	// GGServer
	gg_server = new TGGServer();
	gg_server->Run();
	
	fResourceCount = 0;
}

TGuiBerry::~TGuiBerry()
{
	delete fLoadPanel;
	BMessenger(gg_server).SendMessage(B_QUIT_REQUESTED);
	fUnloadAddons();
#if B_BEOS_VERSION < 0x0520
	delete be_locale;
#endif
}

void TGuiBerry::AboutRequested()
{
	for(int32 i=0; i < be_app->CountWindows(); i++) {
		if (fAboutWindow == be_app->WindowAt(i)) {
			fAboutWindow->Show();
			fAboutWindow->Activate(true);
			return;
		}
	}
	BScreen screen;
	BRect rect;
	float center_x = screen.Frame().right / 2;
	float center_y = screen.Frame().bottom / 2;
	rect.left = center_x - 175;
	rect.right = center_x + 175;
	rect.top = center_y - 75;
	rect.bottom = center_y + 75;
	fAboutWindow = new TAboutWindow(rect);
	fAboutWindow->Show();
}

void TGuiBerry::RefsReceived(BMessage *msg)
{
	TResManager *new_manager;
	
	for(int32 i=0; ; i++) {
		bool cont = false;
		entry_ref ref;
		if (msg->FindRef("refs", i, &ref) != B_OK) return;
		
		BEntry entry(&ref);
		if (entry.InitCheck() != B_OK) continue;
		
		for(int32 i=0; i < res_manager_list.CountItems(); i++) {
			TResManager *manager = (TResManager *)res_manager_list.ItemAt(i);
			if (*manager->Entry() == entry) {
				manager->Activate(true);
				cont = true;
			}
		}
		if (cont) continue;
		
		char name[B_FILE_NAME_LENGTH];
		entry.GetName(name);
		new_manager = new TResManager(BRect(260, 200, 760, 400), name);
		printf("GuiBerry: Opening '%s'\n", name);
		if (new_manager->OpenResource(&entry) != B_OK) {
			delete new_manager;
			continue;
		}
		new_manager->Show();
	}
}

void TGuiBerry::MessageReceived(BMessage *msg)
{
	switch(msg->what) {
		case B_SIMPLE_DATA:
			RefsReceived(msg);
			break;
		case RM_MODIFIED:
			BMessenger(fResManager).SendMessage(msg);
			break;
		case GB_LOAD:
			fLoadPanel->Show();
			break;
		case GB_SAVE:
		case GB_TEST_RUN:
		case GB_OPEN_VIEWIT:
			BMessenger(fMainWindow).SendMessage(msg);
			break;
		case GB_NEW_RESOURCE:
			NewResource();
			break;
		case GG_START_COMMUNICATION:
			BMessenger(gg_server).SendMessage(msg);
			//fGGServer->GGStartCommunication(msg);
			break;
		default: BApplication::MessageReceived(msg);
	}
}

void TGuiBerry::ReadyToRun()
{
	if (res_manager_list.CountItems() == 0) NewResource();
}

void TGuiBerry::NewResource()
{
	fResourceCount++;
	BString newTitle("Untitled ");
	newTitle << fResourceCount << ".rsrc";
	TResManager *new_manager = new TResManager(BRect(260, 200, 760, 400), newTitle.String());
	new_manager->Show();
}

void TGuiBerry::fLoadAddons(const char *addonsDir, BList *addonList)
{
	BDirectory dir(addonsDir);
	BEntry entry;
	
	dir.Rewind();
	while (dir.GetNextEntry(&entry) == B_OK) {
		entry_ref ref;
		entry.GetRef(&ref);
		fLoadAddon(&ref, addonList);
	}
}

void TGuiBerry::fLoadAddon(const entry_ref *ref, BList *addonList)
{
	void *(*instantiate_objects)(instantiation_info_t *info, register_component *RegisterComponent);
	TPropEditor *(*register_prop_editors)(const type_info *typeInfo, propeditor_info_t *peInfo);
	TAddonItem *addonItem;
	image_id addon_image_id;
	status_t er1;
	
	BPath path(ref);
	addon_image_id = load_add_on(path.Path());
	if (addon_image_id < 0) {
		fprintf(stderr, APP_NAME ": load_add_on(%s) failed(%d)\n", path.Path(), (int)addon_image_id);
		return;
	}
	
	er1 = get_image_symbol(addon_image_id, "instantiate_objects", B_SYMBOL_TYPE_TEXT, (void **)&instantiate_objects);
	if (er1 != B_NO_ERROR) {
		unload_add_on(addon_image_id);
		fprintf(stderr, APP_NAME ": Unloaded. %s is not BCL add-on. (%d)\n", path.Path(), (int)er1 );
		return;
	}
	
	er1 = get_image_symbol(addon_image_id, "register_prop_editors", B_SYMBOL_TYPE_TEXT, (void **)&register_prop_editors);
	if (er1 != B_NO_ERROR) {
		fprintf(stderr, APP_NAME ": register_prop_editors() not found.\n");
	}
	
	fprintf(stderr, APP_NAME ": Addon(%s)\n", path.Path());
	addonItem = (TAddonItem *)malloc(sizeof(TAddonItem));
	addonItem->instantiate_objects = instantiate_objects;
	addonItem->register_prop_editors = register_prop_editors;
	addonItem->id = addon_image_id;
	addonList->AddItem(addonItem);
}

void TGuiBerry::fUnloadAddons()
{
	for(int32 i=0; i<BCLList.CountItems(); i++) {
		delete (TBerryComponentLibrary *)BCLList.ItemAt(i);
	}
	
	for(int32 i=0; i<AddonList.CountItems(); i++) {
		TAddonItem *item = (TAddonItem *)AddonList.ItemAt(i);
		status_t status = unload_add_on(item->id);
		if (status!=B_OK) {
			fprintf(stderr, APP_NAME ": unload_add_on() error. (%d)\n", (int)status);
		}
		free(item);
	}
}
