#include <String.h>
#include "Locale.h"

const char *get_local_string_d(const char* key, const char* defaultString)
{
	BString yt_key(key);
	const char *yt_str = GETSTRING(yt_key.String());
	if (strcmp(yt_str, yt_key.String())==0) {
		return defaultString;
	} else {
		return yt_str;
	}
}

#if B_BEOS_VERSION < 0x0520
#include <Application.h>
#include <Roster.h>
#include <Path.h>
#include <stdio.h>

TZetaLocale *be_locale;

const char* get_local_string(const char* key)
{
	return be_locale->GetString(key);
}

TZetaLocale::TZetaLocale(const char *name, const char *lang)
 : BMallocIO()
{
	if (name) LoadLanguageFile(name, lang);
}

void TZetaLocale::LoadLanguageFile(const char *name, const char *lang)
{
	app_info ai;
	be_app->GetAppInfo(&ai);
	BPath path(&ai.ref);
	if (path.InitCheck() == B_NO_ERROR) {
		path.GetParent(&path);
		path.Append(name);
		SetTo(path.Path(), lang);
	}
}

void TZetaLocale::SetTo(const char *path, const char *lang)
{
	SetSize(0);
	BString fpath(path);
	fpath.Append(".");
	fpath.Append(lang);
	FILE *file = fopen(fpath.String(), "r");
	if (file) {
		char data[1024];
		while (!feof(file)) {
			fgets(data, 1024, file);
			if (data[0] && data[0] != '#') {
				int s = 0;
				for (s= 0; data[s]; s++) {
					if (data[s] == ' ' || data[s] == '\t') {
						Write(data, ++s);
						while (data[s]) {
							if (data[s] == ' ' || data[s] == '\t') s++; else break;
						}
						if (data[s] == '"') s++;
						int e = strlen(data);
						if (data[e-1] == '\n') data[--e] = '\0';
						if (data[e-1] == '"' ) data[--e] = '\0';
						data[e] = '\0';
						Write(data+s, e-s+1);
						break;
					}
				}
			}
		}
		fclose(file);
	}
}

const char *TZetaLocale::GetString(const char* msg)
{
	int size = strlen(msg);
	const char *data = (const char*)Buffer();
	size_t s= 0, length = BufferLength();
	while (s < length) {
		if (strncmp(msg, data+s, size) == 0 && data[s+size] == '\t') {
			msg = data+s+size+1;
			break;
		} else {
			s += strlen(data+s)+1;
		}
	}
	return msg;
}

#endif
