#include <stdio.h>
#include <File.h>
#include <Resources.h>
#include <TranslationUtils.h>
#include <List.h>
#include <Bitmap.h>
#include <Roster.h>
#include <Application.h>
#include <ctype.h>

#include "BerryUtils.h"
#include "MainView.h"
#include "BCLPictureButton.h"
#include "BCLView.h"
#include "SimplePictureButton.h"
#include "ToolTip.h"
#include "Locale.h"


TMainView *main_view;
BList bcl_picture_button_list;
BPicture *default_on_icon, *default_off_icon;


void RegisterComponent(const char *page, TBerryComponentLibrary *bcl);


void RegisterComponent(const char *page, TBerryComponentLibrary *bcl)
{
	BString out;
	const char *rawName = bcl->SupportedClass();
	if (isdigit(rawName[0])) {
		out.Append(rawName, GetNumber(rawName));
	} else {
		out.SetTo(rawName);
	}
	
	BCLList.AddItem(bcl);
	
	if (page==NULL) {
		fprintf(stderr, APP_NAME ": %s is abstract.\n", out.String());
	} else {
		fprintf(stderr, APP_NAME ": Adding component. (%s)\n", out.String());
		
		image_info info;
		if (get_image_info(bcl->Image(), &info) != B_OK) {
			fprintf(stderr, APP_NAME ": get_image_info() failed.  image_id = %d\n", (int)bcl->Image());
			return;
		}
		
		BFile self(info.name, B_READ_ONLY);
		BResources res;
		if (res.SetTo(&self) != B_OK) {
			fprintf(stderr, APP_NAME ": BResources initialize failed. (%s)\n", info.name);
			return;
		}
		
		size_t outSize;
		const void *iconImage = res.LoadResource('BMP ', out.String(), &outSize);
		iconImage = res.LoadResource('BMP ', out.String(), &outSize);
		if (iconImage != NULL) {
			BMemoryIO stream(iconImage, outSize);
			BBitmap *bitmap = BTranslationUtils::GetBitmap(&stream);
			BPicture *off, *on;
			TBCLPictureButton::MakeOnOffPicture(bitmap, &on, &off);
			TBCLPictureButton *button = new TBCLPictureButton(out.String(), page, off, on, bcl);
			bcl_picture_button_list.AddItem(button);
		} else {
			fprintf(stderr, APP_NAME ": Icon image not found.\n");
			TBCLPictureButton *button = new TBCLPictureButton(out.String(), page,
				default_off_icon, default_on_icon, bcl);
			bcl_picture_button_list.AddItem(button);
		}
	}
}

TMainView::TMainView(
			BRect frame,
			const char *name,
			uint32 resizingMode,
			uint32 flags)
	:	BView(frame, name, resizingMode, flags)
{
	// Main Menu
	fMenuBar = new BMenuBar(BRect(0,0,0,0), "MainMenu");
	this->AddChild(fMenuBar);
	
	// File Menu
	fFileMenu = new BMenu("File");
	fMenuBar->AddItem(fFileMenu);
	
	// New Resource MenuItem
	fNewResourceMenuItem = new BMenuItem("New Resource", new BMessage(GB_NEW_RESOURCE), 'N', 0);
	fNewResourceMenuItem->SetTarget(be_app);
	fFileMenu->AddItem(fNewResourceMenuItem);
	
	fFileMenu->AddSeparatorItem();
	
	// Load MenuItem
	fLoadMenuItem = new BMenuItem("Open", new BMessage(GB_LOAD), 'O', 0);
	fFileMenu->AddItem(fLoadMenuItem);
	
	// Save MenuItem
	fSaveMenuItem = new BMenuItem("Save", new BMessage(GB_SAVE), 'S', 0);
	fFileMenu->AddItem(fSaveMenuItem);
	
	// SaveAs MenuItem
	fSaveAsMenuItem = new BMenuItem("Save As...", new BMessage(GB_SAVEAS));
	fFileMenu->AddItem(fSaveAsMenuItem);
	
	fFileMenu->AddSeparatorItem();
	
	// About MenuItem
	fAboutMenuItem = new BMenuItem("About " APP_NAME, new BMessage(B_ABOUT_REQUESTED));
	fAboutMenuItem->SetTarget(be_app);
	fFileMenu->AddItem(fAboutMenuItem);
	
	fFileMenu->AddSeparatorItem();
	
	// Quit MenuItem
	fQuitMenuItem = new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED), 'Q', 0);
	fFileMenu->AddItem(fQuitMenuItem);
	
	// Debugger Menu
	fDebuggerMenu = new BMenu("Debugger");
	fMenuBar->AddItem(fDebuggerMenu);
	
	// TestRun MenuItem
	fTestRunMenuItem = new BMenuItem("Test Run", new BMessage(GB_TEST_RUN), 'R', 0);
	fDebuggerMenu->AddItem(fTestRunMenuItem);
	
	// Open ViewIt MenuItem
	fOpenViewItItem = new BMenuItem("Open ViewIt", new BMessage(GB_OPEN_VIEWIT), 'B', 0);
	fDebuggerMenu->AddItem(fOpenViewItItem);
	
	// New Button
	fNewButton = new TSimplePictureButton(BRect(2, 23, 25, 46), "New",
		"NEW_UP", 'BMP ', "NEW_DOWN", 'BMP ', new BMessage(GB_NEW_RESOURCE));
	fNewButton->SetTarget(be_app);
	AddChild(fNewButton);
	
	// Open Button
	fOpenButton = new TSimplePictureButton(BRect(27, 23, 50, 46), "Open",
		"LOAD_UP", 'BMP ', "LOAD_DOWN", 'BMP ', new BMessage(GB_LOAD));
	AddChild(fOpenButton);
	
	// Save Button
	fSaveButton = new TSimplePictureButton(BRect(52, 23, 75, 46), "Save",
		"SAVE_UP", 'BMP ', "SAVE_DOWN", 'BMP ', new BMessage(GB_SAVE));
	AddChild(fSaveButton);
	
	// Test Run Button
	fTestRunButton = new TSimplePictureButton(BRect(2, 47, 25, 70), "Test Run",
		"TESTRUN_UP", 'BMP ', "TESTRUN_DOWN", 'BMP ', new BMessage(GB_TEST_RUN));
	AddChild(fTestRunButton);
	
	// Open ViewIt Button
	fOpenViewItButton = new TSimplePictureButton(BRect(27, 47, 50, 70), "Open ViewIt",
		"VIEWIT_UP", 'BMP ', "VIEWIT_DOWN", 'BMP ', new BMessage(GB_OPEN_VIEWIT));
	AddChild(fOpenViewItButton);
	
	// Hand Cursor button
	fHandCursorButton = new TSimplePictureButton(BRect(90, 40, 117, 67), "HandCursor",
			"BCL_UP_HandCursor", 'BMP ', "BCL_DOWN_HandCursor", 'BMP ',
			new BMessage(GB_HAND_CURSOR_CLICKED), B_TWO_STATE_BUTTON,
			B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW);
	this->AddChild(fHandCursorButton);
	fHandCursorButton->SetValue(B_CONTROL_ON);
	
	// BCL icon button
	app_info info;
	be_app->GetAppInfo(&info);
	BFile self(&info.ref, B_READ_ONLY);
	BResources res;
	if (res.SetTo(&self) == B_OK) {
		size_t outSize;
		const void *iconImage = res.LoadResource('BMP ', "BCL_DefaultIcon", &outSize);
		if (iconImage != NULL) {
			BMemoryIO stream(iconImage, outSize);
			BBitmap *bitmap = BTranslationUtils::GetBitmap(&stream);
			TBCLPictureButton::MakeOnOffPicture(bitmap, &default_on_icon, &default_off_icon);
		}
	}
	
	fBCLTabView = new TBCLTabView(
			BRect(124, 20, this->Frame().right, this->Frame().bottom),
			"BCLTabView");
	
	// Tool Tip
	tool_tip.SetToolTip(fNewButton, "New Resource");
	tool_tip.SetToolTip(fOpenButton, "Open");
	tool_tip.SetToolTip(fSaveButton, "Save");
	tool_tip.SetToolTip(fTestRunButton, "Test Run");
	tool_tip.SetToolTip(fOpenViewItButton, "Open ViewIt");
	
	LanguageChanged();
}

TMainView::~TMainView()
{
}

void TMainView::LanguageChanged()
{
	BMenuItem *item;
	#define _SET_LABEL(menuItem, string, default) \
		item = fMenuBar->ItemAt(fMenuBar->IndexOf(menuItem));\
		if (item != NULL) item->SetLabel(GET_STRING(string, default));
	
	_SET_LABEL(fFileMenu, "File", "File");
	_SET_LABEL(fDebuggerMenu, "Debugger", "Debugger");
	
	fNewResourceMenuItem->SetLabel(GET_STRING("NewResource", "New Resource"));
	fLoadMenuItem->SetLabel(GET_STRING("Open", "Open"));
	fSaveMenuItem->SetLabel(GET_STRING("Save", "Save"));
	fSaveAsMenuItem->SetLabel(GET_STRING("SaveAs", "Save As..."));
	fAboutMenuItem->SetLabel(GET_STRING("About", "About"));
	fQuitMenuItem->SetLabel(GET_STRING("Quit", "Quit"));
	fTestRunMenuItem->SetLabel(GET_STRING("TestRun", "Test Run"));
	fOpenViewItItem->SetLabel(GET_STRING("OpenViewIt", "Open ViewIt"));
	
	tool_tip.SetToolTip(fNewButton, GET_STRING("T_NewResource", "New Resource"));
	tool_tip.SetToolTip(fOpenButton, GET_STRING("T_Open", "Open"));
	tool_tip.SetToolTip(fSaveButton, GET_STRING("T_Save", "Save"));
	tool_tip.SetToolTip(fTestRunButton, GET_STRING("T_TestRun", "Test Run"));
	tool_tip.SetToolTip(fOpenViewItButton, GET_STRING("T_OpenViewIt", "Open ViewIt"));
}

void TMainView::AttachedToWindow()
{
	fLoadMenuItem->SetTarget(be_app);
	fOpenButton->SetTarget(be_app);
	
	instantiation_info_t info;
	info.locked_list = &LockedList;
	
	fHandCursorButton->SetTarget(this);
	
	for(int32 i=0; i < AddonList.CountItems(); i++) {
		TAddonItem *item = (TAddonItem *)AddonList.ItemAt(i);
		info.id = item->id;
		item->instantiate_objects(&info, RegisterComponent);
	}
	
	int32 i=0;
	for(i=0; i < bcl_picture_button_list.CountItems(); i++) {
		TBCLPictureButton *btn = (TBCLPictureButton *)bcl_picture_button_list.ItemAt(i);
		int32 tab_index = fBCLTabView->FindTab(btn->Page.String());
		if (tab_index >= 0) {
			BView *bgview = fBCLTabView->TabAt(tab_index)->View();
			TBCLView *bcview = (TBCLView *)bgview->FindView("bcview");
			bcview->AddButton(btn);
		} else {
			BTab *tab = new BTab();
			BView *bgview = new BView(fBCLTabView->Bounds(),
					btn->Page.String(), B_FOLLOW_LEFT_RIGHT, B_FRAME_EVENTS | B_WILL_DRAW);
			bgview->SetViewColor(216, 216, 216, 0);
			tab->SetLabel(btn->Page.String());
			fBCLTabView->AddTab(bgview, tab);
			
			BRect rect(bgview->Bounds());
			rect.top += 1;
			rect.bottom -= 23;
			rect.left += 15;
			rect.right -= 18;
			TBCLView *bcview = new TBCLView(rect, "bcview");
			bcview->AddButton(btn);
			bgview->AddChild(bcview);
			
			TSimplePictureButton *leftArrow = new TSimplePictureButton(
					BRect(5, 2, 13, 34), "LeftArrow",
					"BCL_UP_LeftArrow", 'BMP ',
					"BCL_DOWN_LeftArrow", 'BMP ',
					new BMessage(GB_BCL_LEFTBTN_PRESSED), B_ONE_STATE_BUTTON,
					B_FOLLOW_LEFT, B_FRAME_EVENTS | B_WILL_DRAW
				);
			bgview->AddChild(leftArrow);
			leftArrow->SetTarget(this);
			
			rect = bgview->Bounds();
			rect.left = rect.right - 14;
			rect.right = rect.left + 8;
			rect.top = 2;
			rect.bottom = 34;
			TSimplePictureButton *rightArrow = new TSimplePictureButton(
					rect, "RightArrow",
					"BCL_UP_RightArrow", 'BMP ',
					"BCL_DOWN_RightArrow", 'BMP ',
					new BMessage(GB_BCL_RIGHTBTN_PRESSED), B_ONE_STATE_BUTTON,
					B_FOLLOW_RIGHT, B_FRAME_EVENTS | B_WILL_DRAW
				);
			bgview->AddChild(rightArrow);
			rightArrow->SetTarget(this);
		}
		btn->SetTarget(this);
	}
	
	if (i>0) {
		this->AddChild(fBCLTabView);
		fBCLTabView->SetTabHeight(17);
	}
}

void TMainView::MessageReceived(BMessage *msg)
{
	switch(msg->what){
		case GB_BCL_PRESSED: {
			fHandCursorButton->SetValue(B_CONTROL_OFF);
			for(int32 i=0; i < fBCLTabView->CountTabs(); i++) {
				BView *bgview = fBCLTabView->TabAt(i)->View();
				TBCLView *bcview = (TBCLView *)bgview->FindView("bcview");
				BMessenger(bcview).SendMessage(msg);
			}
			break;
		}
		case GB_HAND_CURSOR_CLICKED: {
			if (fHandCursorButton->Value() == B_CONTROL_OFF) {
				fHandCursorButton->SetValue(B_CONTROL_ON);
			}
			for(int32 i=0; i < fBCLTabView->CountTabs(); i++) {
				BView *bgview = fBCLTabView->TabAt(i)->View();
				TBCLView *bcview = (TBCLView *)bgview->FindView("bcview");
				bcview->Deselect();
			}
			break;
		}
		case GB_BCL_LEFTBTN_PRESSED:
		case GB_BCL_RIGHTBTN_PRESSED: {
			void *source;
			if (msg->FindPointer("source", &source)==B_OK) {
				TSimplePictureButton *btn = reinterpret_cast<TSimplePictureButton *>(source);
				BView *bgview = btn->Parent();
				TBCLView *bcview = (TBCLView *)bgview->FindView("bcview");
				switch(msg->what) {
					case GB_BCL_LEFTBTN_PRESSED:
						bcview->ScrollBy(-32, 0);
						if (bcview->Bounds().left < 0) {
							bcview->ScrollTo(0, 0);
						}
						break;
					case GB_BCL_RIGHTBTN_PRESSED:
						float rpos = bcview->Bounds().left + bcview->Frame().Width();
						if (rpos < bcview->Width()) {
							bcview->ScrollBy(32, 0);
						}
						break;
				}
			}
			break;
		}
		case B_SIMPLE_DATA:
			be_app_messenger.SendMessage(msg);
			break;
#ifdef B_LANGUAGE_CHANGED
		case B_LANGUAGE_CHANGED:
			LanguageChanged();
			break;
#endif			
	}
}

void TMainView::DeselectAll()
{
	for(int32 i=0; i < fBCLTabView->CountTabs(); i++) {
		BView *bgview = fBCLTabView->TabAt(i)->View();
		TBCLView *bcview = (TBCLView *)bgview->FindView("bcview");
		bcview->Deselect();
	}
	fHandCursorButton->SetValue(B_CONTROL_ON);
}
