#ifndef _OBJECTINSPECTORITEM_H
#define _OBJECTINSPECTORITEM_H

#include <Button.h>
#include <TextControl.h>
#include <MessageFilter.h>
#include <Messenger.h>
#include <Message.h>
#include <String.h>
#include <PopUpMenu.h>
#include "Berry.h"
#include "CLVEasyItem.h"

enum {
	COLUMN_NAME = 1,
	COLUMN_VALUE = 2
};

enum {
	ENTER_MSG = 'ogen',
	GONEXT_MSG = 'ognm',
	GOPREV_MSG = 'ogpm',
	OPEN_PROP_EDITOR = 'oped'
};


class TItemEditor : public BTextControl
{
public:
	TItemEditor(ColumnListView *owner,
		BRect frame,
		const char *name,
		const char *label,
		const char *text,
		const selectable_values *selectableValues,
		uint32 resizingMode = B_FOLLOW_LEFT | B_FOLLOW_TOP,
		uint32 flags = B_WILL_DRAW | B_NAVIGABLE);
	~TItemEditor();
	void AttachedToWindow();
	void MouseDown(BPoint point);
	bool IsModified();
	void ResetText();
	void UpdateMenuItemMark();
	const char *OriginalText() const { return fOriginalText.String(); };
private:
	BString fOriginalText;
	BPopUpMenu *fPopUpMenu;
	ColumnListView *fOwner;
	const selectable_values *fSelectableValues;
};

class TItemEditMessageFilter : BMessageFilter
{
public:
	TItemEditMessageFilter(ColumnListView *owner);
	~TItemEditMessageFilter();
	filter_result Filter(BMessage *message, BHandler **handler);
private:
	ColumnListView *fOwner;
};

class TObjectInspectorItem : public CLVEasyItem
{
public:
	TObjectInspectorItem(
			ColumnListView *owner,
			const TPropertyItem *PropertyItem,
			int32 level,
			BList *objectList,
			bool superitem = false,
			TBerryComponentLibrary *bcl = NULL
		);
	~TObjectInspectorItem();
	void Selected();
	void Deselected();
	const char *EditorText();
	TItemEditor *ItemEditor() { return fEditor; };
	const char *Name() { return GetColumnContentText(COLUMN_NAME); };
	void UpdateContent();
	void SetValueFromItemEditor(bool noAlert = true);
	void SetValue(const char *value, bool noAlert = true);
	BArchivable *Object(int32 index) const;
private:
	ColumnListView *fOwner;
	TItemEditor *fEditor;
	TPropertyItem fPropertyItem;
	BButton *fOpenPropEditorBtn;
	TBerryComponentLibrary *fBCL;
	BList fObjectList;
};


#endif
