#include <typeinfo>
#include <String.h>
#include <image.h>
#include <Alert.h>
#include <stdlib.h>
#include <stdio.h>
#include <Application.h>
#include <Roster.h>
#include <NetPositive.h>

#include "ObjectInspectorView.h"
#include "BerryUtils.h"
#include "LockedList.h"


BTextControl *ObjectTypeName;
TObjectInspectorView *ObjectInspectorView;


// Prototype
int compareFunc(const void *item1, const void *item2);
status_t _updater_func(const char *value);


int compareFunc(const void *item1, const void *item2)
{
	const TPropertyItem *p1 = (const TPropertyItem *)*(const void**)item1;
	const TPropertyItem *p2 = (const TPropertyItem *)*(const void**)item2;
	return strcmp(p1->Name, p2->Name);
}

TObjectInspectorView::TObjectInspectorView(BRect frame,
						const char *name,
						uint32 resizingMode,
						uint32 flags)
	:	ColumnListView(frame, &iContainerView, name,
						resizingMode, flags,
						B_SINGLE_SELECTION_LIST,	// view type
						true,						// hierarchical
						true,						// horizontal scrollbars
						true,						// vertical scrollbars
						true,						// scroll-view corner
						B_FANCY_BORDER)				// border style
{
	this->SetListType(B_SINGLE_SELECTION_LIST);
	
	this->AddColumn(new CLVColumn(NULL, 20.0, CLV_EXPANDER | CLV_LOCK_AT_BEGINNING | CLV_NOT_MOVABLE));
	this->AddColumn(new CLVColumn("Name", 90, CLV_HEADER_TRUNCATE | CLV_TELL_ITEMS_WIDTH, 50) );
	this->AddColumn(new CLVColumn("Value", 114, CLV_HEADER_TRUNCATE | CLV_TELL_ITEMS_WIDTH, 50) );

	fLastSelectedItem = NULL;
}

TObjectInspectorView::~TObjectInspectorView()
{
}

void TObjectInspectorView::SelectionChanged()
{
	int32 selected = this->CurrentSelection();
	if (selected < 0) {
		if (this->fLastSelectedItem != NULL) {
			this->UpdateValue(fLastSelectedItem);
			this->fLastSelectedItem->Deselected();
		}
		this->fLastSelectedItem = NULL;
		return;	
	}
	TObjectInspectorItem *selectedItem = (TObjectInspectorItem *)this->ItemAt(selected);
	selectedItem->Selected();
	if (this->fLastSelectedItem != NULL) {
		this->UpdateValue(fLastSelectedItem);
		this->fLastSelectedItem->Deselected();
	}
	this->fLastSelectedItem = selectedItem;
}

void TObjectInspectorView::MessageReceived(BMessage *msg)
{
	TObjectInspectorItem *item;
	int32 selected = CurrentSelection();
	
	switch(msg->what) {
/*
		case BERRY_OBJECT_INSTANTIATED: {
			type_code typeCode;
			int32 objectCount;
			void *object;
			msg->GetInfo("instantiated_object", &typeCode, &objectCount);
			for(int32 i=0; i < objectCount; i++) {
				if (msg->FindPointer("instantiated_object", i, &object) == B_OK) {
					if (dynamic_cast<BArchivable *>((BArchivable *)object) != NULL) {
						BArchivable *instantiatedObject = reinterpret_cast<BArchivable *>(object);
						TBerryComponentLibrary *bcl;
						FindSupportBCL(instantiatedObject, &bcl);
						if (bcl != NULL) {
							LockObjectLooper(instantiatedObject);
							bcl->Instantiated(instantiatedObject);
							UnlockObjectLooper(instantiatedObject);
						}
					}
				}
			}
			break;
		}
*/
		case BERRY_OBJECT_DELETED: {
			void *p;
			type_code typeCode;
			int32 objectCount;
			for(int32 i=0; i < be_app->CountWindows(); i++) {
				BWindow *window = be_app->WindowAt(i);
				TSinglePropEditor *found = dynamic_cast<TSinglePropEditor *>(window);
				if (found != NULL ) {
					msg->GetInfo("object", &typeCode, &objectCount);
					for(int32 i=0; i < objectCount; i++) {
						if (msg->FindPointer("object", i, &p) == B_OK) {
							if (found->Object() == p) {
								BMessenger(found).SendMessage(B_QUIT_REQUESTED);
								break;
							}
						}
					}
				}
			}
			break;
		}
		case BERRY_OBJECT_SELECTED: {
			BList lastSelectedList;
			lastSelectedList.AddList(&fObjectList);
			type_code typeCode;
			int32 objectCount;
			void *object;
			// Invalidate する BMessenger を検索
			msg->FindMessenger("messenger", &fTargetMessenger);
			// msg に含まれる "object" を検索して fObjectList に格納する
			msg->GetInfo("object", &typeCode, &objectCount);
			fObjectList.MakeEmpty();
			for(int32 i=0; i < objectCount; i++) {
				if (msg->FindPointer("object", i, &object) == B_OK) {
					if (dynamic_cast<BArchivable *>((BArchivable *)object) != NULL) {
						fObjectList.AddItem(object);
					}
				}
			}
			// lastSelectedList == fObjectList
			bool equal = true;
			if (lastSelectedList.CountItems() == fObjectList.CountItems()) {
				for(int32 i=0; i < lastSelectedList.CountItems(); i++) {
					if (lastSelectedList.ItemAt(i) != fObjectList.ItemAt(i)) {
						equal = false;
						break;
					}
				}
			} else {
				equal = false;
			}
			
			if (equal) {
				UpdateInspector();
			} else {
				// Object Inspector の表示を更新
				fViewSelected();
			}
			
			break;
		}
		case ENTER_MSG:
		case GONEXT_MSG:
		case GOPREV_MSG:
			if (selected < 0) return;
			switch(msg->what) {
				case ENTER_MSG:
					this->UpdateValue((TObjectInspectorItem *)ItemAt(selected));
					break;
				case GONEXT_MSG:
					item = (TObjectInspectorItem *)ItemAt(selected + 1);
					if (item!=NULL) {
						Select(selected + 1);
					}
					break;
				case GOPREV_MSG:
					item = (TObjectInspectorItem *)ItemAt(selected - 1);
					if (item!=NULL) {
						Select(selected - 1);
					}
					break;
			}
			break;
		case OPEN_PROP_EDITOR:
			void *info;
			if (msg->FindPointer("prop_editor", &info) == B_OK) {
				fOpenPropertyEditor((TObjectInspectorItem *)ItemAt(selected),
									(const type_info *)info);
			}
			break;
		case GB_SHOW_HELP:
			Help();
			break;
	}
}

void TObjectInspectorView::WindowActivated(bool active)
{
	if (!active) {
		this->UpdateValue((TObjectInspectorItem *)ItemAt(CurrentSelection()));
		//DeselectAll();
	}
}

void TObjectInspectorView::fViewSelected()
{
	TBerryComponentLibrary *bcl;
	BArchivable *aObject;
	BList propertyList, listList;
	BString value, lastValue;
	
	ObjectTypeName->SetText("");
	DeselectAll();
	this->MakeEmpty();
	
	// Object Inspector に表示する propertyList を作る
	TPropertyRegistrar propertyRegistrar(fObjectList.CountItems() != 1);
	for(int32 i=0; i < fObjectList.CountItems(); i++) {
		BList *list = new BList();
		propertyRegistrar.fPropList = list;
		listList.AddItem(list);
		aObject = (BArchivable *)fObjectList.ItemAt(i);
		FindSupportBCL(aObject, &bcl);
		if (bcl!=NULL) {
			bcl->PropertyList(&propertyRegistrar);
//			list->SortItems(&compareFunc);
		} else {
			// サポートするBCLが無い
			goto _clean_up;
		}
	}
	if (listList.CountItems() <= 0) goto _clean_up;
	if (listList.CountItems() == 1) {
		// Object Inspector 更新(単一)
		ObjectTypeName->SetText(typeid(*aObject).name());
		propertyList.AddList((BList *)listList.ItemAt(0));
		TObjStack objStack;
		int32 lastLevel = 0;
		BList objectList;
		BArchivable *subObject;
		for(int32 i=0; i < propertyList.CountItems(); i++) {
			TPropertyItem *item = (TPropertyItem *)propertyList.ItemAt(i);
			if (lastLevel > item->Level) {
				objStack.Pop(&aObject);
			}
			if (item->SuperItem) {
				objStack.Push(aObject);
				((TBerryComponentLibrary *)item->BCL)->GetSuperObject(aObject, item->Name, &subObject);
				aObject = subObject;
			} else {
				//
			}
			lastLevel = item->Level;
			objectList.MakeEmpty();
			objectList.AddItem(aObject);
			TObjectInspectorItem *oiItem = new TObjectInspectorItem(this, item, item->Level,
									&objectList, item->SuperItem, (TBerryComponentLibrary *)item->BCL);
			this->AddItem(oiItem);
			oiItem->UpdateContent();
		}
	} else {
		// Object Inspector 更新(複数)
		bool ok, __ok;
		BList *baseList = (BList *)listList.ItemAt(0);
		for(int32 i=0; i < baseList->CountItems(); i++) {
			TPropertyItem *baseItem = (TPropertyItem *)baseList->ItemAt(i);
			ok = true;
			for(int32 j=1; j < listList.CountItems(); j++) {
				BList *list = (BList *)listList.ItemAt(j);
				__ok = false;
				for(int32 k=0; k < list->CountItems(); k++) {
					TPropertyItem *item = (TPropertyItem *)list->ItemAt(k);
					if (strcmp(item->Name, "Name") == 0) continue;
					if ((strcmp(baseItem->Name, item->Name)==0) &&
					    (baseItem->TypeCode == item->TypeCode) &&
					    (item->TypeCode != B_RAW_TYPE) &&
					    (baseItem->TypeInfo == item->TypeInfo)) {
						__ok = true;
						break;
					}
				}
				ok &= __ok;
				if (!ok) break;
			}
			if (ok) propertyList.AddItem(baseItem);
		}
		
		for(int32 j=0; j < propertyList.CountItems(); j++) {
			TPropertyItem *item = (TPropertyItem *)propertyList.ItemAt(j);
			TObjectInspectorItem *oiItem = new TObjectInspectorItem(this, item, 0, &fObjectList);
			this->AddItem(oiItem);
			oiItem->UpdateContent();
		}
	}
	
_clean_up:
	
	// propertyList(listList) を開放
	for(int32 i=0; i < listList.CountItems(); i++) {
		BList *list = (BList *)listList.ItemAt(i);
		for(int32 j=0; j < list->CountItems(); j++) {
			delete (TPropertyItem *)list->ItemAt(j);
		}
		delete list;
	}
}

// Object Inspector の表示値のみ更新
void TObjectInspectorView::UpdateInspector()
{
	for(int32 i=0; i < this->CountItems(); i++) {
		TObjectInspectorItem *item = (TObjectInspectorItem *)this->ItemAt(i);
		item->UpdateContent();
	}
}

void TObjectInspectorView::MakeChangedMessage(BMessage *msg, TObjectInspectorItem *item)
{
	for(int32 i = 0; i < fObjectList.CountItems(); i++) {
		msg->AddPointer("object", fObjectList.ItemAt(i));
	}
	msg->AddString("name", item->Name());
	msg->AddString("old_value", item->ItemEditor()->OriginalText());
	msg->AddString("new_value", item->ItemEditor()->Text());
}

void TObjectInspectorView::UpdateValue(TObjectInspectorItem *item)
{
	if (item == NULL) return;
	if (!item->ItemEditor()->IsModified()) return;
	item->SetValueFromItemEditor(false);
	this->Invalidate();
	if (fTargetMessenger.IsValid()) {
		BMessage msg(BERRY_VIEW_INVALIDATE);
		MakeChangedMessage(&msg, item);
		fTargetMessenger.SendMessage(&msg);
	}
	UpdateInspector();
}

TObjectInspectorItem *TObjectInspectorView::LastSelectedItem()
{
	return fLastSelectedItem;
}

BList *TObjectInspectorView::ObjectList()
{
	return &fObjectList;
}

BMessenger *TObjectInspectorView::TargetMessenger()
{
	return &fTargetMessenger;
}

TObjectInspectorItem *selected_item;

status_t _updater_func(const char *value)
{
	ObjectInspectorView->Looper()->Lock();
	selected_item->SetValue(value);
	if (ObjectInspectorView->TargetMessenger()->IsValid()) {
		BMessage msg(BERRY_VIEW_INVALIDATE);
		ObjectInspectorView->MakeChangedMessage(&msg, selected_item);
		ObjectInspectorView->TargetMessenger()->SendMessage(&msg);
	}
	ObjectInspectorView->Looper()->Unlock();
	return B_OK;
/*
	BArchivable *object;
	TBerryComponentLibrary *bcl = NULL;
	
	ObjectInspectorView->Looper()->Lock();
	BList *objectList = ObjectInspectorView->ObjectList();
	selected_item->SetColumnContent(COLUMN_VALUE, value);
	ObjectInspectorView->Invalidate();
	const char *propertyName = selected_item->GetColumnContentText(COLUMN_NAME);
	for(int32 i=0; i < objectList->CountItems(); i++) {
		object = (BArchivable *)objectList->ItemAt(i);
		for(int32 j=0; j < BCLList.CountItems(); j++) {
			bcl = (TBerryComponentLibrary *)BCLList.ItemAt(j);
			if (strcmp(bcl->SupportedClass(), typeid(*object).name())==0) {
				break;
			} else {
				bcl = NULL;
			}
		}
		if (bcl != NULL) {
			LockObjectLooper(object);
			if (bcl->SetValue(object, propertyName, value)!=B_OK) {
				fprintf(stderr, APP_NAME ": ERROR _updater_func() SetValue.\n");
			}
			UnlockObjectLooper(object);
			ObjectInspectorView->UpdateInspector();
		} else {
			fprintf(stderr, APP_NAME ": ERROR _updater_func() bcl not found.\n");
		}
	}
	if (ObjectInspectorView->TargetMessenger()->IsValid()) {
		ObjectInspectorView->TargetMessenger()->SendMessage(GB_VIEW_INVALIDATE);
	}
	ObjectInspectorView->Looper()->Unlock();
	
	return B_OK;
*/
}

TSinglePropEditor *TObjectInspectorView::FindPropEditor(TSinglePropEditor *editor) const
{
	for(int32 i=0; i < be_app->CountWindows(); i++) {
		BWindow *window = be_app->WindowAt(i);
		if (window == editor) continue;
		TSinglePropEditor *found = dynamic_cast<TSinglePropEditor *>(window);
		if (found != NULL ) {
			if (strcmp(editor->fClassName.String(), found->fClassName.String()) == 0) {
				return found;
			}
		}
	}
	return NULL;
}

void TObjectInspectorView::fOpenPropertyEditor(TObjectInspectorItem *item, const type_info *typeInfo)
{
	selected_item = (TObjectInspectorItem *)ItemAt(CurrentSelection());
	TPropEditor *propEditor;
	TAddonItem *addonItem;
	for(int32 i=0; i < AddonList.CountItems(); i++) {
		addonItem = (TAddonItem *)AddonList.ItemAt(i);
		if (addonItem->register_prop_editors != NULL) {
			propeditor_info_t peInfo;
			peInfo.updaterfunc = _updater_func;
			peInfo.locked_list = &LockedList;
			propEditor = (TMultiPropEditor *)addonItem->register_prop_editors(typeInfo, &peInfo);
			if (propEditor != NULL) {
				TSinglePropEditor *singlePropEditor = dynamic_cast<TSinglePropEditor *>(propEditor);
				TMultiPropEditor *multiPropEditor = dynamic_cast<TMultiPropEditor *>(propEditor);
				if (singlePropEditor != NULL) {
					singlePropEditor->fClassName.SetTo(typeInfo->name());
					TSinglePropEditor *editor = FindPropEditor(singlePropEditor);
					if (editor != NULL) {
						delete singlePropEditor;
						singlePropEditor = editor;
					}
					BMessenger msgr(this);
					singlePropEditor->Lock();
					singlePropEditor->fMessenger = msgr;
					singlePropEditor->SetObject(item->Object(0));
					singlePropEditor->Unlock();
					singlePropEditor->Show();
					singlePropEditor->Activate();
					return;
				} else if (multiPropEditor != NULL) {
					multiPropEditor->SetFeel(B_MODAL_APP_WINDOW_FEEL);
					multiPropEditor->Lock();
					multiPropEditor->SetValue(item->EditorText());
					multiPropEditor->Unlock();
					multiPropEditor->Show();
					return;
				}
			}
		}
	}
}

void TObjectInspectorView::KeyDown(const char *bytes, int32 numBytes)
{
	if (numBytes < 0) return;
	BMessenger msgr(this);
	int32 idx = CurrentSelection();
	switch(bytes[0]){
		case B_UP_ARROW:
			if (idx == 0) {} else
			if (idx < 0) { Select(CountItems()-1); } else
			msgr.SendMessage(GOPREV_MSG);
			break;
		case B_DOWN_ARROW:
			if (idx == CountItems()-1) {} else
			if (idx < 0) { Select(0); } else
			msgr.SendMessage(GONEXT_MSG);
			break;
		case B_F1_KEY:
		case 16:
			Help();
			break;
	}
}

void TObjectInspectorView::MakeEmpty()
{
	for(int32 i=0; i < this->CountItems(); i++) {
		TObjectInspectorItem *item = (TObjectInspectorItem *)this->ItemAt(i);
		delete item;
	}
	ColumnListView::MakeEmpty();
}

void TObjectInspectorView::Help(const char *bclClassName, const char *propertyName)
{
	if (fObjectList.CountItems() <= 0) return;
	
	BString bclname;
	BString prop;
	TBerryComponentLibrary *bcl;
	
	if (bclClassName == NULL) {
		BArchivable *aObject = (BArchivable *)fObjectList.ItemAt(0);
		FindSupportBCL(aObject, &bcl);
		const char *rawname = bcl->SupportedClass();
		bclname.Append(rawname, GetNumber(rawname));
		
		TObjectInspectorItem *item = (TObjectInspectorItem *)ItemAt(CurrentSelection());
		if (item != NULL) prop.SetTo(item->Name());
	} else {
		bclname.SetTo(bclClassName);
		prop.SetTo("");
	}
	
	BString url;
	GetAddonDir("help", &url);
	url << "/" << bclname << ".html";
	BEntry entry(url.String());
	if (!entry.Exists()) return;
	
	if (prop.Length() > 0) url << "#" << prop;
	const char *argv[1];
	argv[0] = url.String();
	fprintf(stderr, APP_NAME ": Help %s\n", argv[0]);
	be_roster->Launch(B_NETPOSITIVE_APP_SIGNATURE, 1, (char **)argv);
}

// TBCLStack

TObjStack::TObjStack()
{
}

TObjStack::~TObjStack()
{
}

void TObjStack::Push(const BArchivable *object)
{
	fObjectList.AddItem((void *)object);
}

status_t TObjStack::Pop(BArchivable **object)
{
	int32 idx = fObjectList.CountItems() - 1;
	if (idx < 0) return B_ERROR;
	*object = (BArchivable *)fObjectList.ItemAt(idx);
	fObjectList.RemoveItem(idx);
	return B_OK;
}
