#include <View.h>
#include <Message.h>
#include <stdio.h>
#include "ResAddon.h"
#include "ResManager.h"

#include "ContainerWindow.h"
#include "EditorWindow.h"
#include "CstrEditorWindow.h"
#include "AppfEditorWindow.h"
#include "BpntEditorWindow.h"


TResAddons res_addons;


// TResEditorWindow

TResEditorWindow::TResEditorWindow(BRect frame, const char *title,
				window_type type, uint32 flags, uint32 workspaces)
	:	BWindow(frame, title, type, flags, workspaces)
{
}

TResEditorWindow::~TResEditorWindow()
{
	BMessage msg(RM_QUIT);
	msg.AddPointer("editor_window", this);
	fMessenger.SendMessage(&msg);
}

void TResEditorWindow::SetParameter(BMessenger *msgr, const char *name, type_code type, int32 id, const entry_ref *ref)
{
	fMessenger = *msgr;
	fName.SetTo(name);
	fType = type;
	fID = id;
	fRef = *ref;
}

void TResEditorWindow::GetParameter(BMessenger *msgr, BString *name, type_code *type, int32 *id, entry_ref *ref)
{
	if (msgr) *msgr = fMessenger;
	if (name) name->SetTo(fName.String());
	if (type) *type = fType;
	if (id) *id = fID;
	if (ref) *ref = fRef;
}

void TResEditorWindow::SetDirty()
{
	BMessage msg(RM_MODIFIED);
	msg.AddPointer("editor_window", this);
	fMessenger.SendMessage(&msg);
}

void TResEditorWindow::SendData()
{
	BMallocIO data;
	GetData(&data);
	BMessage msg(RM_DATA);
	msg.AddPointer("editor_window", this);
	msg.AddData("data", B_ANY_TYPE, data.Buffer(), data.BufferLength());
	fMessenger.SendMessage(&msg);
}


// TResAddon

TResAddon::TResAddon()
{
}

TResAddon::~TResAddon()
{
}

void TResAddon::String(BMallocIO *data, BString *string)
{
}


// TResAddon_unknown

TResAddon_unknown::TResAddon_unknown()
	:	TResAddon()
{
}

TResAddon_unknown::~TResAddon_unknown()
{
}

void TResAddon_unknown::GetType(BString *desc, type_code *type)
{
	desc->SetTo("");
}

void TResAddon_unknown::New(BDataIO *data)
{
}

void TResAddon_unknown::String(BMallocIO *data, BString *string)
{
	char h[3];
	const uchar *d = (const uchar *)data->Buffer();
	uint32 i;
	string->SetTo("");
	for(i = 0; (i < 16) && (i < data->BufferLength()); i++) {
		sprintf(h, "%2.2X", d[i]);
		*string << h << " ";
	}
	if (i < data->BufferLength()) *string << " ...";
}

TResEditorWindow *TResAddon_unknown::OpenEditor(const char *title)
{
	return NULL;
}


// ----------------------------------------------------------------------------

// TResAddon_ARCV

class TResAddon_ARCV : public TResAddon
{
public:
	TResAddon_ARCV() : TResAddon() {};
	~TResAddon_ARCV() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("GuiBerry / Interface Elements");
		*type = 'ARCV';
	};
	void New(BDataIO *data) {
		BView *view = new BView(BRect(0, 0, 379, 239), "BGView", B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FRAME_EVENTS);
		view->SetViewColor(216, 216, 216);
		BMessage msg;
		view->Archive(&msg);
		msg.Flatten(data);
		delete view;
	}
	void String(BMallocIO *data, BString *string) {
		string->SetTo("(BMessage archive)");
	}
	TResEditorWindow *OpenEditor(const char *title) {
		TResEditorWindow *window = new TContainerWindow(BRect(260, 220, 400, 300), title);
		return window;
	}
};

// TResAddon_APPF

class TResAddon_APPF : public TResAddon
{
public:
	TResAddon_APPF() : TResAddon() {};
	~TResAddon_APPF() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("Application Flags");
		*type = 'APPF';
	};
	void New(BDataIO *data) {
		int32 d = 0;
		data->Write(&d, sizeof(d));
	}
	void String(BMallocIO *data, BString *string) {
		uint32 flag;
		data->ReadAt(0, &flag, sizeof(flag));
		switch(flag & 3) {
			case 0: *string << "Single"; break;
			case 1: *string << "Multiple"; break;
			case 2: *string << "Exclusive"; break;
		}
		if (flag & 8) *string << ", Argv Only";
		if (flag & 4) *string << ", Background";
	}
	TResEditorWindow *OpenEditor(const char *title) {
		TAppfEditorWindow *window = new TAppfEditorWindow(BRect(260, 320, 500, 430), title, 'APPF', this);
		return window;
	}
};


// TResAddon_BPNT

class TResAddon_BPNT : public TResAddon
{
public:
	TResAddon_BPNT() : TResAddon() {};
	~TResAddon_BPNT() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("BPoint");
		*type = 'BPNT';
	};
	void New(BDataIO *data) {
		BPoint d(0,0);
		data->Write(&d, sizeof(d));
	}
	void String(BMallocIO *data, BString *string) {
		BPoint d;
		data->ReadAt(0, &d, sizeof(d));
		*string << "(" << d.x << "," << d.y << ")";
	}
	TResEditorWindow *OpenEditor(const char *title) {
		TBpntEditorWindow *window = new TBpntEditorWindow(BRect(260, 320, 450, 410), title, 'BPNT', this);
		return window;
	}
};


// TResAddon_RECT

class TResAddon_RECT : public TResAddon
{
public:
	TResAddon_RECT() : TResAddon() {};
	~TResAddon_RECT() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("BRect");
		*type = 'RECT';
	};
	void New(BDataIO *data) {
		BRect d(0,0,0,0);
		data->Write(&d, sizeof(d));
	}
	void String(BMallocIO *data, BString *string) {
		BRect d;
		data->ReadAt(0, &d, sizeof(d));
		*string << "(" << d.left << "," << d.top << ")-(" << d.right << "," << d.bottom << ")";
	}
	TResEditorWindow *OpenEditor(const char *title) {
		return NULL;
	}
};


// TResAddon_BBMP

class TResAddon_BBMP : public TResAddon
{
public:
	TResAddon_BBMP() : TResAddon() {};
	~TResAddon_BBMP() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("Bitmap");
		*type = 'BBMP';
	};
	void New(BDataIO *data) {
		BBitmap bitmap(BRect(0,0,31,31), B_RGBA32);
		BMessage a;
		bitmap.Archive(&a);
		a.Flatten(data);
	}
	void String(BMallocIO *data, BString *string) {
		BMessage a;
		a.Unflatten(data);
		BBitmap bitmap(&a);
		BRect rect(bitmap.Bounds());
		*string << (int)rect.Width()+1 << "x" << (int)rect.Height()+1 << "x";
		switch(bitmap.ColorSpace()) {
			case B_NO_COLOR_SPACE: *string << "B_NO_COLOR_SPACE"; break;
			case B_RGB32: *string << "B_RGB32"; break;
			case B_RGBA32: *string << "B_RGBA32"; break;
			case B_RGB24: *string << "B_RGB24"; break;
			case B_RGB16: *string << "B_RGB16"; break;
			case B_RGB15: *string << "B_RGB15"; break;
			case B_RGBA15: *string << "B_RGBA15"; break;
			case B_CMAP8: *string << "B_CMAP8"; break;
			case B_GRAY8: *string << "B_GRAY8"; break;
			case B_GRAY1: *string << "B_GRAY1"; break;
			case B_RGB32_BIG: *string << "B_RGB32_BIG"; break;
			case B_RGBA32_BIG: *string << "B_RGBA32_BIG"; break;
			case B_RGB24_BIG: *string << "B_RGB24_BIG"; break;
			case B_RGB16_BIG: *string << "B_RGB16_BIG"; break;
			case B_RGB15_BIG: *string << "B_RGB15_BIG"; break;
			case B_RGBA15_BIG: *string << "B_RGBA15_BIG"; break;
			case B_YCbCr422: *string << "B_YCbCr422"; break;
			case B_YCbCr411: *string << "B_YCbCr411"; break;
			case B_YCbCr444: *string << "B_YCbCr444"; break;
			case B_YCbCr420: *string << "B_YCbCr420"; break;
			case B_YUV422: *string << "B_YUV422"; break;
			case B_YUV411: *string << "B_YUV411"; break;
			case B_YUV444: *string << "B_YUV444"; break;
			case B_YUV420: *string << "B_YUV420"; break;
			case B_YUV9: *string << "B_YUV9"; break;
			case B_YUV12: *string << "B_YUV12"; break;
			case B_UVL24: *string << "B_UVL24"; break;
			case B_UVL32: *string << "B_UVL32"; break;
			case B_UVLA32: *string << "B_UVLA32"; break;
			case B_LAB24: *string << "B_LAB24"; break;
			case B_LAB32: *string << "B_LAB32"; break;
			case B_LABA32: *string << "B_LABA32"; break;
			case B_HSI24: *string << "B_HSI24"; break;
			case B_HSI32: *string << "B_HSI32"; break;
			case B_HSIA32: *string << "B_HSIA32"; break;
			case B_HSV24: *string << "B_HSV24"; break;
			case B_HSV32: *string << "B_HSV32"; break;
			case B_HSVA32: *string << "B_HSVA32"; break;
			case B_HLS24: *string << "B_HLS24"; break;
			case B_HLS32: *string << "B_HLS32"; break;
			case B_HLSA32: *string << "B_HLSA32"; break;
			case B_CMY24: *string << "B_CMY24"; break;
			case B_CMY32: *string << "B_CMY32"; break;
			case B_CMYA32: *string << "B_CMYA32"; break;
			case B_CMYK32: *string << "B_CMYK32"; break;
			default: *string << "?";
		}
	}
	TResEditorWindow *OpenEditor(const char *title) {
		return NULL;
	}
};


// TResAddon_BOOL

class TResAddon_BOOL : public TResAddon
{
public:
	TResAddon_BOOL() : TResAddon() {};
	~TResAddon_BOOL() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("Boolean");
		*type = 'BOOL';
	};
	void New(BDataIO *data) {
		bool d = true;
		data->Write(&d, sizeof(d));
	}
	void String(BMallocIO *data, BString *string) {
		bool d;
		data->ReadAt(0, &d, sizeof(d));
		*string << (d ? "True" : "False");
	}
	TResEditorWindow *OpenEditor(const char *title) {
		return NULL;
	}
};


// TResAddon_CHAR

class TResAddon_CHAR : public TResAddon
{
public:
	TResAddon_CHAR() : TResAddon() {};
	~TResAddon_CHAR() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("Character");
		*type = 'CHAR';
	};
	void New(BDataIO *data) {
		char d = 0;
		data->Write(&d, sizeof(d));
	}
	void String(BMallocIO *data, BString *string) {
		char d;
		data->ReadAt(0, &d, sizeof(d));
		*string << (int)d;
	}
	TResEditorWindow *OpenEditor(const char *title) {
		TEditorWindow *window = new TEditorWindow(BRect(260, 320, 500, 400), title, 'CHAR', this);
		return window;
	}
};


// TResAddon_CURS

class TResAddon_CURS : public TResAddon
{
public:
	TResAddon_CURS() : TResAddon() {};
	~TResAddon_CURS() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("Cursor");
		*type = 'CURS';
	};
	void New(BDataIO *data) {
		data->Write(B_HAND_CURSOR, 68);
	}
	void String(BMallocIO *data, BString *string) {
		unsigned char d[68];
		data->ReadAt(0, &d, 68);
		*string << (uint32)d[0] << "x" << (uint32)d[0] << "x" << (uint32)d[1];
	}
	TResEditorWindow *OpenEditor(const char *title) {
		return NULL;
	}
};


// TResAddon_FLOT

class TResAddon_FLOT : public TResAddon
{
public:
	TResAddon_FLOT() : TResAddon() {};
	~TResAddon_FLOT() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("Floating Point: 4 Bytes");
		*type = 'FLOT';
	};
	void New(BDataIO *data) {
		float d = 0;
		data->Write(&d, sizeof(d));
	}
	void String(BMallocIO *data, BString *string) {
		float d;
		data->ReadAt(0, &d, sizeof(d));
		*string << d;
	}
	TResEditorWindow *OpenEditor(const char *title) {
		TEditorWindow *window = new TEditorWindow(BRect(260, 320, 500, 400), title, 'FLOT', this);
		return window;
	}
};


// TResAddon_DBLE

class TResAddon_DBLE : public TResAddon
{
public:
	TResAddon_DBLE() : TResAddon() {};
	~TResAddon_DBLE() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("Floating Point: 8 Bytes");
		*type = 'DBLE';
	};
	void New(BDataIO *data) {
		double d = 0;
		data->Write(&d, sizeof(d));
	}
	void String(BMallocIO *data, BString *string) {
		double d;
		char s[64];
		data->ReadAt(0, &d, sizeof(d));
		sprintf(s, "%.16f", d);
		*string << s;
	}
	TResEditorWindow *OpenEditor(const char *title) {
		TEditorWindow *window = new TEditorWindow(BRect(260, 320, 500, 400), title, 'DBLE', this);
		return window;
	}
};


// TResAddon_ICON

class TResAddon_ICON : public TResAddon
{
public:
	TResAddon_ICON() : TResAddon() {};
	~TResAddon_ICON() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("Icon");
		*type = 'ICON';
	};
	void New(BDataIO *data) {
		BBitmap icon(BRect(0, 0, 31, 31), B_CMAP8);
		const void *d = icon.Bits();
		int32 length = icon.BitsLength();
		data->Write(d, length);
	}
	void String(BMallocIO *data, BString *string) {
		*string << "32x32x8";
	}
	TResEditorWindow *OpenEditor(const char *title) {
		return NULL;
	}
};

// TResAddon_MICN

class TResAddon_MICN : public TResAddon
{
public:
	TResAddon_MICN() : TResAddon() {};
	~TResAddon_MICN() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("Mini Icon");
		*type = 'MICN';
	};
	void New(BDataIO *data) {
		BBitmap icon(BRect(0, 0, 15, 15), B_CMAP8);
		const void *d = icon.Bits();
		int32 length = icon.BitsLength();
		data->Write(d, length);
	}
	void String(BMallocIO *data, BString *string) {
		*string << "16x16x8";
	}
	TResEditorWindow *OpenEditor(const char *title) {
		return NULL;
	}
};


// TResAddon_SHRT

class TResAddon_SHRT : public TResAddon
{
public:
	TResAddon_SHRT() : TResAddon() {};
	~TResAddon_SHRT() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("Integer: 16Bit Signed");
		*type = 'SHRT';
	};
	void New(BDataIO *data) {
		int16 d = 0;
		data->Write(&d, sizeof(d));
	}
	void String(BMallocIO *data, BString *string) {
		int16 d;
		data->ReadAt(0, &d, sizeof(d));
		*string << d;
	}
	TResEditorWindow *OpenEditor(const char *title) {
		TEditorWindow *window = new TEditorWindow(BRect(260, 320, 500, 400), title, 'SHRT', this);
		return window;
	}
};


// TResAddon_USHT

class TResAddon_USHT : public TResAddon
{
public:
	TResAddon_USHT() : TResAddon() {};
	~TResAddon_USHT() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("Integer: 16Bit Unsigned");
		*type = 'USHT';
	};
	void New(BDataIO *data) {
		uint16 d = 0;
		data->Write(&d, sizeof(d));
	}
	void String(BMallocIO *data, BString *string) {
		uint16 d;
		data->ReadAt(0, &d, sizeof(d));
		*string << (int32)d;
	}
	TResEditorWindow *OpenEditor(const char *title) {
		TEditorWindow *window = new TEditorWindow(BRect(260, 320, 500, 400), title, 'USHT', this);
		return window;
	}
};


// TResAddon_LONG

class TResAddon_LONG : public TResAddon
{
public:
	TResAddon_LONG() : TResAddon() {};
	~TResAddon_LONG() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("Integer: 32 Bit Signed");
		*type = 'LONG';
	};
	void New(BDataIO *data) {
		int32 d = 0;
		data->Write(&d, sizeof(d));
	}
	void String(BMallocIO *data, BString *string) {
		int32 d;
		data->ReadAt(0, &d, sizeof(d));
		*string << d;
	}
	TResEditorWindow *OpenEditor(const char *title) {
		TEditorWindow *window = new TEditorWindow(BRect(260, 320, 500, 400), title, 'LONG', this);
		return window;
	}
};


// TResAddon_ULNG

class TResAddon_ULNG : public TResAddon
{
public:
	TResAddon_ULNG() : TResAddon() {};
	~TResAddon_ULNG() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("Integer: 32Bit Unsigned");
		*type = 'ULNG';
	};
	void New(BDataIO *data) {
		uint32 d = 0;
		data->Write(&d, sizeof(d));
	}
	void String(BMallocIO *data, BString *string) {
		uint32 d;
		data->ReadAt(0, &d, sizeof(d));
		*string << (int32)d;
	}
	TResEditorWindow *OpenEditor(const char *title) {
		TEditorWindow *window = new TEditorWindow(BRect(260, 320, 500, 400), title, 'ULNG', this);
		return window;
	}
};


// TResAddon_LLNG

class TResAddon_LLNG : public TResAddon
{
public:
	TResAddon_LLNG() : TResAddon() {};
	~TResAddon_LLNG() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("Integer: 64Bit Signed");
		*type = 'LLNG';
	};
	void New(BDataIO *data) {
		int64 d = 0;
		data->Write(&d, sizeof(d));
	}
	void String(BMallocIO *data, BString *string) {
		int64 d;
		data->ReadAt(0, &d, sizeof(d));
		*string << d;
	}
	TResEditorWindow *OpenEditor(const char *title) {
		TEditorWindow *window = new TEditorWindow(BRect(260, 320, 500, 400), title, 'LLNG', this);
		return window;
	}
};


// TResAddon_ULLG

class TResAddon_ULLG : public TResAddon
{
public:
	TResAddon_ULLG() : TResAddon() {};
	~TResAddon_ULLG() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("Integer: 64Bit Unsigned");
		*type = 'ULLG';
	};
	void New(BDataIO *data) {
		uint64 d = 0;
		data->Write(&d, sizeof(d));
	}
	void String(BMallocIO *data, BString *string) {
		uint64 d;
		data->ReadAt(0, &d, sizeof(d));
		*string << d;
	}
	TResEditorWindow *OpenEditor(const char *title) {
		TEditorWindow *window = new TEditorWindow(BRect(260, 320, 500, 400), title, 'ULLG', this);
		return window;
	}
};


// TResAddon_BYTE

class TResAddon_BYTE : public TResAddon
{
public:
	TResAddon_BYTE() : TResAddon() {};
	~TResAddon_BYTE() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("Integer: 8Bit Signed");
		*type = 'ULLG';
	};
	void New(BDataIO *data) {
		int8 d = 0;
		data->Write(&d, sizeof(d));
	}
	void String(BMallocIO *data, BString *string) {
		int8 d;
		data->ReadAt(0, &d, sizeof(d));
		*string << d;
	}
	TResEditorWindow *OpenEditor(const char *title) {
		TEditorWindow *window = new TEditorWindow(BRect(260, 320, 500, 400), title, 'BYTE', this);
		return window;
	}
};


// TResAddon_UBYT

class TResAddon_UBYT : public TResAddon
{
public:
	TResAddon_UBYT() : TResAddon() {};
	~TResAddon_UBYT() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("Integer: 8Bit Unsigned");
		*type = 'UBYT';
	};
	void New(BDataIO *data) {
		uint8 d = 0;
		data->Write(&d, sizeof(d));
	}
	void String(BMallocIO *data, BString *string) {
		uint8 d;
		data->ReadAt(0, &d, sizeof(d));
		*string << (int32)d;
	}
	TResEditorWindow *OpenEditor(const char *title) {
		TEditorWindow *window = new TEditorWindow(BRect(260, 320, 500, 400), title, 'UBYT', this);
		return window;
	}
};


// TResAddon_MIMS

class TResAddon_MIMS : public TResAddon
{
public:
	TResAddon_MIMS() : TResAddon() {};
	~TResAddon_MIMS() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("MIME String");
		*type = 'MIMS';
	};
	void New(BDataIO *data) {
		
	}
	void String(BMallocIO *data, BString *string) {
		BString d;
		d.Append((const char *)data->Buffer(), min_c(data->BufferLength(), 64));
		*string = d;
	}
	TResEditorWindow *OpenEditor(const char *title) {
		TEditorWindow *window = new TEditorWindow(BRect(260, 320, 500, 400), title, 'MIMS', this);
		return window;
	}
};


// TResAddon_OFFT

class TResAddon_OFFT : public TResAddon
{
public:
	TResAddon_OFFT() : TResAddon() {};
	~TResAddon_OFFT() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("Offset");
		*type = 'OFFT';
	};
	void New(BDataIO *data) {
		int64 d = 0;
		data->Write(&d, sizeof(d));
	}
	void String(BMallocIO *data, BString *string) {
		int64 d;
		data->ReadAt(0, &d, sizeof(d));
		*string << d;
	}
	TResEditorWindow *OpenEditor(const char *title) {
		TEditorWindow *window = new TEditorWindow(BRect(260, 320, 500, 400), title, 'OFFT', this);
		return window;
	}
};


// TResAddon_PNTR

class TResAddon_PNTR : public TResAddon
{
public:
	TResAddon_PNTR() : TResAddon() {};
	~TResAddon_PNTR() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("Pointer");
		*type = 'PNTR';
	};
	void New(BDataIO *data) {
		void *d = 0;
		data->Write(&d, sizeof(d));
	}
	void String(BMallocIO *data, BString *string) {
		int d;
		char s[11];
		data->ReadAt(0, &d, sizeof(d));
		if (d == 0) {
			strcpy(s, "(nil)");
		} else {
			sprintf(s, "0x%X", d);
		}
		*string << s;
	}
	TResEditorWindow *OpenEditor(const char *title) {
		TEditorWindow *window = new TEditorWindow(BRect(260, 320, 500, 400), title, 'PNTR', this);
		return window;
	}
};


// TResAddon_RGBC

class TResAddon_RGBC : public TResAddon
{
public:
	TResAddon_RGBC() : TResAddon() {};
	~TResAddon_RGBC() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("RGB Color");
		*type = 'RGBC';
	};
	void New(BDataIO *data) {
		rgb_color d = {0,0,0,0};
		data->Write(&d, sizeof(d));
	}
	void String(BMallocIO *data, BString *string) {
		rgb_color d;
		data->ReadAt(0, &d, sizeof(d));
		*string << "R=" << (int32)d.red << ",G=" << (int32)d.green << ",B=" << (int32)d.blue << ",A=" << (int32)d.alpha;
	}
	TResEditorWindow *OpenEditor(const char *title) {
		return NULL;
	}
};


// TResAddon_RAWT

class TResAddon_RAWT : public TResAddon
{
public:
	TResAddon_RAWT() : TResAddon() {};
	~TResAddon_RAWT() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("Raw Data");
		*type = 'RAWT';
	};
	void New(BDataIO *data) {
		
	}
	TResEditorWindow *OpenEditor(const char *title) {
		return NULL;
	}
};


// TResAddon_SIZT

class TResAddon_SIZT : public TResAddon
{
public:
	TResAddon_SIZT() : TResAddon() {};
	~TResAddon_SIZT() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("Size");
		*type = 'SIZT';
	};
	void New(BDataIO *data) {
		size_t d = 0;
		data->Write(&d, sizeof(d));
	}
	void String(BMallocIO *data, BString *string) {
		size_t d;
		data->ReadAt(0, &d, sizeof(d));
		*string << d;
	}
	TResEditorWindow *OpenEditor(const char *title) {
		TEditorWindow *window = new TEditorWindow(BRect(260, 320, 500, 400), title, 'SIZT', this);
		return window;
	}
};


// TResAddon_SSZT

class TResAddon_SSZT : public TResAddon
{
public:
	TResAddon_SSZT() : TResAddon() {};
	~TResAddon_SSZT() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("Status and Size");
		*type = 'SSZT';
	};
	void New(BDataIO *data) {
		status_t d = 0;
		data->Write(&d, sizeof(d));
	}
	void String(BMallocIO *data, BString *string) {
		status_t d;
		data->ReadAt(0, &d, sizeof(d));
		*string << d;
	}
	TResEditorWindow *OpenEditor(const char *title) {
		TEditorWindow *window = new TEditorWindow(BRect(260, 320, 500, 400), title, 'SSZT', this);
		return window;
	}
};


// TResAddon_CSTR

class TResAddon_CSTR : public TResAddon
{
public:
	TResAddon_CSTR() : TResAddon() {};
	~TResAddon_CSTR() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("String");
		*type = 'CSTR';
	};
	void New(BDataIO *data) {
		
	}
	void String(BMallocIO *data, BString *string) {
		BString d;
		d.Append((const char *)data->Buffer(), min_c(data->BufferLength(), 64));
		*string = d;
	}
	TResEditorWindow *OpenEditor(const char *title) {
		TCstrEditorWindow *window = new TCstrEditorWindow(BRect(260, 320, 590, 500), title, 'CSTR', this);
		return window;
	}
};


// TResAddon_TIME

class TResAddon_TIME : public TResAddon
{
public:
	TResAddon_TIME() : TResAddon() {};
	~TResAddon_TIME() {};
	void GetType(BString *desc, type_code *type) {
		desc->SetTo("Time");
		*type = 'TIME';
	};
	void New(BDataIO *data) {
		time_t d = 0;
		data->Write(&d, sizeof(d));
	}
	void String(BMallocIO *data, BString *string) {
		time_t d;
		data->ReadAt(0, &d, sizeof(d));
		*string << d;
	}
	TResEditorWindow *OpenEditor(const char *title) {
		TEditorWindow *window = new TEditorWindow(BRect(260, 320, 500, 400), title, 'TIME', this);
		return window;
	}
};


// ----------------------------------------------------------------------------

// TResAddons

TResAddons::TResAddons()
{
	// Unknown type
	fUnknownResAddon = new TResAddon_unknown();
	
	// User type
	fAddonList.AddItem(new TResAddon_ARCV);
	fAddonList.AddItem(new TResAddon_APPF);
	fAddonList.AddItem(new TResAddon_BPNT);
	fAddonList.AddItem(new TResAddon_RECT);
	fAddonList.AddItem(new TResAddon_BBMP);
	fAddonList.AddItem(new TResAddon_BOOL);
	fAddonList.AddItem(new TResAddon_CHAR);
	fAddonList.AddItem(new TResAddon_CURS);
	fAddonList.AddItem(new TResAddon_FLOT);
	fAddonList.AddItem(new TResAddon_DBLE);
	fAddonList.AddItem(new TResAddon_ICON);
	fAddonList.AddItem(new TResAddon_MICN);
	fAddonList.AddItem(new TResAddon_SHRT);
	fAddonList.AddItem(new TResAddon_USHT);
	fAddonList.AddItem(new TResAddon_LONG);
	fAddonList.AddItem(new TResAddon_ULNG);
	fAddonList.AddItem(new TResAddon_LLNG);
	fAddonList.AddItem(new TResAddon_ULLG);
	fAddonList.AddItem(new TResAddon_BYTE);
	fAddonList.AddItem(new TResAddon_UBYT);
	fAddonList.AddItem(new TResAddon_MIMS);
	fAddonList.AddItem(new TResAddon_OFFT);
	fAddonList.AddItem(new TResAddon_PNTR);
	fAddonList.AddItem(new TResAddon_RGBC);
	fAddonList.AddItem(new TResAddon_RAWT);
	fAddonList.AddItem(new TResAddon_SIZT);
	fAddonList.AddItem(new TResAddon_SSZT);
	fAddonList.AddItem(new TResAddon_CSTR);
	fAddonList.AddItem(new TResAddon_TIME);
}

TResAddons::~TResAddons()
{
	for(int32 i=0; i < fAddonList.CountItems(); i++) {
		TResAddon *item = (TResAddon *)fAddonList.ItemAt(i);
		delete item;
	}
}

TResAddon_unknown *TResAddons::UnknownResAddon() const
{
	return fUnknownResAddon;
}

int32 TResAddons::CountAddons() const
{
	return fAddonList.CountItems();
}

TResAddon *TResAddons::AddonAt(int32 index) const
{
	return (TResAddon *)fAddonList.ItemAt(index);
}

TResAddon *TResAddons::FindResAddon(type_code type) const
{
	for(int32 i=0; i < this->CountAddons(); i++) {
		TResAddon *addon = this->AddonAt(i);
		BString desc;
		type_code typeCode;
		addon->GetType(&desc, &typeCode);
		if (type == typeCode) return addon;
	}
	return this->UnknownResAddon();
}
