#include <stdio.h>
#include <Alert.h>
#include <Application.h>
#include <stdlib.h>
#include <Mime.h>
#include <Path.h>
#include <TranslationUtils.h>
#include "Berry.h"
#include "ResManager.h"
#include "Locale.h"
#include "ToolTip.h"


#define _DECREASE(r,z) r.left += z; r.right -= z; r.top += z; r.bottom -= z;

BList res_manager_list;
TResManager *last_activated_manager = NULL;

enum {
	COL_MODIFIED = 0,
	COL_TYPE,
	COL_ID,
	COL_NAME,
	COL_DATA,
	COL_SIZE,
};

int htoi(const char *p);
type_code str_to_type_code(BString *typeStr);


void get_type_string(type_code type, BString *string)
{
	string->SetTo("");
	uchar *typ = (uchar *)&type;
	if ((typ[0] >= 0x20) && (typ[1] >= 0x20) && (typ[2] >= 0x20) && (typ[3] >= 0x20) &&
	    (typ[0] <= 0x7f) && (typ[1] <= 0x7f) && (typ[2] <= 0x7f) && (typ[3] <= 0x7f)) {
		*string << "'";
		int x = 1;
		if (*(char *)&x) {
			*string << (char)typ[3];
			*string << (char)typ[2];
			*string << (char)typ[1];
			*string << (char)typ[0];
		} else {
			*string << (char)typ[0];
			*string << (char)typ[1];
			*string << (char)typ[2];
			*string << (char)typ[3];
		}
	    *string << "'";
	} else {
		char str[9];
		sprintf(str, "%8.8X", (int)type);
		*string << "0x" << str;
	}
}

int htoi(const char *p)
{
	int i;
	for(i = 0; *p != 0; p++) {
		i *= 16;
		if (*p >= '0' && *p <= '9') i += *p - '0';
		else if (*p >= 'A' && *p <= 'F') i += *p - 'A' + 10;
		else if (*p >= 'a' && *p <= 'f') i += *p - 'a' + 10;
		else break;
	}
	return i;
}

type_code str_to_type_code(BString *typeStr)
{
	int32 length = typeStr->Length();
	if (length <= 0) return 0;
	
	type_code type = 0;
	const char *str = typeStr->String();
	switch(str[0]) {
		case '0':
			if (length < 3) {
				type = 0;
			} else {
				if (str[1] == 'x') {
					// 16進数
					type = htoi(&str[2]);
				} else {
					type = atoi(str);
				}
			}
			break;
		case 0x27:	// 0x27 = ' (single quotation)
			if (length == 6) {
				if (str[5] == 0x27) {
					((char *)&type)[0] = str[4];
					((char *)&type)[1] = str[3];
					((char *)&type)[2] = str[2];
					((char *)&type)[3] = str[1];
					break;
				}
			}
			break;
		default: type = atoi(str);
	}
	return type;
}


// TThumbView

TThumbView::TThumbView(BRect frame, const char *name, BBitmap *bitmap)
	:	BView(frame, name, B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW)
{
	fBitmap = bitmap;
}

TThumbView::~TThumbView()
{
	if (fBitmap) delete fBitmap;
}

void TThumbView::Draw(BRect updateRect)
{
	if (fBitmap) DrawBitmap(fBitmap, BRect(0,0,15,15));
}


// TTextInput

TTextInput::TTextInput(BRect frame, const char *name, const char *text, BMessage *message)
	:	BTextControl(frame, name, "", text, message)
{
	ResizeTo(frame.Width(), frame.Height());
	SetDivider(0);
	BTextView *textview = TextView();
	textview->MoveTo(1, 1);
	textview->ResizeBy(Bounds().right - textview->Frame().right - 1,
						Bounds().bottom - textview->Frame().bottom - 1);
}

TTextInput::~TTextInput()
{
}

void TTextInput::AttachedToWindow()
{
	SetViewColor(0, 0, 0);
}

void TTextInput::Draw(BRect updateRect)
{
}


// TResourceItem

TResourceItem::TResourceItem(ColumnListView *owner)
	:	CLVEasyItem(0, false, false, 20)
{
	fOwner = owner;
	fEditorWindow = NULL;
	fThumbView = NULL;
}

TResourceItem::~TResourceItem()
{
	if (fEditorWindow) {
		BMessenger(fEditorWindow).SendMessage(B_QUIT_REQUESTED);
	}
	DeleteThumbView();
}

bool TResourceItem::CreateThumbView()
{
	BBitmap *thumbBitmap = NULL;
	
	if (fThumbView) {
		fOwner->RemoveChild(fThumbView);
		delete fThumbView;
		fThumbView = NULL;
	}
	
	switch (fType) {
		case 'MICN':
			thumbBitmap = new BBitmap(BRect(0,0,15,15), B_CMAP8);
			fData.ReadAt(0, thumbBitmap->Bits(), thumbBitmap->BitsLength());
			break;
		case 'ICON':
			thumbBitmap = new BBitmap(BRect(0,0,31,31), B_CMAP8);
			fData.ReadAt(0, thumbBitmap->Bits(), thumbBitmap->BitsLength());
			break;
		case 'CURS': {
			thumbBitmap = new BBitmap(BRect(0,0,15,15), B_GRAY1);
			char *limit = (char *)thumbBitmap->Bits() + thumbBitmap->BitsLength();
			fData.Seek(4, SEEK_SET);
			for(char *bp = (char *)thumbBitmap->Bits(); bp < limit; bp += 4) {
				fData.Read(bp, 2);
			}
			break;
		}
		default:
			// Translator が存在する画像フォーマットか？
			thumbBitmap = BTranslationUtils::GetBitmap(&fData);
			if (thumbBitmap) {
				BString data;
				BString cs;
				switch (thumbBitmap->ColorSpace()) {
					case B_NO_COLOR_SPACE: cs = "NO_COLOR_SPACE"; break;
					case B_RGB32: cs = "RGB32"; break;
					case B_RGBA32: cs = "RGBA32"; break;
					case B_RGB24: cs = "RGB24"; break;
					case B_RGB16: cs = "RGB16"; break;
					case B_RGB15: cs = "RGB15"; break;
					case B_RGBA15: cs = "RGBA15"; break;
					case B_CMAP8: cs = "CMAP8"; break;
					case B_GRAY8: cs = "GRAY8"; break;
					case B_GRAY1: cs = "GRAY1"; break;
					case B_RGB32_BIG: cs = "RGB32_BIG"; break;
					case B_RGBA32_BIG: cs = "RGBA32_BIG"; break;
					case B_RGB24_BIG: cs = "RGB24_BIG"; break;
					case B_RGB16_BIG: cs = "RGB16_BIG"; break;
					case B_RGB15_BIG: cs = "RGB15_BIG"; break;
					case B_RGBA15_BIG: cs = "RGBA15_BIG"; break;
					case B_YCbCr422: cs = "YCbCr422"; break;
					case B_YCbCr411: cs = "YCbCr411"; break;
					case B_YCbCr444: cs = "YCbCr444"; break;
					case B_YCbCr420: cs = "YCbCr420"; break;
					case B_YUV422: cs = "YUV422"; break;
					case B_YUV411: cs = "YUV411"; break;
					case B_YUV444: cs = "YUV444"; break;
					case B_YUV420: cs = "YUV420"; break;
					case B_YUV9: cs = "YUV9"; break;
					case B_YUV12: cs = "YUV12"; break;
					case B_UVL24: cs = "UVL24"; break;
					case B_UVL32: cs = "UVL32"; break;
					case B_UVLA32: cs = "UVLA32"; break;
					case B_LAB24: cs = "LAB24"; break;
					case B_LAB32: cs = "LAB32"; break;
					case B_LABA32: cs = "LABA32"; break;
					case B_HSI24: cs = "HSI24"; break;
					case B_HSI32: cs = "HSI32"; break;
					case B_HSIA32: cs = "HSIA32"; break;
					case B_HSV24: cs = "HSV24"; break;
					case B_HSV32: cs = "HSV32"; break;
					case B_HSVA32: cs = "HSVA32"; break;
					case B_HLS24: cs = "HLS24"; break;
					case B_HLS32: cs = "HLS32"; break;
					case B_HLSA32: cs = "HLSA32"; break;
					case B_CMY24: cs = "CMY24"; break;
					case B_CMY32: cs = "CMY32"; break;
					case B_CMYA32: cs = "CMYA32"; break;
					case B_CMYK32: cs = "CMYK32"; break;
				}
				BRect bounds = thumbBitmap->Bounds();
				data << (int32)(bounds.right - bounds.left) + 1 << "x" << (int32)(bounds.bottom - bounds.top) + 1 << "x" << cs.String();
				SetColumnContent(COL_DATA, data.String());
			} else {
				delete thumbBitmap;
				return false;
			}
	}
	
	BRect r(ItemColumnFrame(COL_DATA, fOwner));
	r.left += 2;
	r.top += 2;
	r.right = r.left + 15;
	r.bottom = r.top + 15;
	fThumbView = new TThumbView(r, "ThumbView", thumbBitmap);
	fOwner->AddChild(fThumbView);
	
	// サムネイル表示域分だけテキストを右寄せ
	BString data("       ");
	data << GetColumnContentText(COL_DATA);
	SetColumnContent(COL_DATA, data.String());
	
	return true;
}

void TResourceItem::DeleteThumbView()
{
	if (fThumbView) {
		fOwner->RemoveChild(fThumbView);
		delete fThumbView;
		fThumbView = NULL;
	}
}

void TResourceItem::SetContent(type_code type, int32 id, const char *name, size_t size, const void *data)
{
	fResAddon = res_addons.FindResAddon(type);
	fType = type;
	fID = id;
	BString string;
	
	get_type_string(type, &string);
	SetColumnContent(COL_TYPE, string.String());
	
	string.SetTo("");
	string << id;
	SetColumnContent(COL_ID, string.String());
	
	SetColumnContent(COL_NAME, name);
	
	string.SetTo("");
	string << size;
	SetColumnContent(COL_SIZE, string.String());
	
	fData.SetSize(0);
	fData.WriteAt(0, data, size);
	
	string.SetTo("");
	fResAddon->String(&fData, &string);
	SetColumnContent(COL_DATA, string.String());
	
	if (fEditorWindow != NULL) {
		BMessenger(fEditorWindow).SendMessage(B_QUIT_REQUESTED);
		fEditorWindow = NULL;
	}
	
	CreateThumbView();
}

void TResourceItem::SetContent(type_code type, int32 id, const char *name, BDataIO *data)
{
	fResAddon = res_addons.FindResAddon(type);
	fType = type;
	fID = id;
	BString string;
	
	get_type_string(type, &string);
	SetColumnContent(COL_TYPE, string.String());
	
	string.SetTo("");
	string << id;
	SetColumnContent(COL_ID, string.String());
	
	SetColumnContent(COL_NAME, name);
	
	fData.SetSize(0);
	ssize_t size;
	do {
		char a[16384];
		size = data->Read(a, sizeof(a));
		fData.Write(a, size);
	} while (size > 0);
	fData.Seek(SEEK_SET, 0);
	
	string.SetTo("");
	string << fData.BufferLength();
	SetColumnContent(COL_SIZE, string.String());
	
	string.SetTo("");
	fResAddon->String(&fData, &string);
	SetColumnContent(COL_DATA, string.String());
	
	if (fEditorWindow != NULL) {
		BMessenger(fEditorWindow).SendMessage(B_QUIT_REQUESTED);
		fEditorWindow = NULL;
	}
	
	CreateThumbView();
}

void TResourceItem::SetContent(type_code type, int32 id, const char *name)
{
	type_code last_type = fType;
	BString string;
	get_type_string(type, &string);
	SetColumnContent(COL_TYPE, string.String());
	
	string.SetTo("");
	string << id;
	SetColumnContent(COL_ID, string.String());
	
	SetColumnContent(COL_NAME, name);
	
	fType = type;
	fID = id;
	fResAddon = res_addons.FindResAddon(type);
	
	string.SetTo("");
	fResAddon->String(&fData, &string);
	SetColumnContent(COL_DATA, string.String());
	
	if (fEditorWindow != NULL) {
		if (last_type != fType) {
			BMessenger(fEditorWindow).SendMessage(B_QUIT_REQUESTED);
			fEditorWindow = NULL;
		} else {
			fEditorWindow->SetTitle(name);
		}
	}
	
	CreateThumbView();
}

void TResourceItem::GetContent(type_code *type, int32 *id, const char **name, size_t *size, const void **data)
{
	if (type) *type = fType;
	if (id) *id = fID;
	if (name) *name = GetColumnContentText(COL_NAME);
	if (size) *size = fData.BufferLength();
	if (data) *data = fData.Buffer();
}

void TResourceItem::SetData(const void *data, ssize_t numBytes)
{
	fData.SetSize(0);
	fData.WriteAt(0, data, numBytes);
	
	BString string;
	string << fData.BufferLength();
	SetColumnContent(COL_SIZE, string.String());
	
	string.SetTo("");
	fResAddon->String(&fData, &string);
	SetColumnContent(COL_DATA, string.String());
	
	CreateThumbView();
}

void TResourceItem::OpenEditor()
{
	if (fEditorWindow == NULL) {
		fEditorWindow = fResAddon->OpenEditor(GetColumnContentText(COL_NAME));
		if (fEditorWindow == NULL) return;
		BMessenger msgr(fOwner->Window());
		entry_ref ref;
		((TResManager *)fOwner->Window())->Entry()->GetRef(&ref);
		fEditorWindow->SetParameter(&msgr, GetColumnContentText(COL_NAME), fType, fID, &ref);
		fEditorWindow->InstantiateFromData(fData.Buffer(), fData.BufferLength());
	}
	fEditorWindow->Show();
	fEditorWindow->Activate(true);
}

void TResourceItem::EditorWindowClosed()
{
	fEditorWindow = NULL;
}


// TResourceView

TResourceView::TResourceView(BRect frame, const char *name)
	:	ColumnListView(frame, &iContainerView, name,
						B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP_BOTTOM,
						B_WILL_DRAW | B_FRAME_EVENTS | B_NAVIGABLE,
						B_SINGLE_SELECTION_LIST,	// view type
						true,						// hierarchical
						true,						// horizontal scrollbars
						true,						// vertical scrollbars
						true,						// scroll-view corner
						B_FANCY_BORDER)				// border style
{
	this->SetListType(B_SINGLE_SELECTION_LIST);
	this->AddColumn(new CLVColumn("", 10, CLV_HEADER_TRUNCATE | CLV_TELL_ITEMS_WIDTH | CLV_LOCK_AT_BEGINNING | CLV_NOT_MOVABLE));
	this->AddColumn(new CLVColumn("Type", 70, CLV_HEADER_TRUNCATE | CLV_TELL_ITEMS_WIDTH | CLV_NOT_MOVABLE, 50));
	this->AddColumn(new CLVColumn("ID", 60, CLV_HEADER_TRUNCATE | CLV_TELL_ITEMS_WIDTH | CLV_NOT_MOVABLE, 50));
	this->AddColumn(new CLVColumn("Name", 120, CLV_HEADER_TRUNCATE | CLV_TELL_ITEMS_WIDTH | CLV_NOT_MOVABLE, 50));
	this->AddColumn(new CLVColumn("Data", 140, CLV_HEADER_TRUNCATE | CLV_TELL_ITEMS_WIDTH | CLV_NOT_MOVABLE, 50));
	this->AddColumn(new CLVColumn("Size", 70, CLV_HEADER_TRUNCATE | CLV_TELL_ITEMS_WIDTH | CLV_NOT_MOVABLE, 50));
	SetInvocationMessage(new BMessage(RM_OPEN));
	fLastSelectedItem = NULL;
	OpenButton = NULL;
	DeleteButton = NULL;
	MakeEmpty();
}

TResourceView::~TResourceView()
{
	MakeEmpty();
}

void TResourceView::MakeEmpty()
{
	for(int32 i = this->CountItems() - 1; i >= 0; i--) {
		TResourceItem *item = (TResourceItem *)this->ItemAt(i);
		delete item;
		RemoveItem(i);
	}
}

status_t TResourceView::Invoke(BMessage *msg)
{
	if (msg != NULL) OpenEditor();
	return ColumnListView::Invoke(msg);
}

void TResourceView::KeyDown(const char *bytes, int32 numBytes)
{
	if (numBytes > 0) {
		if (bytes[0] == 0x0a) OpenEditor();
	}
	ColumnListView::KeyDown(bytes, numBytes);
}

void TResourceView::OpenEditor()
{
	int32 i = CurrentSelection();
	if (i < 0) return;
	
	TResourceItem *item = (TResourceItem *)ItemAt(i);
	item->OpenEditor();
}

void TResourceView::Sort()
{
}

int32 TResourceView::FindTypeID(type_code type, int32 id) const
{
	for(int32 i=0; i < CountItems(); i++) {
		TResourceItem *item = (TResourceItem *)ItemAt(i);
		type_code a_type = item->TypeCode();
		int32 a_id = item->ID();
		if ((a_type == type) && (a_id == id)) return i;
	}
	return -1;
}

void TResourceView::MessageReceived(BMessage *msg)
{
	switch(msg->what) {
		case RM_NAME_CHANGED: {	// Name の変更
			if (fLastSelectedItem == NULL) break;
			BString typeStr(fTypeEditor->Text());
			type_code type = str_to_type_code(&typeStr);
			int32 id = atoi(fIDEditor->Text());
			const char *name = fNameEditor->Text();
			fLastSelectedItem->SetContent(type, id, name);
			fNameEditor->SetText(fLastSelectedItem->GetColumnContentText(COL_NAME));
			fLastSelectedItem->SetColumnContent(COL_MODIFIED, " *");
			((TResManager *)Window())->SetDirty();
			fNameEditor->TextView()->SelectAll();
			Invalidate();
			break;
		}
		case RM_TYPE_CHANGED:	// Type の変更
		case RM_ID_CHANGED: {	// ID の変更
			if (fLastSelectedItem == NULL) break;
			BString typeStr(fTypeEditor->Text());
			type_code type = str_to_type_code(&typeStr);
			int32 id = atoi(fIDEditor->Text());
			const char *name = fNameEditor->Text();
			if (FindTypeID(type, id) >= 0) {
				// 同一 Type で、重複 ID があれば警告表示して変更しない
				(new BAlert("",
					GET_STRING("IDmustBeUnique", "ID must be unique within a data type."),
					GET_STRING("OK", "OK")))->Go();
				fTypeEditor->SetText(fLastSelectedItem->GetColumnContentText(COL_TYPE));
				fIDEditor->SetText(fLastSelectedItem->GetColumnContentText(COL_ID));
			} else {
				// 変更を反映
				fLastSelectedItem->SetContent(type, id, name);
				fTypeEditor->SetText(fLastSelectedItem->GetColumnContentText(COL_TYPE));
				fIDEditor->SetText(fLastSelectedItem->GetColumnContentText(COL_ID));
				fTypeEditor->TextView()->SelectAll();
				fIDEditor->TextView()->SelectAll();
				fLastSelectedItem->SetColumnContent(COL_MODIFIED, " *");
				((TResManager *)Window())->SetDirty();
			}
			Invalidate();
			break;
		}
		case RM_OPEN: {
			Invoke(msg);
			break;
		}
		case RM_DELETE: {
			int32 selected = CurrentSelection();
			if (selected < 0) break;
			TResourceItem *selectedItem = (TResourceItem *)this->ItemAt(selected);
			DeleteItem(selectedItem);
			break;
		}
		case B_COPY_TARGET: {
			entry_ref ref;
			const char *fileName;
			const void *data;
			size_t size;
			ssize_t ssize;
			int32 selected;
			BMessage _previous_;
			if (msg->FindRef("directory", &ref) != B_OK) return;
			if (msg->FindString("name", &fileName) != B_OK) return;
			if (msg->FindMessage("_previous_", &_previous_) != B_OK) {
				const void *p;
				if (msg->FindData("_previous_", B_RAW_TYPE, &p, &ssize) != B_OK) return;
				_previous_.Unflatten((const char *)p);
			}
//			if (_previous_.FindData("GuiBerry:raw_data", B_RAW_TYPE, &data, &ssize) != B_OK) return;
			if (_previous_.FindInt32("GuiBerry:index", &selected) != B_OK) return;
			
			TResourceItem *selectedItem = (TResourceItem *)this->ItemAt(selected);
			if (!selectedItem) return;
			selectedItem->GetContent(NULL, NULL, NULL, &size, &data);
			
			BDirectory dir(&ref);
			BFile clip(&dir, fileName, B_WRITE_ONLY);
			clip.Write(data, size);
			break;
		}
		default: ColumnListView::MessageReceived(msg);
	}
}

void TResourceView::DeleteItem(TResourceItem *item)
{
	RemoveItem(item);
	delete item;
	((TResManager *)Window())->SetDirty();
}

void TResourceView::fRemoveEditors()
{
	RemoveChild(fTypeEditor);
	RemoveChild(fIDEditor);
	RemoveChild(fNameEditor);
	delete fTypeEditor;
	delete fIDEditor;
	delete fNameEditor;
}

void TResourceView::MouseDown(BPoint point)
{
	ColumnListView::MouseDown(point);
	
	int32 selected = this->CurrentSelection();
	if (selected < 0) return;
	TResourceItem *selectedItem = (TResourceItem *)this->ItemAt(selected);
	if (!selectedItem) return;
	
	const char *name;
	const void *data;
	size_t size;
	selectedItem->GetContent(NULL, NULL, &name, &size, &data);
	BString clipName(name);
	clipName << ".raw";
	
	BMessage msg(B_SIMPLE_DATA);
	msg.AddString("be:types", B_FILE_MIME_TYPE);
	msg.AddInt32("be:actions", B_COPY_TARGET);
	msg.AddString("be:clip_name", clipName.String());
//	msg.AddData("GuiBerry:raw_data", B_RAW_TYPE, data, size);
	msg.AddInt32("GuiBerry:index", selected);
	
	DragMessage(&msg, ItemFrame(selected), this);
}

void TResourceView::SelectionChanged()
{
	int32 selected = this->CurrentSelection();
	if (selected < 0) {
		if (this->fLastSelectedItem!=NULL) fRemoveEditors();
		this->fLastSelectedItem = NULL;
		if (OpenButton != NULL) OpenButton->SetEnabled(false);
		if (DeleteButton != NULL) DeleteButton->SetEnabled(false);
		return;	
	}
	if (this->fLastSelectedItem != NULL) fRemoveEditors();
	TResourceItem *selectedItem = (TResourceItem *)this->ItemAt(selected);
	
	BRect typeRect = selectedItem->ItemColumnFrame(COL_TYPE, this);
	BRect IDRect = selectedItem->ItemColumnFrame(COL_ID, this);
	BRect nameRect = selectedItem->ItemColumnFrame(COL_NAME, this);
	
	_DECREASE(typeRect, 2);
	_DECREASE(IDRect, 2);
	_DECREASE(nameRect, 2);
	
	type_code type;
	int32 id;
	const char *name;
	const void *data;
	size_t size;
	BString typeStr;
	selectedItem->GetContent(&type, &id, &name, &size, &data);
	get_type_string(type, &typeStr);
	
	fTypeEditor = new TTextInput(typeRect, "TypeEditor", typeStr.String(), new BMessage(RM_TYPE_CHANGED));
	AddChild(fTypeEditor);
	fTypeEditor->SetTarget(this);
	
	fIDEditor = new TTextInput(IDRect, "IDEditor", selectedItem->GetColumnContentText(COL_ID), new BMessage(RM_ID_CHANGED));
	AddChild(fIDEditor);
	fIDEditor->SetTarget(this);
	
	fNameEditor = new TTextInput(nameRect, "NameEditor", name, new BMessage(RM_NAME_CHANGED));
	AddChild(fNameEditor);
	fNameEditor->SetTarget(this);
	
	// マウスの現在位置から MakeFocus() する Edit を選ぶ
	#define _MKFOCUS(e) \
		rect = e->Bounds();\
		rect.InsetBy(-2, -2);\
		e->GetMouse(&loc, &button);\
		if (rect.Contains(loc)) e->MakeFocus();
	BPoint loc;
	uint32 button;
	BRect rect;
	_MKFOCUS(fTypeEditor);
	_MKFOCUS(fIDEditor);
	_MKFOCUS(fNameEditor);
	
	this->fLastSelectedItem = selectedItem;
	if (OpenButton != NULL) OpenButton->SetEnabled(true);
	if (DeleteButton != NULL) DeleteButton->SetEnabled(true);
}

// カラムヘッダのリサイズイベント
void TResourceView::ColumnWidthChanged(int32 columnIndex, float newWidth)
{
	// 各エディタ位置の追従
	int32 selected = this->CurrentSelection();
	if (selected >= 0) {
		TResourceItem *selectedItem = (TResourceItem *)this->ItemAt(selected);
		if (selectedItem) {
			BRect typeRect = selectedItem->ItemColumnFrame(COL_TYPE, this);
			BRect IDRect = selectedItem->ItemColumnFrame(COL_ID, this);
			BRect nameRect = selectedItem->ItemColumnFrame(COL_NAME, this);
			_DECREASE(typeRect, 2);
			_DECREASE(IDRect, 2);
			_DECREASE(nameRect, 2);
			if (fTypeEditor) {
				fTypeEditor->MoveTo(BPoint(typeRect.left, typeRect.top));
				fTypeEditor->ResizeTo(typeRect.Width(), typeRect.Height());
			}
			if (fIDEditor) {
				fIDEditor->MoveTo(BPoint(IDRect.left, IDRect.top));
				fIDEditor->ResizeTo(IDRect.Width(), IDRect.Height());
			}
			if (fNameEditor) {
				fNameEditor->MoveTo(BPoint(nameRect.left, nameRect.top));
				fNameEditor->ResizeTo(nameRect.Width(), nameRect.Height());
			}
		}
	}
	
	// サムネイルビュー位置の追従
	for(int32 i = 0; i < CountItems(); i++) {
		TResourceItem *selectedItem = (TResourceItem *)ItemAt(i);
		TThumbView *thumbView = selectedItem->ThumbView();
		if (thumbView) {
			BRect dataRect = selectedItem->ItemColumnFrame(COL_DATA, this);
			thumbView->MoveTo(BPoint(dataRect.left + 2, dataRect.top + 2));
		}
	}
}


// TResManager

TResManager::TResManager(BRect frame, const char *title)
	:	BWindow(frame, title, B_DOCUMENT_WINDOW, 0)
{
	BRect rect(Bounds());
	
	BView *bgview = new BView(rect, "bgview", B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FRAME_EVENTS);
	bgview->SetViewColor(216, 216, 216);
	AddChild(bgview);
	
	fNewMenu = new BMenu("New");
	BuildMenuItems(fNewMenu);
	
	fNewMenuField = new BMenuField(BRect(5, 5, 80, 30), "New", "", fNewMenu);
	fNewMenuField->SetDivider(0);
	bgview->AddChild(fNewMenuField);
	
	rect.top += 32;
	rect.right -= B_V_SCROLL_BAR_WIDTH;
	rect.bottom -= B_H_SCROLL_BAR_HEIGHT;
	fResourceView = new TResourceView(rect, "ResourceView");
	bgview->AddChild(fResourceView->View());
	fResourceView->MakeFocus();
	
	fDirty = false;
	res_manager_list.AddItem(this);
	
	fSavePanel = new BFilePanel(B_SAVE_PANEL, new BMessenger(this), NULL, 0, false);
	fSavePanel->SetSaveText(Title());
	
	rect = bgview->Bounds();
	rect.top += 2;
	rect.bottom = rect.top + 10;
	rect.left = rect.right - 170;
	rect.right -= 90;
	fResourceView->OpenButton = new BButton(rect, "OpenButton", "Open", new BMessage(RM_OPEN),
								B_FOLLOW_RIGHT | B_FOLLOW_TOP);
	fResourceView->OpenButton->SetEnabled(false);
	bgview->AddChild(fResourceView->OpenButton);
	fResourceView->OpenButton->SetTarget(fResourceView);
	
	rect.left += 85;
	rect.right += 85;
	fResourceView->DeleteButton = new BButton(rect, "DeleteButton", "Delete", new BMessage(RM_DELETE),
								B_FOLLOW_RIGHT | B_FOLLOW_TOP);
	fResourceView->DeleteButton->SetEnabled(false);
	bgview->AddChild(fResourceView->DeleteButton);
	fResourceView->DeleteButton->SetTarget(fResourceView);
	
	fSaveButton = new TSimplePictureButton(BRect(72, 3, 95, 26), "Save",
		"SAVE_UP", 'BMP ', "SAVE_DOWN", 'BMP ', new BMessage(GB_SAVE));
	bgview->AddChild(fSaveButton);
	
	AddShortcut('S', 0, new BMessage(GB_SAVE));
	
	tool_tip.SetToolTip(fSaveButton, "Save");
	
	LanguageChanged();
}

TResManager::~TResManager()
{
	if (last_activated_manager == this) last_activated_manager = NULL;
	res_manager_list.RemoveItem(this);
	delete fSavePanel;
}

void TResManager::WindowActivated(bool active)
{
	last_activated_manager = this;
}

TResourceItem *TResManager::FindItem(BMessage *msg) const
{
	void *pointer;
	if (msg->FindPointer("editor_window", &pointer) != B_OK) return NULL;
	TResEditorWindow *window = reinterpret_cast<TResEditorWindow *>(pointer);
	for(int32 i=0; i < fResourceView->CountItems(); i++) {
		TResourceItem *item = (TResourceItem *)fResourceView->ItemAt(i);
		if (item->EditorWindow() == window) {
			return item;
		}
	}
	return NULL;
}

void TResManager::MessageReceived(BMessage *msg)
{
	static bool quit_app_on_save = false;
	
	switch(msg->what) {
		case RM_QUIT: {
			TResourceItem *item = FindItem(msg);
			if (item != NULL) item->EditorWindowClosed();
			break;
		}
		case RM_DATA: {
			TResourceItem *item = FindItem(msg);
			if (item == NULL) break;
			const void *data;
			ssize_t numBytes;
			if (msg->FindData("data", B_ANY_TYPE, &data, &numBytes) != B_OK) break;
			item->SetData(data, numBytes);
			item->SetColumnContent(COL_MODIFIED, " *");
			fResourceView->Invalidate();
			fDirty = true;
			break;
		}
		case RM_MODIFIED: {
			TResourceItem *item = FindItem(msg);
			if (item != NULL) {
				item->SetColumnContent(COL_MODIFIED, " *");
				fResourceView->Invalidate();
				fDirty = true;
			}
			break;
		}
		case GB_SAVE: {
			msg->FindBool("quit_app_on_save", &quit_app_on_save);
			const BEntry *entry = this->Entry();
			if (entry->InitCheck() == B_OK) {
				if (this->SaveAs(entry) == B_OK) {
					if (quit_app_on_save) {
						BMessenger(be_app).SendMessage(B_QUIT_REQUESTED);
					}
				}
			} else {
				fSavePanel->SetSaveText(Title());
				fSavePanel->Show();
			}
			break;
		}
		case GB_SAVEAS:
			fSavePanel->SetSaveText(Title());
			fSavePanel->Show();
			break;
		case B_SIMPLE_DATA: {
			for(int32 i=0; ; i++) {
				entry_ref ref;
				if (msg->FindRef("refs", i, &ref) != B_OK) return;
				BEntry entry(&ref);
				if (entry.InitCheck() != B_OK) continue;
				BFile file(&entry, B_READ_ONLY);
				if (file.InitCheck() != B_OK) continue;
				BResources rsrc;
				if (rsrc.SetTo(&file) == B_OK) {
					BMessage message(B_REFS_RECEIVED);
					message.AddRef("refs", &ref);
					be_app_messenger.SendMessage(&message);
				} else {
					Import(&entry);
				}
			}
		}
		case B_REFS_RECEIVED:
			be_app_messenger.SendMessage(msg);
			break;
		case B_SAVE_REQUESTED: {
			entry_ref ref;
			BString name;
			if ((msg->FindRef("directory", &ref)==B_OK) &&
			    (msg->FindString("name", &name)==B_OK)) {
			    BDirectory dir(&ref);
			    BEntry entry(&dir, name.String());
				if (this->SaveAs(&entry)==B_OK) {
					if (quit_app_on_save) {
						BMessenger(be_app).SendMessage(B_QUIT_REQUESTED);
					}
				}
			}
			break;
		}
		case RM_NEW_RESOURCE: {
			type_code type;
			if (msg->FindInt32("type_code", (int32 *)&type) != B_OK) return;
			TResAddon *addon = res_addons.FindResAddon(type);
			if (addon == NULL) return;
			BMallocIO data;
			addon->New(&data);
			TResourceItem *item = new TResourceItem(fResourceView);
			fResourceView->AddItem(item);
			int32 id = GetNewID(type);
			BString newTitle("Untitled ");
			newTitle << id;
			item->SetContent(type, id, newTitle.String(), data.BufferLength(), data.Buffer());
			item->SetColumnContent(COL_MODIFIED, " *");
			fResourceView->Invalidate();
			fDirty = true;
			break;
		}
#ifdef B_LANGUAGE_CHANGED
		case B_LANGUAGE_CHANGED: {
			LanguageChanged();
			break;
		}
#endif
		default: BWindow::MessageReceived(msg);
	}
}

void TResManager::LanguageChanged()
{
	BMenuItem *item = fNewMenuField->MenuItem();
	if (item != NULL) item->SetLabel(GET_STRING("R_New", "New"));
	fResourceView->OpenButton->SetLabel(GET_STRING("R_Open", "Open"));
	fResourceView->DeleteButton->SetLabel(GET_STRING("R_Delete", "Delete"));
	tool_tip.SetToolTip(fSaveButton, GET_STRING("T_Save", "Save"));
}

void TResManager::MakeEmpty()
{
	fResourceView->MakeEmpty();
}

status_t TResManager::OpenResource(const BEntry *entry)
{
	MakeEmpty();
	
	fEntry = *entry;
	BFile file(&fEntry, B_READ_ONLY);
	if (file.InitCheck() != B_OK) {
		fprintf(stderr, "OpenResource InitCheck() error\n");
		return B_ERROR;
	}
	
	BResources rsrc;
	if (rsrc.SetTo(&file) == B_OK) {
		int32 index = 0;
		type_code type;
		int32 id;
		const char *name;
		size_t size;
		const void *data;
		while (rsrc.GetResourceInfo(index, &type, &id, &name, &size)) {
			data = rsrc.LoadResource(type, id, &size);
			TResourceItem *item = new TResourceItem(fResourceView);
			fResourceView->AddItem(item);
			item->SetContent(type, id, name, size, data);
			index++;
		}
		fResourceView->Sort();
		fDirty = false;
		return B_OK;
	} else {
		//(new BAlert("", "Open error.", "OK"))->Go();
		printf("GuiBerry: This is not resource file.\n");
		return B_ERROR;
	}
}

status_t TResManager::Import(const char *name, type_code type, BDataIO *sourceData)
{
	TResourceItem *item = new TResourceItem(fResourceView);
	fResourceView->AddItem(item);
	int32 id = GetNewID(type);
	item->SetContent(type, id, name, sourceData);
	item->SetColumnContent(COL_MODIFIED, " *");
	fResourceView->Invalidate();
	fDirty = true;
	return B_OK;
}

status_t TResManager::Import(const BEntry *entry)
{
	BFile file(entry, B_READ_ONLY);
	if (file.InitCheck() != B_OK) {
		fprintf(stderr, "Import InitCheck() error\n");
		return B_ERROR;
	}
	char name[B_FILE_NAME_LENGTH];
	entry->GetName(name);
	return Import(name, 'RAWT', &file);
}

status_t TResManager::Save()
{
//	uint32 openMode = fEntry.Exists() ? B_READ_WRITE : B_READ_WRITE | B_CREATE_FILE;
	BFile file(&fEntry, B_READ_WRITE | B_CREATE_FILE | B_ERASE_FILE);
	if (file.InitCheck() != B_OK) {
		(new BAlert("", "InitCheck() ERROR!", "OK"))->Go();
		return B_ERROR;
	}
	
	const char mims[] = "application/x-be-resource";
	file.WriteAttr("BEOS:TYPE", 'MIMS', 0, mims, sizeof(mims));
	file.Sync();
	
	BResources rsrc(&file);
	
	for(int32 i=0; i < fResourceView->CountItems(); i++) {
		TResourceItem *item = (TResourceItem *)fResourceView->ItemAt(i);
		type_code type;
		int32 id;
		const char *name;
		size_t size;
		const void *data;
		item->GetContent(&type, &id, &name, &size, &data);
		if (rsrc.AddResource(type, id, data, size, name) != B_OK) {
			(new BAlert("", "AddResource() ERROR!", "OK"))->Go();
			return B_ERROR;
		}
		item->SetColumnContent(COL_MODIFIED, "");
	}
	rsrc.Sync();
	fResourceView->Invalidate();
	
	BPath path;
	if (fEntry.GetPath(&path) == B_OK) {
		fResourceView->Window()->SetTitle(path.Leaf());
	} else {
		fResourceView->Window()->SetTitle("Unknown?");
	}
	
	fDirty = false;
	return B_OK;
}

status_t TResManager::SaveAs(const BEntry *entry)
{
	fEntry = *entry;
	return Save();
}

bool TResManager::AppQuitRequested(bool quit_app_on_save)
{
	if (fDirty) {
		BString alertText(GET_STRING("DocumentChanged", "The document named \"%s\" has been changed."));
		alertText.ReplaceFirst("%s", Title());
		alertText << "\n\n" << GET_STRING("DoYouWantToSaveIt", "Do you want to save it?");
		BAlert *alert = new BAlert("GuiBerry", alertText.String(),
							GET_STRING("Cancel", "Cancel"),
							GET_STRING("No", "No"),
							GET_STRING("Yes", "Yes"));
		switch(alert->Go()) {
			case 0:
				return false;
			case 2:
				if (fEntry.InitCheck() == B_OK) {
					return Save() == B_OK;
				} else {
					BMessage msg(GB_SAVE);
					msg.AddBool("quit_app_on_save", quit_app_on_save);
					BMessenger(this).SendMessage(&msg);
					return false;
				}
		}
	}
	return true;
}

bool TResManager::QuitRequested()
{
	return AppQuitRequested(false);
}

const BEntry *TResManager::Entry() const
{
	return &fEntry;
}

void TResManager::BuildMenuItems(BMenu *menu)
{
	for(int32 i=0; i < res_addons.CountAddons(); i++) {
		TResAddon *addon = res_addons.AddonAt(i);
		BString desc, typestr;
		type_code type;
		addon->GetType(&desc, &type);
		BMessage *msg = new BMessage(RM_NEW_RESOURCE);
		msg->AddInt32("type_code", type);
		get_type_string(type, &typestr);
		desc << " (" << typestr << ")";
		BMenuItem *item = new BMenuItem(desc.String(), msg);
		item->SetTarget(this);
		menu->AddItem(item);
	}
}

int32 TResManager::GetNewID(type_code type) const
{
	int32 new_id = -1;
	for(int32 i=0; i < fResourceView->CountItems(); i++) {
		TResourceItem *item = (TResourceItem *)fResourceView->ItemAt(i);
		type_code typeCode;
		int32 id;
		const char *name;
		size_t size;
		const void *data;
		item->GetContent(&typeCode, &id, &name, &size, &data);
		if (typeCode == type) new_id = max_c(new_id, id);
	}
	return new_id + 1;
}

void TResManager::SetDirty()
{
	fDirty = true;
}

status_t TResManager::OpenEditor(type_code type, int32 id)
{
	for(int32 i=0; i < fResourceView->CountItems(); i++) {
		TResourceItem *item = (TResourceItem *)fResourceView->ItemAt(i);
		if ((item->TypeCode() == type) && (item->ID() == id)) {
			item->OpenEditor();
			return B_OK;
		}
	}
	return B_ERROR;
}

status_t TResManager::GetData(type_code type, int32 id, const void **data, size_t *size)
{
	for(int32 i=0; i < fResourceView->CountItems(); i++) {
		TResourceItem *item = (TResourceItem *)fResourceView->ItemAt(i);
		if ((item->TypeCode() == type) && (item->ID() == id)) {
			item->GetContent(NULL, NULL, NULL, size, data);
			
			return B_OK;
		}
	}
	return B_ERROR;
}
