#ifndef _RESMANAGER_H
#define _RESMANAGER_H

#include <Window.h>
#include <Entry.h>
#include <Message.h>
#include <Messenger.h>
#include <Resources.h>
#include <File.h>
#include <String.h>
#include <FilePanel.h>
#include <MenuField.h>
#include <MenuItem.h>
#include <Button.h>
#include "ColumnListView.h"
#include "CLVEasyItem.h"
#include "ResAddon.h"
#include "SimplePictureButton.h"

#include <TextControl.h>


enum {
	RM_MODIFIED = 'rmmd',
	RM_NEW_RESOURCE = 'rmnr',
	RM_NAME_CHANGED = 'rmnc',
	RM_ID_CHANGED = 'rmic',
	RM_TYPE_CHANGED = 'rmtc',
	RM_OPEN = 'rmop',
	RM_DELETE = 'rmde',
	RM_DATA = 'rmda',
	RM_QUIT = 'rmqu',
};

void get_type_string(type_code type, BString *string);


// TThumbView
class TThumbView : public BView
{
public:
	TThumbView(BRect frame, const char *name, BBitmap *bitmap);
	~TThumbView();
	void Draw(BRect updateRect);
private:
	BBitmap *fBitmap;
};

// TTextInput

class TTextInput : public BTextControl
{
public:
	TTextInput(BRect frame, const char *name, const char *text, BMessage *message);
	~TTextInput();
	void AttachedToWindow();
	void Draw(BRect updateRect);
};


// TResourceItem

class TResourceItem : public CLVEasyItem
{
public:
	TResourceItem(ColumnListView *owner);
	~TResourceItem();
	void SetContent(type_code type, int32 id, const char *name, size_t size, const void *data);
	void SetContent(type_code type, int32 id, const char *name, BDataIO *data);
	void SetContent(type_code type, int32 id, const char *name);
	void GetContent(type_code *type, int32 *id, const char **name, size_t *size, const void **data);
	void SetData(const void *data, ssize_t numBytes);
	TResEditorWindow *EditorWindow() const { return fEditorWindow; };
	void EditorWindowClosed();
	void OpenEditor();
	TResAddon *ResAddon() const { return fResAddon; };
	type_code TypeCode() const { return fType; };
	int32 ID() const { return fID; };
	TThumbView *ThumbView() const { return fThumbView; };
private:
	ColumnListView *fOwner;
	BMallocIO fData;
	type_code fType;
	int32 fID;
	TResEditorWindow *fEditorWindow;
	TResAddon *fResAddon;
	TThumbView *fThumbView;
	bool CreateThumbView();
	void DeleteThumbView();
};


// TResourceView

class TResourceView : public ColumnListView
{
public:
	TResourceView(BRect frame, const char *name);
	~TResourceView();
	status_t Invoke(BMessage *msg = NULL);
	void KeyDown(const char *bytes, int32 numBytes);
	void MouseDown(BPoint point);
	void MakeEmpty();
	void MessageReceived(BMessage *msg);
	void DeleteItem(TResourceItem *item);
	CLVContainerView *View() { return iContainerView; };
	void OpenEditor();
	void Sort();
	void SelectionChanged();
	void ColumnWidthChanged(int32 columnIndex, float newWidth);
	int32 FindTypeID(type_code type, int32 id) const;
	BButton *OpenButton;
	BButton *DeleteButton;
private:
	CLVContainerView *iContainerView;
	TResourceItem *fLastSelectedItem;
	TTextInput *fTypeEditor;
	TTextInput *fIDEditor;
	TTextInput *fNameEditor;
	void fRemoveEditors();
};


// TResManager

class TResManager : public BWindow
{
public:
	TResManager(BRect frame, const char *title);
	~TResManager();
	void WindowActivated(bool active);
	bool AppQuitRequested(bool quit_app_on_save);
	bool QuitRequested();
	void MessageReceived(BMessage *msg);
	void MakeEmpty();
	status_t OpenResource(const BEntry *entry);
	status_t Import(const char *name, type_code type, BDataIO *sourceData);
	status_t Import(const BEntry *entry);
	status_t Save();
	status_t SaveAs(const BEntry *entry);
	const BEntry *Entry() const;
	int32 GetNewID(type_code type) const;
	void SetDirty();
	void LanguageChanged();
	status_t OpenEditor(type_code type, int32 id);
	status_t GetData(type_code type, int32 id, const void **data, size_t *size);
private:
	BMenu *fNewMenu;
	BMenuField *fNewMenuField;
	bool fDirty;
	BEntry fEntry;
	TResourceView *fResourceView;
	BFilePanel *fSavePanel;
	void BuildMenuItems(BMenu *menu);
	TResourceItem *FindItem(BMessage *msg) const;
	TSimplePictureButton *fSaveButton;
};


extern BList res_manager_list;
extern TResManager *last_activated_manager;


#endif
